/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

 /*
  * Programming info contained in this module for AD9389
  * can be found in
  * http://www.analog.com/static/imported-files/data_sheets/AD9393.pdf
 */
#include <linux/kernel.h>
#include <linux/module.h>

#include <linux/i2c.h>
#include "via_chrome9_external_card.h"
#include "via_chrome9_display_common.h"

extern struct via_cea_timing_info_rec via_cea_timing_table[];
extern struct via_dot_clock via_dot_clocks[];

struct ad9389_priv {
	int status;
	u8 std_edid[128];
	u8 ext_edid[128];
};

void via_chrome9_set_ad9389_con_edid(void *options, int size,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_chrome9_external_card *card =
		via_chrome9_connector_get_private(via_chrome9_connector);
	struct ad9389_priv *ad9389 = card->private_data;
	KMS_DEBUG("\n");
	copy_from_user(ad9389->std_edid, options, size);
	KMS_DEBUG("%d,%d,", ad9389->std_edid[1], ad9389->std_edid[0x7e]);
}

static int ad9389_init(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);
	struct drm_encoder *encoder = card->encoder;
	struct drm_device *dev = card->drm_dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct ad9389_priv *ad9389;
	struct via_chrome9_connector *via_con;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");
	p_priv->is_ad9389_hpd_irq = false;

	ad9389 = kzalloc(sizeof(struct ad9389_priv), GFP_KERNEL);
	if (ad9389 == NULL) {
		printk(KERN_ERR "no enough mem\n");
		return -ENOMEM;
	}
	card->private_data = ad9389;

	if (VX900_DEVICE == dev->pci_device)
		encoder->possible_crtcs = 0x3;
	else
		encoder->possible_crtcs = 0x2;

	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_HDMIA);
	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_DVID);
	list_for_each_safe(p, n, &card->list_connectors) {
		via_con = list_entry(p, struct via_chrome9_connector, list);
		via_con->set_edid = via_chrome9_set_ad9389_con_edid;
	}
	return 0;
}

static void ad9389_deinit(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);
	struct via_chrome9_connector *connector;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");

	list_for_each_safe(p, n, &card->list_connectors) {
		connector = list_entry(p, struct via_chrome9_connector, list);
		via_chrome9_external_card_unregister_connector(connector);
	}

	kfree(card->private_data);
}


static void ad9389_dpms(struct via_chrome9_external_card *card, int mode)
{
	KMS_DEBUG("move to ums\n");
}

static int via_chrome9_ad9389_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *via_dvi_con_info = NULL;
	KMS_DEBUG("\n");

	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_HDMIA:
		/* Currently, only VX855 can support CEA interlace mode. */
		if ((p_priv->pseudo_chip_revision < REVISION_VX855_A0) &&
			(mode->flags & DRM_MODE_FLAG_INTERLACE))
			return MODE_NOMODE;

		if (mode->type & DRM_MODE_TYPE_DRIVER)
			return MODE_OK;
		else
			return MODE_NOMODE;
		break;
	case DRM_MODE_CONNECTOR_DVID:
		via_dvi_con_info =
			(struct via_dvi_connector_private_info *)
			via_con->connector_info;
		/* Check Clock Range */
		if (mode->clock > 165000)
			return MODE_CLOCK_HIGH;
		if (mode->clock < 25000)
			return MODE_CLOCK_LOW;
		/* Check monitor physical size */
		if (mode->hdisplay > via_dvi_con_info->physical_width)
			return MODE_BAD_HVALUE;
		if (mode->vdisplay > via_dvi_con_info->physical_height)
			return MODE_BAD_VVALUE;
		/* Check Resolution Range */
		if (via_chrome9_is_valid_mode(mode->hdisplay,
			mode->vdisplay) == 0)
			return MODE_BAD;
		break;
	default:
		break;
	}
	return MODE_OK;
}

static int via_chrome9_ad9389_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct drm_device *dev = connector->dev;
	struct via_dvi_connector_private_info *dvi_private_info = NULL;
	struct via_dvi_options *via_dvi_options = NULL;

	int ret = 0;
	u8 *block;
	KMS_DEBUG("enter\n");
	if (!via_con->edid)
		return 0;
	block = kmalloc(EDID_LENGTH * 2, GFP_KERNEL);
	if (block == NULL)
		return -1;

	drm_mode_connector_update_edid_property(connector, via_con->edid);
	ret = drm_add_edid_modes(connector, via_con->edid);
	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_HDMIA:
		memcpy(block, via_con->edid, EDID_LENGTH * 2);
		via_chrome9_prune_un_preferred_modes(dev,
			&(connector->probed_modes));
		via_chrome9_prune_un_cea_modes(dev, &(connector->probed_modes));
		ret = via_chrome9_add_cea_mode_line(connector, block + 128);
		break;

	case DRM_MODE_CONNECTOR_DVID:
		dvi_private_info =
			(struct via_dvi_connector_private_info *)
			via_con->connector_info;
		via_dvi_options = (struct via_dvi_options *)via_con->options;

		if (via_dvi_options->no_ddc_value) {
			dvi_private_info->physical_width = 4096;
			dvi_private_info->physical_height = 2048;
		} else {
			via_chrome9_get_monitor_physical_size(connector,
				&dvi_private_info->physical_width,
				&dvi_private_info->physical_height);
		}
		break;
	}
	kfree(block);
	return ret;
}

/*
static void ad9389_force(struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{

	struct drm_connector_helper_funcs *connector_func =
			connector->helper_private;

	KMS_DEBUG("enter\n");

	connector_func->get_modes =
		via_chrome9_ad9389_get_modes;
	connector_func->mode_valid =
		via_chrome9_ad9389_mode_valid;
}
*/

/**
 * ad9389_hotplug_uevent_send - generate a DRM uevent
 * @dev: DRM device
 * Send an uevent for the ad9389 hdmi or dvi device hotplug.
 */
static void ad9389_hotplug_uevent_send(struct drm_device *dev)
{
	char *event_string = "DVP1HPD=1";
	char *envp[] = { event_string, NULL };

	DRM_DEBUG("generating ad9389 device hotplug uevent\n");
	kobject_uevent_env(&dev->primary->kdev.kobj, KOBJ_CHANGE, envp);
}

static void ad9389_read_edid_process(struct drm_connector *connector,
		struct edid *edid)
{
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	KMS_DEBUG("\n");
	if (edid == NULL) {
		KMS_DEBUG("EDID is NULL!\n");
		return;
	}
	if (p_priv->is_ad9389_hpd_irq == true) {
		p_priv->is_ad9389_hpd_irq = false;
		memset(edid, 0, sizeof(struct edid));

		ad9389_hotplug_uevent_send(dev);
		KMS_DEBUG(
			"prepare to delay 1.2s for getting ad9389 device edid!\n");
		schedule_timeout_uninterruptible(msecs_to_jiffies(1200));
		KMS_DEBUG("Done delay 1.2s.\n");
	}
}

static enum drm_connector_status ad9389_try_detect(
		struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct edid *edid;
	struct ad9389_priv *ad9389 = card->private_data;
	struct via_dvi_options *via_dvi_options = NULL;
	int ret = connector_status_disconnected;
	bool hdmi = false;
	KMS_DEBUG("\n");
	via_con->edid = (struct edid *)(ad9389->std_edid);

	edid = via_con->edid;
	ad9389_read_edid_process(connector, edid);
	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_HDMIA:
		if (edid == NULL)
			KMS_DEBUG("ad9389 has no edid\n");

		if ((edid != NULL) && (drm_edid_is_valid(edid)))
			hdmi = drm_detect_hdmi_monitor(edid);

		if (!hdmi) {
			KMS_DEBUG("this connector is not an hdmi\n");
			KMS_DEBUG("%d, %d", ad9389->ext_edid[0],
				ad9389->ext_edid[1]);
			ret = connector_status_disconnected;
		} else {
			KMS_DEBUG("hdmi is connected!\n");
			ret = connector_status_connected;
		}
		break;
	case DRM_MODE_CONNECTOR_DVID:
		via_dvi_options = (struct via_dvi_options *)via_con->options;
		if (edid == NULL)
			printk(KERN_WARNING "ad9389 has no edid\n");

		if ((edid != NULL) && (drm_edid_is_valid(edid))) {
			hdmi = drm_detect_hdmi_monitor(edid);
			if (hdmi) {
				ret = connector_status_disconnected;
				KMS_DEBUG("It is not DVI, \
					so disconnected DVI device!\n");
			} else {
				ret = connector_status_connected;
			}
		} else {
			if (via_dvi_options->no_ddc_value) {
				ret = connector_status_connected;
				KMS_DEBUG("DVI is connected!\n");
			}
		}
		break;
	}

	connector_func->get_modes = via_chrome9_ad9389_get_modes;
	connector_func->mode_valid = via_chrome9_ad9389_mode_valid;

	return ret;
}

static void ad9389_hdmi_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct drm_device *drm_dev = card->drm_dev;
	struct drm_encoder *encoder = card->encoder;
	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)drm_dev->dev_private;
	int iga_path = via_chrome9_crtc->crtc_id;

	struct via_cea_timing_info_rec *cea_timing_info =
		&via_cea_timing_table[mode->hskew - 1];

	if (IGA1 == iga_path)
		via_chrome9_load_crtc_pixel_timing(mode);

	/* Set Hsync Offset, delay one clock (To meet 861-D spec.)*/
	via_chrome9_write_vga_io_bits(REG_CR8A, 0x01, 0x7);

	/* If CR8A +1, HSyc must -1 */
	via_chrome9_write_vga_io(REG_CR56,
		via_chrome9_read_vga_io(REG_CR56) - 1);
	via_chrome9_write_vga_io(REG_CR57,
		via_chrome9_read_vga_io(REG_CR57) - 1);

	if (mode->flags & DRM_MODE_FLAG_INTERLACE) {
		p_priv->is_ext_hdmi_interlace_mode = true;
		if (IGA2 == iga_path) {
			switch (drm_dev->pci_device) {
			case VX855_DEVICE:
				via_chrome9_write_vga_io_bits(REG_CRFB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRFC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				break;

			case VX900_DEVICE:
				via_chrome9_write_vga_io_bits(REG_CRAB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRAC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				/**
				 * According to information from HW team,
				 * we need to set 0xC280[1] = 1
				 * (HDMI function enable)
				 * or 0xC640[0] = 1 (DP1 enable) to
				 * let the half line function work.
				 * Otherwise, the clock for interlace mode
				 * will not correct.
				 * This is a special setting for 410.
				 */
				MMIO_WR_MASK(0xC280, BIT1, BIT1);
				break;

			default:
				via_chrome9_write_vga_io_bits(REG_CRFB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRFC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				break;
			}
		}
	} else { /* non-interlace, clear interlace setting. */
		p_priv->is_ext_hdmi_interlace_mode = false;
		if (IGA2 == iga_path) {
			via_chrome9_write_vga_io_bits(REG_CRFB, 0, 0xFF);
			via_chrome9_write_vga_io_bits(REG_CRFC, 0, 0x07);
		}
	}
	via_chrome9_patch_dot_clock(encoder, mode);
	via_chrome9_hdmi_gfx_dpa_setting(card, mode);
}

static void ad9389_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct drm_connector *current_con = card->current_con;
	KMS_DEBUG("\n");
	switch (current_con->connector_type) {
	case DRM_MODE_CONNECTOR_HDMIA:
		KMS_DEBUG("hdmi mode set!\n");
		ad9389_hdmi_mode_set(card, mode);
		break;
	case DRM_MODE_CONNECTOR_DVID:
		KMS_DEBUG("DVI mode set!\n");
		via_chrome9_dvi_gfx_dpa_setting(card, mode);
		break;
	}
}

static bool via_chrome9_external_hdmi_mode_fixup(
		struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjust_mode)
{
	return via_hdmi_mode_fixup(encoder, mode, adjust_mode);
}

static bool ad9389_mode_fixup(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjust_mode)
{
	struct drm_connector *current_con = card->current_con;
	KMS_DEBUG("%d\n", current_con->connector_type);

	switch (current_con->connector_type) {
	case DRM_MODE_CONNECTOR_HDMIA:
		return via_chrome9_external_hdmi_mode_fixup(card->encoder,
			mode, adjust_mode);
		break;
	case DRM_MODE_CONNECTOR_DVID:
		return true;
		break;
	}
	return false;
}

struct via_chrome9_external_card_ops ad9389_ops = {
	.init = ad9389_init,
	.deinit = ad9389_deinit,
	.dpms = ad9389_dpms,
	.try_detect = ad9389_try_detect,
	.mode_set = ad9389_mode_set,
	.mode_fixup = ad9389_mode_fixup
};
