/*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

 /*
  * Programming info contained in this module for Chrontel 7301
  * can be found in http://www.chrontel.com/pdf/7301ds.pdf.
  */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/i2c.h>

#include "via_chrome9_external_card.h"


/*#define ch7301_write(c, reg, val)
 *ia_chrome9_external_card_write(c, reg, val)
 */

#define ch7301_read(c, reg)	via_chrome9_external_card_read(c, reg)
#define ch7301_write_mask(c, reg, val, mask)	\
	via_chrome9_external_card_write_mask(c, reg, val, mask)


static enum drm_connector_status ch7301_vga_detect(
		struct via_chrome9_external_card *card)
{
	u8 reg49, reg20, reg21;
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	enum drm_connector_status status;

	/* Save register 0x49, 0x21 */
	reg49 = ch7301_read(client, 0x49);
	reg21 = ch7301_read(client, 0x21);

	/* Enable all DAC */
	ch7301_write_mask(client, 0x49, 0x00, 0x0f);
	ch7301_write_mask(client, 0x21, 0x00, BIT0);

	/* SENSE bit: 1->0, triggger a comparison */
	ch7301_write_mask(client, 0x20, BIT0, BIT0);
	ch7301_write_mask(client, 0x20, 0x00, BIT0);

	/* Read the status bits */
	reg20 = ch7301_read(client, 0x20);
	if (reg20 & 0x0e) {
		KMS_DEBUG("Detected CRT device on CH3701 .\n");
		status = connector_status_connected;
	} else {
		KMS_DEBUG("No CRT device on CH3701 .\n");
		status = connector_status_disconnected;
	}

	ch7301_write_mask(client, 0x49, reg49, 0xff);
	ch7301_write_mask(client, 0x21, reg21, 0xff);

	return status;
}

static enum drm_connector_status ch7301_dvi_detect(
		struct via_chrome9_external_card *card)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	enum drm_connector_status status = connector_status_disconnected;
	u8 reg20, reg49;

	/* Bak the value of reg49. */
	reg49 = ch7301_read(client, 0x49);

	ch7301_write_mask(client, 0x49, BIT7 + BIT6, BIT7 + BIT6 + BIT0);

	reg20 = ch7301_read(client, 0x20);
	if (reg20 & 0x20)
		status = connector_status_connected;

	/* Restore reg49 is needed. Or DVI detect will failed. */
	ch7301_write_mask(client, 0x49, reg49, 0xff);

	return status;
}

static void ch7301_vga_dpms(struct via_chrome9_external_card *card,
		int mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	if (mode == DRM_MODE_DPMS_ON)
		ch7301_write_mask(client, 0x49, 0x00, 0xCF);
	else
		ch7301_write_mask(client, 0x49, BIT0, BIT0);
}

static void ch7301_dvi_dpms(struct via_chrome9_external_card *card,
		int mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	if (mode == DRM_MODE_DPMS_ON)
		ch7301_write_mask(client, 0x49, 0xCE, 0xCF);
	else
		ch7301_write_mask(client, 0x49, BIT0, BIT0);
}

static void ch7301_vga_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	u8 i, reg4d = 0;

	KMS_DEBUG("enter\n");

	/* reg 0x1C: 0x00 -- dual edge */
	ch7301_write_mask(client, 0x1C, 0x00, BIT2 + BIT0);
	/* reg 0x1F: 0x00 -- IDF: 0 */
	ch7301_write_mask(client, 0x1F, 0x00, 0x5F);
	/* reg 0x49[7:6]=0, turn off DVIP and DVIL */
	ch7301_write_mask(client, 0x49, 0x00, 0xC0);
	/* set pll registers in terms of input clock frequency */
	if (mode->clock < 65000) {
		ch7301_write_mask(client, 0x31, 0x00, 0xff);
		ch7301_write_mask(client, 0x33, 0x08, 0xfd);
		ch7301_write_mask(client, 0x34, 0x16, 0x3f);
		ch7301_write_mask(client, 0x36, 0x60, 0xf0);
	} else {
		ch7301_write_mask(client, 0x31, 0x00, 0xff);
		ch7301_write_mask(client, 0x33, 0x06, 0xfd);
		ch7301_write_mask(client, 0x34, 0x26, 0x3f);
		ch7301_write_mask(client, 0x36, 0xA0, 0xf0);
	}

	/* delay 50ms */
	mdelay(50);
	/* reg 0x21: 0x09 -- Enable VSYNC and HSYNC */
	ch7301_write_mask(client, 0x21, 0x09, 0x0F);
	/* follow CH7301C spec : for CRT, 0x21.0 = 1, and 0x56.0 = 1. */
	ch7301_write_mask(client, 0x56, BIT0, BIT0);
	/* reg 0x49[7:6]=11b, turn on DVIP and DVIL */
	ch7301_write_mask(client, 0x49, BIT7 + BIT6, BIT7 + BIT6);
	/* delay 10ms */
	mdelay(10);

	/* Check PLL lock status  */
	for (i = 0; i < 20; i++) {
		/* reg 0x48[3]:0->1, reset data path */
		ch7301_write_mask(client, 0x48, 0x00, BIT3);
		ch7301_write_mask(client, 0x48, BIT3, BIT3);
		reg4d = ch7301_read(client, 0x4D);
		if (BIT2 & reg4d)
			break;
	}

}

static void ch7301_dvi_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	u8 i, reg4d = 0;

	KMS_DEBUG("enter\n");

	/* reg 0x1C: 0x00 -- dual edge */
	ch7301_write_mask(client, 0x1C, 0x00, BIT2 + BIT0);
	/* reg 0x1F: 0x00 -- IDF: 0 */
	ch7301_write_mask(client, 0x1F, 0x00, 0x5F);
	/* reg 0x49[7:6]=0, turn off DVIP and DVIL */
	ch7301_write_mask(client, 0x49, 0x00, BIT7 + BIT6);
	/* set pll registers in terms of input clock frequency */
	if (mode->clock < 65000) {
		ch7301_write_mask(client, 0x31, 0x00, 0xff);
		ch7301_write_mask(client, 0x33, 0x08, 0xfd);
		ch7301_write_mask(client, 0x34, 0x16, 0x3f);
		ch7301_write_mask(client, 0x36, 0x60, 0xf0);
	} else {
		ch7301_write_mask(client, 0x31, 0x00, 0xff);
		ch7301_write_mask(client, 0x33, 0x06, 0xfd);
		ch7301_write_mask(client, 0x34, 0x26, 0x3f);
		ch7301_write_mask(client, 0x36, 0xA0, 0xf0);
	}

	/* delay 50ms */
	mdelay(50);
	/* reg 0x21: 0x08 -- Enable VSYNC and HSYNC */
	ch7301_write_mask(client, 0x21, 0x08, 0x0F);
	/* reg 0x49[7:6]=11b, turn on DVIP and DVIL */
	ch7301_write_mask(client, 0x49, BIT7 + BIT6, BIT7 + BIT6);
	/* delay 10ms */
	mdelay(10);

	/* Check PLL lock status  */
	for (i = 0; i < 20; i++) {
		/* reg 0x48[3]:0->1, reset data path */
		ch7301_write_mask(client, 0x48, 0x00, BIT3);
		ch7301_write_mask(client, 0x48, BIT3, BIT3);
		reg4d = ch7301_read(client, 0x4D);
		if (BIT2 & reg4d)
			break;
	}
}



static int ch7301_init(struct i2c_client *client)
{
	struct via_chrome9_external_card *card = i2c_get_clientdata(client);

	KMS_DEBUG("enter\n");
	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_DVID);
	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_VGA);
	return 0;
}

static void ch7301_deinit(struct i2c_client *client)
{
	struct via_chrome9_external_card *card = i2c_get_clientdata(client);
	struct via_chrome9_connector *connector;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");
	list_for_each_safe(p, n, &card->list_connectors) {
		connector = list_entry(p, struct via_chrome9_connector, list);
		via_chrome9_external_card_unregister_connector(connector);
	}
}

static void ch7301_dpms(struct via_chrome9_external_card *card, int mode)
{
	struct drm_connector *current_con = card->current_con;

	KMS_DEBUG("enter\n");

	if (!current_con) {
		KMS_DEBUG("ch7301 encoder has no connector\n");
		return;
	}

	switch (current_con->connector_type) {
	case DRM_MODE_CONNECTOR_DVID:
		ch7301_dvi_dpms(card, mode);
		break;
	case DRM_MODE_CONNECTOR_VGA:
		ch7301_vga_dpms(card, mode);
		break;
	}
}


static int via_chrome9_ch7301_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	int ret = 0;
	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_DVID:
		ret = via_dvi_mode_valid(connector, mode);
		break;
	case DRM_MODE_CONNECTOR_VGA:
		ret = via_vga_mode_valid(connector, mode);
		break;
	}
	return ret;
}


static int via_chrome9_ch7301_get_modes(struct drm_connector *connector)
{
	int ret = 0;
	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_DVID:
		ret = via_dvi_get_modes(connector);
		break;
	case DRM_MODE_CONNECTOR_VGA:
		ret = via_vga_get_modes(connector);
		break;
	}
	return ret;
}

static void ch7301_force(struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;

	connector_func->get_modes = via_chrome9_ch7301_get_modes;
	connector_func->mode_valid = via_chrome9_ch7301_mode_valid;
}

static enum drm_connector_status ch7301_try_detect(
		struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	enum drm_connector_status ret = connector_status_disconnected;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *) via_con->options;
	struct via_vga_options *via_vga_options =
		(struct via_vga_options *) via_con->options;

	KMS_DEBUG("enter\n");

	switch (connector->connector_type) {
	case DRM_MODE_CONNECTOR_DVID:
		KMS_DEBUG("try to detect dvi on ch7301!\n");
		ret = ch7301_dvi_detect(card);

		if (connector_status_disconnected == ret) {
			KMS_DEBUG("CH7301C daughter card disconnected!\n");
			return ret;
		}

		if (via_dvi_options->no_ddc_value) {
			KMS_DEBUG("dvi NoDDCValue.\n");
			ret = connector_status_connected;
		} else {
			if (true == via_chrome9_detect_device_by_read_edid(
				connector, DIGITAL_DEVICE))
				ret = connector_status_connected;
			else
				ret = connector_status_disconnected;
		}
		break;

	case DRM_MODE_CONNECTOR_VGA:
		KMS_DEBUG("try to detect vga on ch7301!\n");
		ret = ch7301_vga_detect(card);

		if (connector_status_disconnected == ret) {
			KMS_DEBUG("CH7301C daughter card disconnected!\n");
			return ret;
		}

		if (via_vga_options->no_ddc_value) {
			KMS_DEBUG("vga NoDDCValue.\n");
			ret = connector_status_connected;
		} else {
			if (true == via_chrome9_detect_device_by_read_edid(
				connector, ANALOG_DEVICE))
				ret = connector_status_connected;
			else
				ret = connector_status_disconnected;
		}
		break;
	}
	connector_func->get_modes = via_chrome9_ch7301_get_modes;
	connector_func->mode_valid = via_chrome9_ch7301_mode_valid;

	return ret;

}


static void ch7301_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct drm_connector *current_con = card->current_con;

	KMS_DEBUG("enter\n");

	if (!current_con) {
		KMS_DEBUG("ch7301 encoder has no connector\n");
		return;
	}

	switch (current_con->connector_type) {
	case DRM_MODE_CONNECTOR_DVID:
		ch7301_dvi_mode_set(card, mode);
		via_chrome9_dvi_gfx_dpa_setting(card, mode);
		break;
	case DRM_MODE_CONNECTOR_VGA:
		ch7301_vga_mode_set(card, mode);
		via_chrome9_vga_gfx_dpa_setting(card, mode);
		break;
	}

}

struct via_chrome9_external_card_ops ch7301_ops = {
	.init = ch7301_init,
	.deinit = ch7301_deinit,
	.dpms = ch7301_dpms,
	.force = ch7301_force,
	.try_detect = ch7301_try_detect,
	.mode_set = ch7301_mode_set,
};
