/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef __VIA_CHROME9_DISPLAY_COMMON_H__
#define __VIA_CHROME9_DISPLAY_COMMON_H__

#include "drm.h"
#include "drm_edid.h"

struct via_chrome9_io_reg {
	u16    index;
	u8     mask;
	u8     value;
};

struct via_chrome9_io_bit {
	u16     io_addr;
	u8      start_bit;
	u8      end_bit;
};

/*Subchip IO data structure*/
struct via_chrome9_io_data {
	unsigned char   index;
	unsigned char   mask;
	unsigned char   data;
};

/*Bit Defination*/
#ifndef BIT0
#define BIT0        0x00000001
#endif

#ifndef BIT1
#define BIT1        0x00000002
#endif

#ifndef BIT2
#define BIT2        0x00000004
#endif

#ifndef BIT3
#define BIT3        0x00000008
#endif

#ifndef BIT4
#define BIT4        0x00000010
#endif

#ifndef BIT5
#define BIT5        0x00000020
#endif

#ifndef BIT6
#define BIT6        0x00000040
#endif

#ifndef BIT7
#define BIT7        0x00000080
#endif

#ifndef BIT8
#define BIT8        0x00000100
#endif

#ifndef BIT9
#define BIT9        0x00000200
#endif

#ifndef BIT10
#define BIT10       0x00000400
#endif

#ifndef BIT11
#define BIT11       0x00000800
#endif

#ifndef BIT12
#define BIT12       0x00001000
#endif

#ifndef BIT13
#define BIT13       0x00002000
#endif

#ifndef BIT14
#define BIT14       0x00004000
#endif

#ifndef BIT15
#define BIT15       0x00008000
#endif

#ifndef BIT16
#define BIT16       0x00010000
#endif

#ifndef BIT17
#define BIT17       0x00020000
#endif

#ifndef BIT18
#define BIT18       0x00040000
#endif

#ifndef BIT19
#define BIT19       0x00080000
#endif

#ifndef BIT20
#define BIT20       0x00100000
#endif

#ifndef BIT21
#define BIT21       0x00200000
#endif

#ifndef BIT22
#define BIT22       0x00400000
#endif

#ifndef BIT23
#define BIT23       0x00800000
#endif

#ifndef BIT24
#define BIT24       0x01000000
#endif

#ifndef BIT25
#define BIT25       0x02000000
#endif

#ifndef BIT26
#define BIT26       0x04000000
#endif

#define REG_ZCR      0x3D4
#define REG_ZSR      0x3C4
#define REG_ZGR      0x3CE
#define REG_ZAR      0x3C0
#define REG_RMISC    0x3CC
#define REG_WMISC    0x3C2
#define REG_STATUS   0x3DA
#define REG_PEL      0x3C8

#define IS_ZSR       0x0100
#define IS_ZGR       0x0200
#define IS_ZAR       0x0400
#define REG_INDEX_MASK  0xff
#define VGAIO_BASE   0x8000


/* Sequencer Registers */
#define REG_SR00         0x0100
#define REG_SR01         0x0101
#define REG_SR02         0x0102
#define REG_SR03         0x0103
#define REG_SR04         0x0104
#define REG_SR05         0x0105
#define REG_SR06         0x0106
#define REG_SR07         0x0107
#define REG_SR08         0x0108
#define REG_SR09         0x0109
#define REG_SR10         0x0110
#define REG_SR11         0x0111
#define REG_SR12         0x0112
#define REG_SR13         0x0113
#define REG_SR14         0x0114
#define REG_SR15         0x0115
#define REG_SR16         0x0116
#define REG_SR17         0x0117
#define REG_SR18         0x0118
#define REG_SR19         0x0119
#define REG_SR1A         0x011A
#define REG_SR1B         0x011B
#define REG_SR1C         0x011C
#define REG_SR1D         0x011D
#define REG_SR1E         0x011E
#define REG_SR1F         0x011F
#define REG_SR20         0x0120
#define REG_SR21         0x0121
#define REG_SR22         0x0122
#define REG_SR23         0x0123
#define REG_SR24         0x0124
#define REG_SR25         0x0125
#define REG_SR26         0x0126
#define REG_SR27         0x0127
#define REG_SR28         0x0128
#define REG_SR29         0x0129
#define REG_SR2A         0x012A
#define REG_SR2B         0x012B
#define REG_SR2C         0x012C
#define REG_SR2D         0x012D
#define REG_SR2E         0x012E
#define REG_SR2F         0x012F
#define REG_SR30         0x0130
#define REG_SR31         0x0131
#define REG_SR32         0x0132
#define REG_SR33         0x0133
#define REG_SR34         0x0134
#define REG_SR35         0x0135
#define REG_SR36         0x0136
#define REG_SR37         0x0137
#define REG_SR38         0x0138
#define REG_SR39         0x0139
#define REG_SR3A         0x013A
#define REG_SR3B         0x013B
#define REG_SR3C         0x013C
#define REG_SR3D         0x013D
#define REG_SR3E         0x013E
#define REG_SR3F         0x013F
#define REG_SR40         0x0140
#define REG_SR41         0x0141
#define REG_SR42         0x0142
#define REG_SR43         0x0143
#define REG_SR44         0x0144
#define REG_SR45         0x0145
#define REG_SR46         0x0146
#define REG_SR47         0x0147
#define REG_SR48         0x0148
#define REG_SR49         0x0149
#define REG_SR4A         0x014A
#define REG_SR4B         0x014B
#define REG_SR4C         0x014C
#define REG_SR4D         0x014D
#define REG_SR4F         0x014F
#define REG_SR50         0x0150
#define REG_SR51         0x0151
#define REG_SR52         0x0152
#define REG_SR57         0x0157
#define REG_SR58         0x0158
#define REG_SR59         0x0159
#define REG_SR5E         0x015E
#define REG_SR5D         0x015D
#define REG_SR63         0x0163
#define REG_SR65         0x0165
#define REG_SR66         0x0166
#define REG_SR67         0x0167
#define REG_SR69         0x0169
#define REG_SR70         0x0170
#define REG_SR71         0x0171
#define REG_SR72         0x0172
#define REG_SR73         0x0173
#define REG_SR74         0x0174
#define REG_SR75         0x0175
#define REG_SR78         0x0178
#define REG_SR79         0x0179
#define REG_SR7A         0x017A

/* CRT Controller Registers */
#define REG_CR00         0x0000
#define REG_CR01         0x0001
#define REG_CR02         0x0002
#define REG_CR03         0x0003
#define REG_CR04         0x0004
#define REG_CR05         0x0005
#define REG_CR06         0x0006
#define REG_CR07         0x0007
#define REG_CR08         0x0008
#define REG_CR09         0x0009
#define REG_CR0A         0x000A
#define REG_CR0B         0x000B
#define REG_CR0C         0x000C
#define REG_CR0D         0x000D
#define REG_CR0E         0x000E
#define REG_CR0F         0x000F
#define REG_CR10         0x0010
#define REG_CR11         0x0011
#define REG_CR12         0x0012
#define REG_CR13         0x0013
#define REG_CR14         0x0014
#define REG_CR15         0x0015
#define REG_CR16         0x0016
#define REG_CR17         0x0017
#define REG_CR18         0x0018

/* Extend REG_CRT Controller Registers */
#define REG_CR30         0x0030
#define REG_CR31         0x0031
#define REG_CR32         0x0032
#define REG_CR33         0x0033
#define REG_CR34         0x0034
#define REG_CR35         0x0035
#define REG_CR36         0x0036
#define REG_CR37         0x0037
#define REG_CR38         0x0038
#define REG_CR39         0x0039
#define REG_CR3A         0x003A
#define REG_CR3B         0x003B
#define REG_CR3C         0x003C
#define REG_CR3D         0x003D
#define REG_CR3E         0x003E
#define REG_CR3F         0x003F
#define REG_CR40         0x0040
#define REG_CR41         0x0041
#define REG_CR42         0x0042
#define REG_CR43         0x0043
#define REG_CR44         0x0044
#define REG_CR45         0x0045
#define REG_CR46         0x0046
#define REG_CR47         0x0047
#define REG_CR48         0x0048
#define REG_CR49         0x0049
#define REG_CR4A         0x004A
#define REG_CR4B         0x004B
#define REG_CR4C         0x004C
#define REG_CR4D         0x004D
#define REG_CR4E         0x004E
#define REG_CR4F         0x004F
#define REG_CR50         0x0050
#define REG_CR51         0x0051
#define REG_CR52         0x0052
#define REG_CR53         0x0053
#define REG_CR54         0x0054
#define REG_CR55         0x0055
#define REG_CR56         0x0056
#define REG_CR57         0x0057
#define REG_CR58         0x0058
#define REG_CR59         0x0059
#define REG_CR5A         0x005A
#define REG_CR5B         0x005B
#define REG_CR5C         0x005C
#define REG_CR5D         0x005D
#define REG_CR5E         0x005E
#define REG_CR5F         0x005F
#define REG_CR60         0x0060
#define REG_CR61         0x0061
#define REG_CR62         0x0062
#define REG_CR63         0x0063
#define REG_CR64         0x0064
#define REG_CR65         0x0065
#define REG_CR66         0x0066
#define REG_CR67         0x0067
#define REG_CR68         0x0068
#define REG_CR69         0x0069
#define REG_CR6A         0x006A
#define REG_CR6B         0x006B
#define REG_CR6C         0x006C
#define REG_CR6D         0x006D
#define REG_CR6E         0x006E
#define REG_CR6F         0x006F
#define REG_CR70         0x0070
#define REG_CR71         0x0071
#define REG_CR72         0x0072
#define REG_CR73         0x0073
#define REG_CR74         0x0074
#define REG_CR75         0x0075
#define REG_CR76         0x0076
#define REG_CR77         0x0077
#define REG_CR78         0x0078
#define REG_CR79         0x0079
#define REG_CR7A         0x007A
#define REG_CR7B         0x007B
#define REG_CR7C         0x007C
#define REG_CR7D         0x007D
#define REG_CR7E         0x007E
#define REG_CR7F         0x007F
#define REG_CR80         0x0080
#define REG_CR81         0x0081
#define REG_CR82         0x0082
#define REG_CR83         0x0083
#define REG_CR84         0x0084
#define REG_CR85         0x0085
#define REG_CR86         0x0086
#define REG_CR87         0x0087
#define REG_CR88         0x0088
#define REG_CR89         0x0089
#define REG_CR8A         0x008A
#define REG_CR8B         0x008B
#define REG_CR8C         0x008C
#define REG_CR8D         0x008D
#define REG_CR8E         0x008E
#define REG_CR8F         0x008F
#define REG_CR90         0x0090
#define REG_CR91         0x0091
#define REG_CR92         0x0092
#define REG_CR93         0x0093
#define REG_CR94         0x0094
#define REG_CR95         0x0095
#define REG_CR96         0x0096
#define REG_CR97         0x0097
#define REG_CR98         0x0098
#define REG_CR99         0x0099
#define REG_CR9A         0x009A
#define REG_CR9B         0x009B
#define REG_CR9C         0x009C
#define REG_CR9D         0x009D
#define REG_CR9E         0x009E
#define REG_CR9F         0x009F
#define REG_CRA0         0x00A0
#define REG_CRA1         0x00A1
#define REG_CRA2         0x00A2
#define REG_CRA3         0x00A3
#define REG_CRA4         0x00A4
#define REG_CRA5         0x00A5
#define REG_CRA6         0x00A6
#define REG_CRA7         0x00A7
#define REG_CRA8         0x00A8
#define REG_CRA9         0x00A9
#define REG_CRAA         0x00AA
#define REG_CRAB         0x00AB
#define REG_CRAC         0x00AC
#define REG_CRD0         0x00D0
#define REG_CRD1         0x00D1
#define REG_CRD2         0x00D2
#define REG_CRD3         0x00D3
#define REG_CRD4         0x00D4
#define REG_CRD5         0x00D5
#define REG_CRD6         0x00D6
#define REG_CRD7         0x00D7
#define REG_CRDD         0x00DD
#define REG_CRE8         0x00E8
#define REG_CREC         0x00EC
#define REG_CRED         0x00ED
#define REG_CREE         0x00EE
#define REG_CREF         0x00EF
#define REG_CRF0         0x00F0
#define REG_CRF1         0x00F1
#define REG_CRF2         0x00F2
#define REG_CRF3         0x00F3
#define REG_CRF4         0x00F4
#define REG_CRF5         0x00F5
#define REG_CRF6         0x00F6
#define REG_CRF7         0x00F7
#define REG_CRF8         0x00F8
#define REG_CRF9         0x00F9
#define REG_CRFA         0x00FA
#define REG_CRFB         0x00FB
#define REG_CRFC         0x00FC
#define REG_CRFD         0x00FD
#define REG_CRFE         0x00FE
#define REG_CRFF         0x00FF

#define REG_GR00         0x0200
#define REG_AR00         0x0400
#define PANEL_TD_TIMING_REG_AMOUNT	2
#define PANEL_TD_TIMING_AMOUNT	4

#define LCD_POWER_SEQUENCE_TD0	200
#define LCD_POWER_SEQUENCE_TD1	25
#define LCD_POWER_SEQUENCE_TD2	0
#define LCD_POWER_SEQUENCE_TD3	250

#define DP_DATA_PASS_ENABLE_REG	0xC000

/*****************************************************************************
*     DisplayPort Register
*****************************************************************************/
#define DP_ATTR_DATA_REG	0xC610
#define DP_LINK_TRAINING_REG	0xC614
#define DP_VIDEO_CTRL_REG	0xC618
#define DP_VER_EXT_PKT_HEAD_REG	0xC61C

/*DP Display Port Enable and InfoFrame Control*/
#define DP_ENABLE_IF_REG	0xC640
#define DP_HWIDTH_TUSIZE_REG	0xC644
#define DP_HLINE_DUR_REG	0xC648
#define DP_MVID_MISC0_REG	0xC64C

#define DP_H_ATTR_REG	0xC650
#define DP_HV_START_REG	0xC654
#define DP_POLARITY_WIDTH_REG	0xC658
#define DP_ACITVE_WH_REG	0xC65C

#define AUX_W_DATA0_REG         0xC710
#define AUX_W_DATA1_REG         0xC714
#define AUX_W_DATA2_REG         0xC718
#define AUX_W_DATA3_REG         0xC71C

#define AUX_R_DATA0_REG         0xC720
#define AUX_R_DATA1_REG         0xC724
#define AUX_R_DATA2_REG         0xC728
#define AUX_R_DATA3_REG         0xC72C
#define    VIA_IRQ_DP_HOT_IRQ	0xC0000000
#define    VIA_IRQ_DP_HOT_UNPLUG	0x80000000
#define    VIA_IRQ_DP_HOT_PLUG	0x40000000
#define    VIA_IRQ_DP_NO_INT	0x00000000

#define AUX_TIMER_REG	0xC730
#define AUX_CMD_REG	0xC734
#define DP_NAUD_MUTE_REG	0xC738

#define DP_EPHY_PLL_REG	0xC740
#define DP_EPHY_TX_PWR_REG	0xC744
#define DP_EPHY_MISC_PWR_REG	0xC748


/*****************************************************************************
*     DisplayPort2 Register
*****************************************************************************/
#define DP2_NVID_MISC0_REG	0xC690
#define DP2_LINK_TRAINING_REG	0xC694
#define DP2_VIDEO_CTRL_REG	0xC698
#define DP2_EXT_REG		0xC69C
#define DP2_VER_EXT_PKT_HEAD_REG	0xC61C

/*DP2 Display Port Enable and InfoFrame Control*/
#define DP2_ENABLE_IF_REG	0xC6C0
#define DP2_HWIDTH_TUSIZE_REG		0xC6C4
#define DP2_HLINE_DUR_REG	0xC6C8
#define DP2_MVID_MISC0_REG	0xC6CC

#define DP2_H_ATTR_REG		0xC6D0
#define DP2_HV_START_REG	0xC6D4
#define DP2_POLARITY_WIDTH_REG	0xC6D8
#define DP2_ACITVE_WH_REG		0xC6DC

/* the same with DP1*/
#define DP2_EPHY_SSC_REG	0xC740
/* the same with DP1*/
#define DP2_EPHY_RT_REG	0xC744

#define DP2_AUX_W_DATA0_REG	0xC790
#define DP2_AUX_W_DATA1_REG	0xC794
#define DP2_AUX_W_DATA2_REG	0xC798
#define DP2_AUX_W_DATA3_REG	0xC79C

#define DP2_AUX_R_DATA0_REG	0xC7A0
#define DP2_AUX_R_DATA1_REG	0xC7A4
#define DP2_AUX_R_DATA2_REG	0xC7A8
#define DP2_AUX_R_DATA3_REG	0xC7AC

#define DP2_AUX_TIMER_REG	0xC7B0
#define DP2_AUX_CMD_REG	0xC7B4
#define DP2_NAUD_MUTE_REG	0xC7B8

#define DP2_EPHY_TX_PWR_REG2	0xC7C0
#define DP2_EPHY_TX_IDLE_REG	0xC7C4
#define DP2_EPHY_TX_PWR_REG	0xC7C8
#define DP2_EPHY_PLL_REG	0xC7CC

#define SPEED_NONE	0
#define SPEED_162MHz	1
#define SPEED_270MHz	2

#define NUM_RE_ISSUE_COMMAND	3
#define NUM_RECHECK	20

#define ACK_OK	0x00000005
#define RE_ISSUE_CMD	0x0005

#define NONEVERSION	0
#define DPCD_VER_1	0x100
#define DPCD_REV_0	0x001
#define DPCD_REV_1	0x002

#define AUX_CMD_I2C_WRITE	0x00
#define AUX_CMD_I2C_READ	0x01

#define AUX_CMD_WRITE	0x08
#define AUX_CMD_READ	0x09

#define LANE	4
#define TU_SIZE	47

#define ANALOG_DEVICE  0x00
#define DIGITAL_DEVICE 0x01

/*Power sequence relations*/
struct td_timer_info {
	u32 td_timer0;
	u32 td_timer1;
	u32 td_timer2;
	u32 td_timer3;
};

struct td_timer {
	struct via_chrome9_io_bit tdRegs[PANEL_TD_TIMING_REG_AMOUNT];
};

struct via_lcd_private_info {
	u32 physical_width;
	u32 physical_height;
};
struct via_hdmi_info_frame_rec {
	u8 packet_header[3];
	u8 ecc_code;
	u8 packet_byte[28];
	u8 bch_code[4];
};

struct via_cea_timing_info_rec {
	u16 id;
	u16 hor_size;
	u16 ver_size;
	u16 refresh_rate;
	int scan_type;
	u16 vsync_offset;
	int ratio;
	 /* If the timing table exists. */
	bool is_built;
};

struct via_hd_audio_wall_clock_reg_setting {
	u32 pixel_clock;
	u32 wall_clock_ratio_l;
	u32 wall_clock_rateo_h;
};

struct via_hd_audio_packet_clock_reg_setting {
	u32 sample_rate;
	u32 pixel_clock;
	u32 packet_clock_ratio_l;
	u32 packet_clock_rateo_h;
};

struct via_hd_audio_cts_and_n_reg_setting {
	u32 sample_rate;
	u32 pixel_clock;
	u32 cts;
	u32 n;
};

enum via_dp_1_ephy_mode {
	via_hdmi_mode = 0x00,
	via_display_port_mode = 0x01,
	via_unknow_mode = 0xff,
};

struct via_hdmi_encoder_private_info {
	enum via_dp_1_ephy_mode dp_1_ephy_mode;
	u32 connector_type;
};

struct via_hdmi_connector_private_info {
	u32 physical_width;
	u32 physical_height;
	u32 connector_type;
	bool read_edid;
	u8 std_edid[128];
	u8 ext_edid[128];
};

struct via_dp_encoder_private_info {
	int link_speed;
	enum via_dp_1_ephy_mode dp_1_ephy_mode;
	u32 tu_ratio;
	bool link_training_done;
	bool dp_connected;
};

struct via_dp_connector_private_info {
	int dpcd_version;
	int link_speed;
	u32 tu_ratio;
	u32 physical_width;
	u32 physical_height;
};

struct via_dvi_connector_private_info {
	u32 physical_width;
	u32 physical_height;
	u32 connector_type;
	bool read_edid;
	u8 std_edid[128];
	u8 ext_edid[128];
};

struct via_dot_clock {
	int dot_clock;
	u32 pll3;
	u32 pll3_409;
};

#define RATIO_4_3	1
#define RATIO_5_4	2
#define RATIO_16_10	3
#define RATIO_16_9	4


#define INTERLACE	1
#define PROGRESSIVE	0

#define DISP_DI_NONE	0x0000
#define DISP_DI_DVP0	BIT0
#define DISP_DI_DVP1	BIT1
#define DISP_DI_DFPL	BIT2
#define DISP_DI_DFPH	BIT3
#define DISP_DI_DFP	BIT4
#define DISP_DI_DAC	BIT5
#define DISP_DI_ALL	(DISP_DI_DVP0 | DISP_DI_DVP1 | DISP_DI_DFPL |  \
		DISP_DI_DFPH | DISP_DI_DFP | DISP_DI_DAC)
/* read */
#define VGA_MISC_OUT_R		0x3CC
/* write */
#define VGA_MISC_OUT_W	0x3C2

extern unsigned char *mmio_map_base;
extern unsigned char *mmio_mb1;
/*for detect EDID*/
/*EDID1: the max monitor name length is 13 bytes
EDID2: the max monitor name length is 32 bytes*/
#define MON_NAME_MAX_LEN    32
/* MISC */
#define CMDISP_ARRAYSIZE(x)   (sizeof(x) / sizeof(*(x)))
#define HIGH_BYTE_OF_WORD(x)    (u8)((x) >> 8)
#define LOW_BYTE_OF_WORD(x)     (u8)((x) & 0x00FF)
#define CMDISP_ALIGN_TO(val, al) (((val) + ((al) - 1)) & ~((al) - 1))

/*temply use globle variable mmio_map_base, we should refine it laterly*/
#define MMIO_WR8(address, data)	\
do {		\
	*((__volatile__ u8*)((address))) = (data)	;		\
} while (0)
#define MMIO_WR16(address, data)		\
do {		\
	*((__volatile__ u16*)((address))) = (data);		\
} while (0)
#define MMIO_WR32(address, data)		\
do {		\
	*((__volatile__ u32*)((address))) = (data);		\
} while (0)
#define MMIO_WRBITS8(address, data, mask)	\
do {		\
	*((__volatile__ u8*)((address))) = ((data) & (mask));		\
} while (0)
#define MMIO_WRBITS16(address, data, mask)	\
do {		\
	*((__volatile__ u16*)((address))) = ((data) & (mask));		\
} while (0)
#define MMIO_WRBITS32(address, data, mask)	\
do {		\
	*((__volatile__ u32*)((address))) = ((data) & (mask));		\
} while (0)
#define MMIO_RD8(address) (*((__volatile__ u8*)(address)))
#define MMIO_RD16(address) (*((__volatile__ u16*)(address)))
#define MMIO_RD32(address) (*((__volatile__ u32*)(address)))
#define STANDVGA_R8(address) MMIO_RD8(mmio_map_base + address)
#define STANDVGA_W8(address, data) MMIO_WR8(mmio_map_base + address, data)
#define REG_R32(address) MMIO_RD32(mmio_map_base + address)
#define REG_W32(address, data) MMIO_WR32(mmio_map_base + address, data)
#define MMIO_WR(addr, data) (*(unsigned int *)(mmio_mb1 + (addr)) = (data))
#define MMIO_RD(addr) (*(unsigned int *)(mmio_mb1 + (addr)))

#define MMIO_WR_MASK(addr, data, mask)	\
do {		\
	*(unsigned int *)(mmio_mb1 + (addr)) =	\
		(data & mask) | (MMIO_RD(addr) & (~mask));	\
} while (0)
extern u8   via_chrome9_read_vga_io(u16 io_name);
inline u8   via_chrome9_read_misc_io(void);
inline u32  via_chrome9_write_misc_io(u8 data);
extern u32 via_chrome9_write_vga_io(u16 io_name, u8 data);
extern u32 via_chrome9_write_vga_io_bits(u16 io_name, u8 data, u8 bits_mask);
extern u32 via_chrome9_write_vga_io_multi_bits(
		struct via_chrome9_io_reg reg_table[], u32 item_num);
extern void via_chrome9_load_regs(u32 reg_value, u32 reg_amount,
		struct via_chrome9_io_bit *p_regbit);
extern void via_chrome9_mem_set(void *p_start,  u32 length_in_bytes);
extern void via_chrome9_delay_usec(struct drm_device *dev, u32 usec);
extern bool via_chrome9_delay_msec(struct drm_device *dev, u32 counter);
extern int via_chrome9_calculate_tu(int link_speed, unsigned long pixel_clock,
		int bpp_byte);
extern void via_chrome9_patch_dot_clock(struct drm_encoder *encoder,
		struct drm_display_mode *mode);
extern bool via_chrome9_is_valid_mode(u16 hdisplay, u16 vdisplay);
#endif
