/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "drmP.h"
#include "via_chrome9_drm.h"
#include "via_chrome9_drv.h"
#include "via_chrome9_dma.h"
#include "via_chrome9_mm.h"
#include "via_chrome9_3d_reg.h"
#include "via_chrome9_init.h"
#include "via_chrome9_object.h"
#include <linux/console.h>

#define RING_BUFFER_INIT_FLAG 1
#define RING_BUFFER_CLEANUP_FLAG 2

int via_chrome9_drm_get_pci_id(struct drm_device *dev,
		void *data, struct drm_file *file_priv)
{
	unsigned int *reg_val = data;
	outl(0x8000002C, 0xCF8);
	*reg_val = inl(0xCFC);
	outl(0x8000012C, 0xCF8);
	*(reg_val + 1) = inl(0xCFC);
	outl(0x8000022C, 0xCF8);
	*(reg_val + 2) = inl(0xCFC);
	outl(0x8000052C, 0xCF8);
	*(reg_val + 3) = inl(0xCFC);

	return 0;
}

int via_chrome9_dma_init(struct drm_device *dev, void *data,
		struct drm_file *file_priv)
{
	return 0;
}

struct drm_ioctl_desc via_chrome9_ioctls[] = {
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_CREATE,
		via_chrome9_ioctl_gem_create, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_USER_CREATE,
		via_chrome9_ioctl_gem_user_create, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_MMAP,
		via_chrome9_ioctl_gem_mmap, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_PREAD,
		via_chrome9_ioctl_gem_pread, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_PWRITE,
		via_chrome9_ioctl_gem_pwrite, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_SET_DOMAIN,
		via_chrome9_ioctl_gem_set_domain, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_FLUSH,
		via_chrome9_ioctl_gem_flush, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_WAIT,
		via_chrome9_ioctl_gem_wait, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_SHADOW_INIT,
		via_chrome9_ioctl_shadow_init, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_ALLOCATE_EVENT_TAG,
		via_chrome9_ioctl_alloc_event, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_FREE_EVENT_TAG,
		via_chrome9_ioctl_free_event, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_WAIT_CHIP_IDLE,
		via_chrome9_ioctl_wait_chip_idle, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_CPU_GRAB,
		via_chrome9_ioctl_cpu_grab, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_CPU_RELEASE,
		via_chrome9_ioctl_cpu_release, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_PIN,
		via_chrome9_ioctl_gem_pin, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_UNPIN,
		via_chrome9_ioctl_gem_unpin, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GEM_CHIPINFO,
		via_chrome9_ioctl_chipinfo_gem, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_XORG_OPTIONS,
		via_chrome9_ioctl_xorg_options, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GET_CRTC_ID,
		via_chrome9_ioctl_get_crtc_id, DRM_UNLOCKED|DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_GET_BO_INFO,
	    via_chrome9_ioctl_get_bo_info, DRM_AUTH),
	DRM_IOCTL_DEF_DRV(VIA_CHROME9_SET_EDID,
		via_chrome9_ioctl_set_edid, DRM_UNLOCKED|DRM_AUTH)
};

int via_chrome9_max_ioctl = DRM_ARRAY_SIZE(via_chrome9_ioctls);

DEFINE_PCI_DEVICE_TABLE(pciidlist) = {
	{0x1106, VX800_DEVICE, PCI_ANY_ID, PCI_ANY_ID, 0, 0,
		VIA_CHROME9_PCIE_GROUP},
	{0x1106, VX855_DEVICE, PCI_ANY_ID, PCI_ANY_ID, 0, 0,
		VIA_CHROME9_PCIE_GROUP},
	{0x1106, VX900_DEVICE, PCI_ANY_ID, PCI_ANY_ID, 0, 0,
		VIA_CHROME9_PCIE_GROUP},
	{0, 0, 0}
};


#if VIA_CHROME9_ENABLE_KMS
static struct drm_driver driver;

static int __devinit via_chrome9_pci_probe(struct pci_dev *pdev,
		const struct pci_device_id *ent)
{
	return drm_get_pci_dev(pdev, ent, &driver);
}

static void via_chrome9_pci_remove(struct pci_dev *pdev)
{
	struct drm_device *dev = pci_get_drvdata(pdev);
	drm_put_dev(dev);
}

static void via_chrome9_save_reg(struct drm_device *dev)
{
	int i;
	unsigned char reg_tmp;
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	for (i = 0; i < 10; i++) {
		dev_priv->pm_backup.gti_backup[i] =
			via_chrome9_read_vga_io(0x166 + i);

	}

	/* Save registers from SR79~SR7B */
	for (i = 0; i < 3; i++) {
		dev_priv->pm_backup.gti_backup[10 + i] =
			via_chrome9_read_vga_io(0x179 + i);
	}

	/*save engine clk form SR47~SR49*/
	for (i = 0; i < 3; i++) {
		dev_priv->pm_backup.eng_clk_backup[i] =
			via_chrome9_read_vga_io(0x147 + i);
	}

	/* Patch for PCIE series S3 hang issue inspired by BIOS team:
	 * Close IGA channel and clear FIFO depth
	 */
	/* Disable first display channel */
	reg_tmp = via_chrome9_read_vga_io(0x17);
	via_chrome9_write_vga_io(0x17, reg_tmp & 0x7f);


	/* Disable second display channel */
	reg_tmp = via_chrome9_read_vga_io(0x6A);
	via_chrome9_write_vga_io(0x6A, reg_tmp & 0x7f);


	/* First screen off */
	reg_tmp = via_chrome9_read_vga_io(0x101);
	via_chrome9_write_vga_io(0x101, reg_tmp | 0x20);


	/* Second screen off */
	reg_tmp = via_chrome9_read_vga_io(0x6B);
	via_chrome9_write_vga_io(0x6B, (reg_tmp | 0x04) & (~0x02));


	/* Clear IGA1 FIFO depth
	 * 3C5.17[7:0]=0
	 */
	via_chrome9_write_vga_io(0x117, 0x0);


	/* Clear IGA2 FIFO depth
	 * 3x5.68[7:4]=0, 3x5.94[7]=0, 3x5.95[7]=0
	 */
	reg_tmp = via_chrome9_read_vga_io(0x68);
	via_chrome9_write_vga_io(0x68, reg_tmp & 0x0f);


	reg_tmp = via_chrome9_read_vga_io(0x94);
	via_chrome9_write_vga_io(0x94, reg_tmp & 0x7f);


	reg_tmp = via_chrome9_read_vga_io(0x95);
	via_chrome9_write_vga_io(0x95, reg_tmp & 0x7f);
}

static void via_chrome9_load_default_cr_sr(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	int i;
	unsigned char val;

	/*enable mmio*/
	val = STANDVGA_R8(0x3c3);
	STANDVGA_W8(0x3c3, val | 0x01);
	val = STANDVGA_R8(VGA_MISC_OUT_R);
	STANDVGA_W8(VGA_MISC_OUT_W, val | 0x01);

	/*restore regs*/
	via_chrome9_write_vga_io_bits(REG_SR10, 0x01, BIT0);
	if (dev_priv->pseudo_chip_revision >= REVISION_VX855_A1)
		via_chrome9_write_vga_io_bits(REG_CR47, 0, BIT4);
	else
		via_chrome9_write_vga_io_bits(REG_CR47, 0, BIT0);

	via_chrome9_init_vq(dev);

	via_chrome9_write_vga_io_bits(REG_CR6A, 0x00, BIT6);
	via_chrome9_write_vga_io_bits(REG_CR6A, 0x00, BIT7);
	via_chrome9_write_vga_io_bits(REG_CR6A, 0x40, BIT6);
	via_chrome9_write_vga_io_bits(REG_CR6A, 0x00, 0x3F);

	via_chrome9_write_vga_io_bits(REG_CR6B, 0x00, 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR6C, 0x00, 0xFF);

	/* Gamma must disable before restore pallette */
	via_chrome9_write_vga_io_bits(REG_CR33, 0x00, BIT7);

	/* restore the standard vga regs */
	via_chrome9_restore_vga(dev);

	/* restore Sequencer regs: From SR14 to SR3F  */
	for (i = 0x14; i < 0x40; i++)
		via_chrome9_write_vga_io_bits(i+REG_SR00,
			dev_priv->pm_backup.sr_regs[i], 0xFF);

	via_chrome9_write_vga_io_bits(REG_SR42,
		dev_priv->pm_backup.sr_regs[0x42], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR4F,
		dev_priv->pm_backup.sr_regs[0x4F], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR50,
		dev_priv->pm_backup.sr_regs[0x50], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR57,
		dev_priv->pm_backup.sr_regs[0x57], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR5D,
		dev_priv->pm_backup.sr_regs[0x5D], 0xFF);

	/* DVP1 Clock and Data Pads Driving: */
	via_chrome9_write_vga_io_bits(REG_SR65,
	dev_priv->pm_backup.sr_regs[0x65], 0xFF);

	/*=* restore VCK, LCDCK and ECK *=*/
	/* VCK: */
	via_chrome9_write_vga_io_bits(REG_SR44,
		dev_priv->pm_backup.sr_regs[0x44], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR45,
		dev_priv->pm_backup.sr_regs[0x45], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR46,
		dev_priv->pm_backup.sr_regs[0x46], 0xFF);
	/* Reset VCK PLL */
	via_chrome9_write_vga_io_bits(REG_SR40, 0x02, BIT1);
	via_chrome9_write_vga_io_bits(REG_SR40, 0x00, BIT1);

	/* LCK: */
	via_chrome9_write_vga_io_bits(REG_SR4A,
		dev_priv->pm_backup.sr_regs[0x4A], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR4B,
		dev_priv->pm_backup.sr_regs[0x4B], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR4C,
		dev_priv->pm_backup.sr_regs[0x4C], 0xFF);
	/* Reset LCK PLL */
	via_chrome9_write_vga_io_bits(REG_SR40, 0x04, BIT2);
	via_chrome9_write_vga_io_bits(REG_SR40, 0x00, BIT2);

	/* ECK: */
	via_chrome9_write_vga_io_bits(REG_SR47,
		dev_priv->pm_backup.sr_regs[0x47], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR48,
		dev_priv->pm_backup.sr_regs[0x48], 0xFF);
	via_chrome9_write_vga_io_bits(REG_SR49,
		dev_priv->pm_backup.sr_regs[0x49], 0xFF);
	/* Reset ECK PLL */
	via_chrome9_write_vga_io_bits(REG_SR40, 0x01, BIT0);
	via_chrome9_write_vga_io_bits(REG_SR40, 0x00, BIT0);

	/* Restore Preemptive Arbiter Control Register */
	if (dev->pci_device == VX900_DEVICE) {
		via_chrome9_write_vga_io_bits(REG_SR4D,
			dev_priv->pm_backup.sr_regs[0x4D], 0xFF);
	}

	/* Restore CRTC controller extended regs: */
	via_chrome9_write_vga_io_bits(REG_CR08,
		dev_priv->pm_backup.cr_regs[0x08], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR09,
		dev_priv->pm_backup.cr_regs[0x09], 0xFF);

	via_chrome9_write_vga_io_bits(REG_CR13,
		dev_priv->pm_backup.cr_regs[0x13], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR30,
		dev_priv->pm_backup.cr_regs[0x30], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR31,
		dev_priv->pm_backup.cr_regs[0x31], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR32,
		dev_priv->pm_backup.cr_regs[0x32], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR33,
		dev_priv->pm_backup.cr_regs[0x33], 0xFF);

	/* From CR35 to CR47 */
	i = 0x35;
	while (1) {
		via_chrome9_write_vga_io_bits(i + REG_CR00,
			dev_priv->pm_backup.cr_regs[i], 0xFF);
		if (i == 0x47)
			break;
		else
			i++;
	}

	/* Starting Address: */
	via_chrome9_write_vga_io_bits(REG_CR0C,
		dev_priv->pm_backup.cr_regs[0x0C], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR0D,
		dev_priv->pm_backup.cr_regs[0x0D], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR48,
		dev_priv->pm_backup.cr_regs[0x48], 0xFF);
	via_chrome9_write_vga_io_bits(REG_CR34,
		dev_priv->pm_backup.cr_regs[0x34], 0xFF);

	/* Before restore scaling setting, disable scaling first.*/
	via_chrome9_write_vga_io_bits(REG_CRFD,
		dev_priv->pm_backup.cr_regs[0xfd], 0xFF);

	/* From CR49 to CRFF */
	for (i = 0x49; i < 0x100; i++) {
		if (i == 0x91) {
			/*
			LCD Panel could use software power sequence,
			so we should resotore it
			individually to avoid incorrect enable LCD.
			*/
		} else if (i == 0xfd) {
			/*
			CR_FD will affect some scaling registers.
			Ex: It makes CR_89[0] to become 1 from 0
			then make some bugs in DuoView.
			Hence It should be writen before these registers.
			*/
		} else if ((i == 0x67) && (VX800_DEVICE == dev->pci_device)) {
			/*
			when light external card and acpi will hang
			(i2c 2c is enabled and restore this reg.).
			if we restore this reg and disable i2c 2c port.
			system is ok
			*/
		} else {
			via_chrome9_write_vga_io_bits(i + REG_CR00,
				dev_priv->pm_backup.cr_regs[i], 0xFF);
		}
	}

	/* solve the lvds interrupt problem */
	via_chrome9_write_vga_io_bits(REG_SR40, BIT3, BIT3);

	/* Restore CR91 after lvds restore*/
	via_chrome9_write_vga_io_bits(REG_CR91,
		dev_priv->pm_backup.cr_regs[0x91], 0xFF);
	/* Reset clock */
	via_chrome9_write_misc_io(via_chrome9_read_misc_io());
}

static void via_chrome9_restore_reg(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	unsigned int i;

	setmmioregister(dev_priv->mmio_map, INV_REG_CR_TRANS, 0x00110000);
	setmmioregister(dev_priv->mmio_map, INV_REG_CR_BEGIN,
		0x06000000);
	setmmioregister(dev_priv->mmio_map, INV_REG_CR_BEGIN,
		0x07100000);
	setmmioregister(dev_priv->mmio_map, INV_REG_CR_TRANS,
		INV_PARATYPE_PRECR);
	setmmioregister(dev_priv->mmio_map, INV_REG_CR_BEGIN,
		INV_SUBA_HSETRBGID | INV_HSETRBGID_CR);

	/* Here restore SR66~SR6F SR79~SR7B */
	for (i = 0; i < 10; i++)
		via_chrome9_write_vga_io(0x166 + i,
			dev_priv->pm_backup.gti_backup[i]);

	for (i = 0; i < 3; i++)
		via_chrome9_write_vga_io(0x179 + i,
			dev_priv->pm_backup.gti_backup[10 + i]);

	for (i = 0; i < 3; i++)
		via_chrome9_write_vga_io(0x147 + i,
			dev_priv->pm_backup.eng_clk_backup[i]);

	via_chrome9_load_default_cr_sr(dev);
}
static int via_chrome9_pci_suspend(struct pci_dev *pdev, pm_message_t state)
{
	struct drm_device *dev = pci_get_drvdata(pdev);
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	if (dev == NULL || dev->dev_private == NULL)
		return -ENODEV;

	/* wait all engine idle */
	waitchipidle_inv(p_priv);

	/*disable vq*/
	via_chrome9_vq_disable(dev);

	if (state.event == PM_EVENT_PRETHAW)
		return 0;

	via_chrome9_drm_save_bos(dev);
	via_chrome9_suspend_gart_table(dev);

	pci_save_state(pdev);

	via_chrome9_save_reg(dev);
	via_chrome9_suspend_exec(dev);
	via_chrome9_suspend_mode_set(dev);

	/* for vx800 nano platform, when S1, system can't back */
	if (dev->pci_device != VX800_DEVICE) {
		console_lock();
		fb_set_suspend(p_priv->fb_info, 1);
		console_unlock();
	}
	/* patch for VX900 S1 */
#if 0
	if (state.event == PM_EVENT_SUSPEND) {
		pci_disable_device(pdev);
		pci_set_power_state(pdev, PCI_D3hot);
	}
#endif
	return 0;
}

static int via_chrome9_pci_resume(struct pci_dev *pdev)
{
	struct drm_device *dev = pci_get_drvdata(pdev);
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	console_lock();
	pci_set_power_state(pdev, PCI_D0);
	pci_restore_state(pdev);
	if (pci_enable_device(pdev)) {
		console_unlock();
		return -1;
	}
	pci_set_master(pdev);

	via_chrome9_restore_reg(dev);

	if (dev->pci_device != VX800_DEVICE)
		fb_set_suspend(p_priv->fb_info, 0);

	console_unlock();

	via_chrome9_resume_mode_set(dev);
	via_chrome9_resume_exec(dev);

	return 0;
}
#endif

static struct drm_driver driver = {
	.driver_features =
		DRIVER_HAVE_DMA | DRIVER_FB_DMA | DRIVER_USE_MTRR | DRIVER_GEM |
		DRIVER_HAVE_IRQ | DRIVER_IRQ_SHARED,
	.load = via_chrome9_driver_load,
	.preclose = via_chrome9_preclose,
	.lastclose = via_chrome9_lastclose,
	.unload = via_chrome9_driver_unload,
	.suspend = via_chrome9_drm_suspend,
	.resume = via_chrome9_drm_resume,
	.irq_preinstall = via_chrome9_driver_irq_preinstall,
	.irq_postinstall = via_chrome9_driver_irq_postinstall,
	.irq_uninstall = via_chrome9_driver_irq_uninstall,
	.irq_handler = via_chrome9_driver_irq_handler,
	.get_vblank_counter = drm_vblank_count,
	.enable_vblank = via_chrome9_enable_vblank,
	.disable_vblank = via_chrome9_disable_vblank,
	.reclaim_buffers = drm_core_reclaim_buffers,
	.gem_init_object = via_chrome9_gem_object_init,
	.gem_free_object = via_chrome9_gem_object_free,
	.ioctls = via_chrome9_ioctls,
	.fops = {
		 .owner = THIS_MODULE,
		 .open = drm_open,
		 .release = drm_release,
		 .unlocked_ioctl = drm_ioctl,
		 .mmap = via_chrome9_mmap,
		 .poll = drm_poll,
		 .fasync = drm_fasync,
		 .read = drm_read,
		 .llseek = noop_llseek,
#ifdef CONFIG_COMPAT
		 .compat_ioctl = via_chrome9_compat_ioctl,
#endif
	},
	.pci_driver = {
		 .name = DRIVER_NAME,
		 .id_table = pciidlist,
#if VIA_CHROME9_ENABLE_KMS
		 .probe = via_chrome9_pci_probe,
		 .remove = via_chrome9_pci_remove,
		 .suspend = via_chrome9_pci_suspend,
		 .resume = via_chrome9_pci_resume,
#endif
	},

	.name = DRIVER_NAME,
	.desc = DRIVER_DESC,
	.date = DRIVER_DATE,
	.major = DRIVER_MAJOR,
	.minor = DRIVER_MINOR,
	.patchlevel = DRIVER_PATCHLEVEL,
};

static int __init via_chrome9_init(void)
{
#if VIA_CHROME9_ENABLE_KMS
	driver.driver_features |= DRIVER_MODESET;
#endif
	driver.num_ioctls = via_chrome9_max_ioctl;
	return drm_init(&driver);
}

static void __exit via_chrome9_exit(void)
{
	drm_exit(&driver);
}

module_init(via_chrome9_init);
module_exit(via_chrome9_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL and additional rights");
