/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "via_chrome9_drv.h"
#include "via_chrome9_display_common.h"
#include "via_chrome9_hdac_patch.h"

static struct internal_encoder_list via_chrome9_internal_encoder_list[] = {
	{ VIA_CHROME9_DEVICE_ENCODER_DAC,
		VX800_INDEX | VX855_INDEX | VX900_INDEX, "DAC" },
	{ VIA_CHROME9_DEVICE_ENCODER_LVDS,
		VX800_INDEX | VX855_INDEX | VX900_INDEX, "LVDS" },
	{ VIA_CHROME9_DEVICE_ENCODER_HDMI, VX900_INDEX, "HDMI" },
	{ VIA_CHROME9_DEVICE_ENCODER_DP_1, VX900_INDEX, "DP_1" },
	{ VIA_CHROME9_DEVICE_ENCODER_DP_2, VX900_INDEX, "DP_2" },
	{ VIA_CHROME9_DEVICE_ENCODER_TMDS, VX800_INDEX, "TMDS" },
	{ VIA_CHROME9_DEVICE_ENCODER_LVDS_2, VX800_INDEX, "LVDS" },
	{ VIA_CHROME9_DEVICE_ENCODER_TV, 0, "TV" }
};

const int via_chrome9_encoder_convert[] = {
	DRM_MODE_ENCODER_NONE,
	/* VIA_CHROME9_ENCODER_DAC_INDEX */
	DRM_MODE_ENCODER_DAC,
	/* VIA_CHROME9_ENCODER_LVDS_INDEX */
	DRM_MODE_ENCODER_LVDS,
	/* VIA_CHROME9_ENCODER_TMDS_INDEX */
	DRM_MODE_ENCODER_TMDS,
	/* VIA_CHROME9_ENCODER_DP_1_INDEX */
	DRM_MODE_ENCODER_TMDS,
	/* VIA_CHROME9_ENCODER_DP_2_INDEX */
	DRM_MODE_ENCODER_TMDS,
	/* VIA_CHROME9_ENCODER_TV_INDEX */
	DRM_MODE_ENCODER_TVDAC,
	/* VIA_CHROME9_ENCODER_HDMI_INDEX */
	DRM_MODE_ENCODER_TMDS,
	/* VIA_CHROME9_ENCODER_LVDS_2_INDEX */
	DRM_MODE_ENCODER_LVDS,
	DRM_MODE_ENCODER_NONE,
};

struct display_timing_table via_lcd_modes[] = {
	/* Panel 480x640 (Special) */
	{ VIA_480X640, REFRESH_60, CLK_22_000M, NEGATIVE, POSITIVE,
	{ 520, 480, 480, 40, 504, 8, 648, 640, 640, 8, 644, 2 } },

	/* Panel 640x480 (DMT) */
	{ VIA_640X480, REFRESH_60, CLK_25_175M, NEGATIVE, NEGATIVE,
	{ 800, 640, 640, 160, 656, 96, 525, 480, 480, 45, 490, 2 } },

	/* The LCD can't accept the VESA's 640x480 timing, because
	*  The VESA's 640x480 timing has border and LCD doesn't have.
	*  Modify hbs and hbe to eliminate the border.
	*/

	/* Panel 800x480 (CVT) */
	{ VIA_800X480, REFRESH_60, CLK_29_581M, NEGATIVE, POSITIVE,
	{ 992, 800, 800, 192, 824, 72, 500, 480, 480, 20, 483, 7 } },

	/* Panel 800x600 (DMT) */
	{ VIA_800X600, REFRESH_60, CLK_40_000M, POSITIVE, POSITIVE,
	{ 1056, 800, 800, 256, 840, 128, 628, 600, 600, 28, 601, 4 } },

	/* Panel 1024x600 */
	{ VIA_1024X600, REFRESH_60, CLK_48_875M, NEGATIVE, POSITIVE,
	{ 1312, 1024, 1024, 288, 1064, 104, 622, 600, 600, 22, 601, 3 } },

	/* Panel 1024x768 (CVT Reduce) */
	{ VIA_1024X768, REFRESH_60, CLK_56_250M, POSITIVE, NEGATIVE,
	{ 1184, 1024, 1024, 160, 1072, 32, 790, 768, 768, 22, 771, 4 } },

	/* Panel 1280x768 (GTF) */
	{ VIA_1280X768, REFRESH_60, CLK_80_136M, NEGATIVE, POSITIVE,
	{ 1680, 1280, 1280, 400, 1344, 136, 795, 768, 768, 27, 769, 3 } },

	/* Panel 1280x800 (CVT Reduce) */
	{ VIA_1280X800, REFRESH_60, CLK_72_000M, POSITIVE, NEGATIVE,
	{ 1440, 1280, 1280, 160, 1328, 32, 823, 800, 800, 23, 803, 6 } },

	/* Panel 1280x1024 (DMT) */
	{ VIA_1280X1024, REFRESH_60, CLK_108_000M, POSITIVE, POSITIVE,
	{ 1688, 1280, 1280, 408, 1328, 112, 1066, 1024, 1024, 42, 1025, 3 } },

	/* Panel 1366x768 (GTF) */
	{ VIA_1366X768, REFRESH_60, CLK_85_860M, NEGATIVE, POSITIVE,
	{ 1800, 1368, 1368, 432, 1440, 144, 795, 768, 768, 27, 769, 3 } },

	/* Panel 1360x768 (CVT) */
	{ VIA_1360X768, REFRESH_60, CLK_84_750M, POSITIVE, POSITIVE,
	{ 1776, 1360, 1360, 416, 1432, 136, 798, 768, 768, 30, 771, 5 } },

	/* Panel 1400x1050 (CVT Reduce) */
	{ VIA_1400X1050, REFRESH_60, CLK_101_000M, POSITIVE, NEGATIVE,
	{ 1560, 1400, 1400, 160, 1448, 32, 1080, 1050, 1050, 30, 1053, 4 } },

	/* Panel 1440x900*/
	{ VIA_1440X900, REFRESH_60, CLK_88_750M, POSITIVE, NEGATIVE,
	{ 1760, 1440, 1440, 320, 1495, 105, 912, 900, 900, 12, 901, 2 } },

	/* Panel 16x12 (CVT Reduce)*/
	{ VIA_1600X1200, REFRESH_60, CLK_130_250M, POSITIVE, NEGATIVE,
	{ 1760, 1600, 1600, 160, 1648, 32, 1235, 1200, 1200, 35, 1203, 4 } },

	/* AUO Panel 1920x1080 */
	{ VIA_1920X1080, REFRESH_60, CLK_150_340M, POSITIVE, POSITIVE,
	{ 2120, 1920, 1920, 200, 1951, 48, 1120, 1080, 1080, 40, 1082, 6 } }
};

#define   NUM_VIA_LCD_TIMING_TBL   CMDISP_ARRAYSIZE(via_lcd_modes)

/* Sync CBIOS's CEA timing table at 20090327 */
/* The table include whole CEA861.D's mode, but we only open some familiar
   modes. If we want to open the others mode, we just need to set the "Use"
   field as true. */
struct via_cea_timing_info_rec via_cea_timing_table[] = {
	/* ID H_Res V_Res  Refresh  Inter/Prog  VSynOff  Ratio    Use */
	{  1,   640,   480,   60,   PROGRESSIVE,     0,   RATIO_4_3, true },
	{  2,   720,   480,   60,   PROGRESSIVE,     0,   RATIO_4_3, true },
	{  3,   720,   480,   60,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{  4,  1280,  720,   60,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{  5,  1920, 1080,   60,  INTERLACE,      1100,  RATIO_16_9, true },
	{  6,  720,   480,    60,   INTERLACE,        22,   RATIO_4_3, false },
	{  7,  720,  480,     60,   INTERLACE,        22,   RATIO_16_9, false },
	{  8,   720,   240,   60,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{  9,   720,   240,   60,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 10,  2880,  480,   60,   INTERLACE,       22,   RATIO_4_3, false },
	{ 11,  2880,  480,   60,   INTERLACE,       22,   RATIO_16_9, false },
	{ 12,  2880,  240,   60,   PROGRESSIVE,     0,  RATIO_4_3, false },
	{ 13,  2880,  240,   60,   PROGRESSIVE,     0,  RATIO_16_9, false },
	{ 14,  1440,  480,   60,   PROGRESSIVE,     0,  RATIO_4_3, false },
	{ 15,  1440,  480,   60,   PROGRESSIVE,     0,  RATIO_16_9, false },
	{ 16,  1920, 1080,  60,   PROGRESSIVE,     0,  RATIO_16_9, true },
	{ 17,   720,   576,  50,    PROGRESSIVE,     0,  RATIO_4_3, true },
	{ 18,   720,   576,  50,    PROGRESSIVE,     0,  RATIO_16_9, true },
	{ 19,  1280,   720,  50,   PROGRESSIVE,     0,  RATIO_16_9, true },
	{ 20,  1920,  1080,  50,  INTERLACE,     1320,  RATIO_16_9, true },
	{ 21,   720,    576,  50,   INTERLACE,      864,   RATIO_4_3, false },
	{ 22,   720,    576,  50,   INTERLACE,      864,   RATIO_16_9, false },
	{ 23,   720,    288,  50,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 24,   720,    288,  50,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 25,  2880,   576,  50,   INTERLACE,     1728,   RATIO_4_3, false },
	{ 26,  2880,   576,  50,   INTERLACE,     1728,   RATIO_16_9, false },
	{ 27,  2880,   288,  50,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 28,  2880,   288,  50,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 29,  1440,   576,  50,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 30,  1440,   576,  50,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 31,  1920,  1080, 50,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{ 32,  1920,  1080, 24,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{ 33,  1920,  1080, 25,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{ 34,  1920,  1080, 30,   PROGRESSIVE,     0,   RATIO_16_9, true },
	{ 35,  2880,   480,  60,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 36,  2880,   480,  60,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 37,  2880,   576,  50,   PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 38,  2880,   576,  50,   PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 39,  1920,  1080, 50,   INTERLACE,     1152,   RATIO_16_9, true },
	{ 40,  1920,  1080, 100,  INTERLACE,     1320,  RATIO_16_9, false },
	{ 41,  1280,  720,   100,  PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 42,   720,   576,   100,  PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 43,   720,   576,   100,  PROGRESSIVE,   0,    RATIO_16_9, false },
	{ 44,   720,   576,   100,  INTERLACE,      864,   RATIO_16_9, false },
	{ 45,   720,   576,   100,  INTERLACE,      864,   RATIO_4_3, false },
	{ 46,  1920,  1080, 120,  INTERLACE,     1100,   RATIO_16_9, false },
	{ 47,  1280,   720,  120,  PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 48,   720,   480,   120,  PROGRESSIVE,     0,   RATIO_4_3, false },
	{ 49,   720,   480,   120,  PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 50,   720,   480,   120,  INTERLACE,     1134,   RATIO_4_3, false },
	{ 51,   720,   480,   120,  INTERLACE,     1134,   RATIO_16_9, false },
	{ 52,   720,   576,   200,  PROGRESSIVE,     0,    RATIO_4_3, false },
	{ 53,   720,   576,   200,  PROGRESSIVE,     0,   RATIO_16_9, false },
	{ 54,   720,   576,   200,  INTERLACE,       864,   RATIO_4_3, false },
	{ 55,   720,  576,   200,   INTERLACE,       864,   RATIO_16_9, false },
	{ 56,   720,   480,   240,  PROGRESSIVE,     0,    RATIO_4_3, false },
	{ 57,   720,   480,   240,  PROGRESSIVE,     0,    RATIO_16_9, false },
	{ 58,   720,   480,   240,  INTERLACE,     1134,   RATIO_4_3, false },
	{ 59,   720,   480,   240,  INTERLACE,     1134,   RATIO_16_9, false },
};

struct via_dot_clock via_dot_clocks[] = {
	/*               PLL3      PLL3_409       */
	{ DCLK_25_200M, 0x8B1003, 0x8D1005 },
	{ DCLK_27_000M, 0x951003, 0x971005 },
	{ DCLK_54_000M, 0x950C03, 0x970C05 },
	{ DCLK_72_000M, 0x9F0C02, 0xA10C04 },
	{ DCLK_74_250M, 0xA40C02, 0xA60C04 },
	{ DCLK_108_000M, 0x950803, 0x970805 },
	{ DCLK_148_500M, 0xA40802, 0xA60804 },
	{      0,      0,        0 }
};
void via_chrome9_patch_dot_clock(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct drm_crtc *crtc = encoder->crtc;
	struct drm_device *dev = encoder->dev;
	int i;
	u32  patch_pll = 0;

	switch (dev->pci_device) {
	case VX855_DEVICE:
	case VX900_DEVICE:
		for (i = 0; via_dot_clocks[i].dot_clock; i++) {
			if (via_dot_clocks[i].dot_clock == mode->clock) {
				patch_pll = via_dot_clocks[i].pll3_409;
				break;
			}
		}
		break;

	default:
		for (i = 0; via_dot_clocks[i].dot_clock; i++) {
			if (via_dot_clocks[i].dot_clock == mode->clock) {
				patch_pll = via_dot_clocks[i].pll3;
				break;
			}
		}
		break;
	}

	via_chrome9_set_vclk(crtc, patch_pll);
}
void via_chrome9_digital_port_pad_on(u32 di_port, int iga_path, int chipset)
{
	/* Set Iga source and turn on DI port clk */
	switch (di_port) {
	case DISP_DI_DVP0:
		if (IGA1 == iga_path)
			/* DVP0 Data Source Selection. */
			via_chrome9_write_vga_io_bits(REG_CR96, 0x00, BIT4);
		else
			/* DVP0 Data Source Selection. */
			via_chrome9_write_vga_io_bits(REG_CR96, 0x10, BIT4);
		/* Turn on DVP0 clk */
		via_chrome9_write_vga_io_bits(REG_SR1E, 0xC0, BIT6 + BIT7);
		break;
	case DISP_DI_DVP1:
		KMS_DEBUG("DVP1 ON, IGA PATH IS = %d!\n", iga_path);
		if (IGA1 == iga_path)
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x00, BIT4);
		else
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x10, BIT4);

		/* enable dvp1 under this chipset */
		if ((VX800_DEVICE == chipset) ||
		(VX855_DEVICE == chipset) ||
		(VX900_DEVICE == chipset))
			via_chrome9_write_vga_io_bits(REG_CRD3, 0x00, BIT5);
		/* Turn on DVP1 clk */
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x30, BIT4 + BIT5);
		break;

	case DISP_DI_DFPH:
		if (iga_path == IGA1) {
			/* 3324,3353 CR96[4] control DVP0 */
			if ((VX800_DEVICE != chipset) &&
				(VX855_DEVICE != chipset) &&
				(VX900_DEVICE != chipset))
				via_chrome9_write_vga_io_bits(REG_CR96,
					0x00, BIT4);
				via_chrome9_write_vga_io_bits(REG_CR97,
					0x00, BIT4);
		} else {
			/* 3353 CR96[4] control DVP0 */
			if ((VX800_DEVICE != chipset) &&
				(VX855_DEVICE != chipset) &&
				(VX900_DEVICE != chipset))
				via_chrome9_write_vga_io_bits(REG_CR96,
					0x10, BIT4);
				via_chrome9_write_vga_io_bits(REG_CR97,
					0x10, BIT4);
		}

		/* Turn on DFPH clk */
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x0C, BIT2 + BIT3);
		break;

	case DISP_DI_DFPL:
		if (IGA1 == iga_path) {
			/* 3353 CR9B[4] control DVP1 */
			if ((VX800_DEVICE != chipset) &&
				(VX855_DEVICE != chipset) &&
				(VX900_DEVICE != chipset))
				via_chrome9_write_vga_io_bits(REG_CR9B,
					0x00, BIT4);
				via_chrome9_write_vga_io_bits(REG_CR99,
					0x00, BIT4);
		} else {
			/* 3353 CR9B[4] control DVP1 */
			if ((VX800_DEVICE != chipset) &&
				(VX855_DEVICE != chipset) &&
				(VX900_DEVICE != chipset))
				via_chrome9_write_vga_io_bits(REG_CR9B,
					0x10, BIT4);
				via_chrome9_write_vga_io_bits(REG_CR99,
					0x10, BIT4);
		}

		/* Turn on DFPL clk */
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x03, BIT1 + BIT0);
		break;

	case DISP_DI_DFP:
		if (IGA1 == iga_path) {
			via_chrome9_write_vga_io_bits(REG_CR97, 0x00, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR99, 0x00, BIT4);
		} else {
			via_chrome9_write_vga_io_bits(REG_CR97, 0x10, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR99, 0x10, BIT4);
		}

		/* Turn on DFP clk */
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x0F,
			BIT3 + BIT2 + BIT1 + BIT0);
		break;

	/* For TTL Type LCD */
	case (DISP_DI_DFPL+DISP_DI_DVP1):
		if (IGA1 == iga_path) {
			via_chrome9_write_vga_io_bits(REG_CR99, 0x00, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x00, BIT4);
		} else {
			via_chrome9_write_vga_io_bits(REG_CR99, 0x10, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x10, BIT4);
		}

		/* Turn on DFPL, DVP1 clk */
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x03, BIT1 + BIT0);
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x30, BIT4 + BIT5);
		break;

	/* For 409 TTL Type LCD */
	case (DISP_DI_DFPH + DISP_DI_DFPL + DISP_DI_DVP1):
		if (IGA1 == iga_path) {
			via_chrome9_write_vga_io_bits(REG_CR97, 0x00, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR99, 0x00, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x00, BIT4);
		} else {
			via_chrome9_write_vga_io_bits(REG_CR97, 0x10, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR99, 0x10, BIT4);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x10, BIT4);
		}

		/* Turn on DFPHL, DVP1 clk */
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x0F,
			BIT3 + BIT2 + BIT1 + BIT0);
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x30, BIT4 + BIT5);
		break;

	default:
		KMS_DEBUG("via_chrome9_digital_output_pad_on: No DIPort.\n");
		break;

	}

	/* If Iga2 is used, enable Iga2 output for DI port.
	In order to enable Iga2, 3324,3353,3293 use CR91 and CRD3 to control
	DVP0 and DVP1 seperately,
	but other chip only use CR91 to control all Di port */
	if (IGA2 == iga_path)
		via_chrome9_write_vga_io_bits(REG_CR91, 0x00, BIT5);

}

void via_chrome9_digital_port_pad_off(u32 di_port)
{
	KMS_DEBUG("\n");
	switch (di_port) {
	case DISP_DI_DVP0:
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x00, BIT6 + BIT7);
		break;

	case DISP_DI_DVP1:
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x00, BIT4 + BIT5);
		break;

	case DISP_DI_DFPH:
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x00, BIT2 + BIT3);
		break;

	case DISP_DI_DFPL:
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x00, BIT0 + BIT1);
		break;

	case DISP_DI_DFP:
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x00,
			BIT2 + BIT3 + BIT1 + BIT0);
		break;

	/* TTL LCD,now used for Qunta case */
	case DISP_DI_DFPL+DISP_DI_DVP1:
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x00, BIT4 + BIT5);
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x00, BIT0 + BIT1);
		break;

	case DISP_DI_DFPH + DISP_DI_DFPL + DISP_DI_DVP1:
		via_chrome9_write_vga_io_bits(REG_SR1E, 0x00, BIT4 + BIT5);
		via_chrome9_write_vga_io_bits(REG_SR2A, 0x00,
			BIT3 + BIT2 + BIT0 + BIT1);
		break;

	case DISP_DI_DAC:
		break;

	default:
		KMS_DEBUG("via_chrome9_digital_port_pad_off: No DIPort.\n");
		break;
	}
}

void via_chrome9_set_digital_port_sync_polarity(u32 port,
		struct drm_display_mode *mode)
{
	u8 syncreg = 0;

	if (mode->flags & DRM_MODE_FLAG_NVSYNC)
		syncreg |= NEGATIVE << 1;
	if (mode->flags & DRM_MODE_FLAG_NHSYNC)
		syncreg |= NEGATIVE;

	switch (port) {
	case DISP_DI_DAC:
		via_chrome9_write_misc_io((via_chrome9_read_misc_io() &
			(~(BIT6 + BIT7))) | (syncreg << 6));
		break;
	case DISP_DI_DVP0:
		via_chrome9_write_vga_io_bits(REG_CR96, syncreg << 5,
			BIT6 | BIT5);
		break;
	case DISP_DI_DVP1:
		via_chrome9_write_vga_io_bits(REG_CR9B, syncreg << 5,
			BIT6 | BIT5);
		break;
	case DISP_DI_DFPH:
		via_chrome9_write_vga_io_bits(REG_CR97, syncreg << 5,
			BIT6 | BIT5);
		break;
	case DISP_DI_DFPL:
		via_chrome9_write_vga_io_bits(REG_CR99, syncreg << 5,
			BIT6 | BIT5);
		break;
	default:
		KMS_DEBUG(
			"via_chrome9_set_digital_port_sync_polarity: No DIPort.\n");
		break;
	}
}


void via_chrome9_enc_destroy(struct drm_encoder *encoder)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);

	/* disable patch timer */
	CTS_PATCH_OFF();

	kfree(via_chrome9_encoder->encoder_private);
	drm_encoder_cleanup(encoder);
	kfree(via_chrome9_encoder);
}

static void via_chrome9_encoder_disable(struct drm_encoder *encoder)
{
	struct drm_encoder_helper_funcs *encoder_funcs;

	encoder_funcs = encoder->helper_private;
	encoder_funcs->dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_dac_dpms(struct drm_encoder *encoder, int mode)
{
	struct via_chrome9_crtc *via_chrome9_crtc;
	int iga_path;
	DRM_DEBUG_KMS("\n");

	switch (mode) {
	case DRM_MODE_DPMS_ON:
	{
		if (encoder->crtc == NULL)
			return;
		via_chrome9_crtc = to_via_chrome9_crtc(encoder->crtc);
		iga_path = via_chrome9_crtc->crtc_id;

		if (IGA1 == iga_path)
			via_chrome9_write_vga_io_bits(REG_SR16, 0x00, BIT6);
		else
			via_chrome9_write_vga_io_bits(REG_SR16, 0x40, BIT6);

		via_chrome9_write_vga_io_bits(REG_CR36, 0x00, BIT4 + BIT5);
		break;
	}

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		via_chrome9_write_vga_io_bits(REG_CR36,
			BIT5 + BIT4, BIT5 + BIT4);
		/* Reset CRT source to IGA1 */
		via_chrome9_write_vga_io_bits(REG_SR16, 0x00, BIT6);
		break;
	}
}

static bool via_chrome9_dac_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	/* implement later if necessary */
	struct drm_crtc *crtc = encoder->crtc;
	struct via_chrome9_crtc *via_chrome9_crtc = to_via_chrome9_crtc(crtc);
	via_chrome9_crtc->crtc_scale_status = 0;
	return true;
}

static void via_chrome9_dac_prepare(struct drm_encoder *encoder)
{
	via_chrome9_dac_dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_dac_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	u8 syncreg = 0;

	if (adjusted_mode->flags & DRM_MODE_FLAG_NVSYNC)
		syncreg |= NEGATIVE << 1;
	if (adjusted_mode->flags & DRM_MODE_FLAG_NHSYNC)
		syncreg |= NEGATIVE;
	via_chrome9_write_misc_io((via_chrome9_read_misc_io() &
		(~(BIT6 + BIT7))) | (syncreg << 6));
}

static void via_chrome9_dac_commit(struct drm_encoder *encoder)
{
	via_chrome9_dac_dpms(encoder, DRM_MODE_DPMS_ON);
}

static enum drm_connector_status via_chrome9_dac_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	struct drm_device *dev = encoder->dev;
	enum drm_connector_status status = connector_status_disconnected;
	u8 regsr01, regcr36, regsr40;

	regsr01 = via_chrome9_read_vga_io(REG_SR01);
	regcr36 = via_chrome9_read_vga_io(REG_CR36);
	regsr40 = via_chrome9_read_vga_io(REG_SR40);
	/* Screen On */
	via_chrome9_write_vga_io_bits(REG_SR01, 0x0, BIT5);
	/* Power On DPMS */
	via_chrome9_write_vga_io_bits(REG_CR36, 0x0, BIT4 + BIT5 + BIT6 + BIT7);

	/* delay 16ms for Vertical Blank */
	via_chrome9_delay_msec(dev, 16);
	/* Enable CRT Sense */
	via_chrome9_write_vga_io_bits(REG_SR40, BIT7, BIT7);

	if ((VX800_DEVICE == dev->pci_device) ||
	(VX855_DEVICE == dev->pci_device) ||
	(VX900_DEVICE == dev->pci_device))
		via_chrome9_write_vga_io_bits(REG_SR40, 0, BIT7);

	/*
	324,353: SR40[7]=1 --> SR40[7] = 0 --> check 3C2[4]
	other: SR40[7]=1 --> check 3C2[4] --> SR40[7]=0
	*/
	if (STANDVGA_R8(0x3C2) & BIT4) {
		status = connector_status_connected;
		KMS_DEBUG(" connector_status_connected\n");
	} else {
		status = connector_status_disconnected;
		KMS_DEBUG(" connector_status_disconnected\n");
	}

	if ((VX800_DEVICE != dev->pci_device) &&
	(VX855_DEVICE != dev->pci_device) &&
	(VX900_DEVICE != dev->pci_device))
		via_chrome9_write_vga_io_bits(REG_SR40, 0, BIT7);

	/* Restore */
	via_chrome9_write_vga_io(REG_SR40, regsr40);
	via_chrome9_write_vga_io(REG_CR36, regcr36);
	via_chrome9_write_vga_io(REG_SR01, regsr01);

	return status;
}



static const struct drm_encoder_helper_funcs via_chrome9_dac_helper_funcs = {
	.dpms = via_chrome9_dac_dpms,
	.mode_fixup = via_chrome9_dac_mode_fixup,
	.prepare = via_chrome9_dac_prepare,
	.mode_set = via_chrome9_dac_mode_set,
	.commit = via_chrome9_dac_commit,
	.detect = via_chrome9_dac_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_dac_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

/* according to the width and height,get the built-in timing */
struct display_timing_table *via_chrome9_get_driver_timing(
		u32 dev_mode_index, u32 mode_index)
{
	struct display_timing_table *dev_mode_table = NULL;
	int i, array_num = 0;

	switch (dev_mode_index) {
	case LCD_MODE_TABLE:
		dev_mode_table = via_lcd_modes;
		array_num = NUM_VIA_LCD_TIMING_TBL;
		break;
	default:
		break;
	}

	if (dev_mode_table) {
		for (i = 0; i < array_num; i++) {
			if (dev_mode_table[i].mode_index == mode_index)
				return &dev_mode_table[i];
		}
	}
	return NULL;
}

bool via_chrome9_fill_mode_by_driver_timing(
		struct display_timing_table *timing_table,
		struct drm_display_mode *adjusted_mode)
{
	struct display_timing *crtc_timing = NULL;

	if (!timing_table)
		return false;

	crtc_timing = &timing_table->timing;
	/* Valuate mode from build-in timing */
	adjusted_mode->hdisplay = crtc_timing->hor_addr;
	adjusted_mode->hsync_start = crtc_timing->hor_sync_start;
	adjusted_mode->hsync_end = crtc_timing->hor_sync_start +
		crtc_timing->hor_sync_end;
	adjusted_mode->htotal = crtc_timing->hor_total;
	adjusted_mode->vdisplay = crtc_timing->ver_addr;
	adjusted_mode->vsync_start = crtc_timing->ver_sync_start;
	adjusted_mode->vsync_end = crtc_timing->ver_sync_start +
		crtc_timing->ver_sync_end;
	adjusted_mode->vtotal = crtc_timing->ver_total;
	adjusted_mode->clock = timing_table->clk/1000;
	return true;
}



/* caculate the cetering timing using mode and adjusted_mode */
void via_chrome9_centering_timing(struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	int panel_hsync_time = adjusted_mode->hsync_end -
		adjusted_mode->hsync_start;
	int panel_vsync_time = adjusted_mode->vsync_end -
		adjusted_mode->vsync_start;
	int panel_hblank_start = adjusted_mode->hdisplay;
	int panel_vbank_start = adjusted_mode->vdisplay;
	int hborder = (adjusted_mode->hdisplay - mode->hdisplay) / 2;
	int vborder = (adjusted_mode->vdisplay - mode->vdisplay) / 2;
	int new_hblank_start = hborder + mode->hdisplay;
	int new_vblank_start = vborder + mode->vdisplay;

	adjusted_mode->hdisplay = mode->hdisplay;
	adjusted_mode->hsync_start = (adjusted_mode->hsync_start -
		panel_hblank_start) + new_hblank_start;
	adjusted_mode->hsync_end = adjusted_mode->hsync_start +
		panel_hsync_time;
	adjusted_mode->vdisplay = mode->vdisplay;
	adjusted_mode->vsync_start = (adjusted_mode->vsync_start -
		panel_vbank_start) + new_vblank_start;
	adjusted_mode->vsync_end = adjusted_mode->vsync_start +
		panel_vsync_time;
	/* Adjust Crtc H and V */
	adjusted_mode->crtc_hdisplay = adjusted_mode->hdisplay;
	adjusted_mode->crtc_hblank_start = new_hblank_start;
	adjusted_mode->crtc_hblank_end = adjusted_mode->crtc_htotal - hborder;
	adjusted_mode->crtc_hsync_start = adjusted_mode->hsync_start;
	adjusted_mode->crtc_hsync_end = adjusted_mode->hsync_end;
	adjusted_mode->crtc_vdisplay = adjusted_mode->vdisplay;
	adjusted_mode->crtc_vblank_start = new_vblank_start;
	adjusted_mode->crtc_vblank_end = adjusted_mode->crtc_vtotal - vborder;
	adjusted_mode->crtc_vsync_start = adjusted_mode->vsync_start;
	adjusted_mode->crtc_vsync_end = adjusted_mode->vsync_end;
}


static void via_chrome9_sw_power_sequence_on(struct drm_device *dev,
		u32 di_port)
{
	u32 td0 = 0, td1 = 0, td2 = 0, td3 = 0;

	td0 = LCD_POWER_SEQUENCE_TD0;
	td1 = LCD_POWER_SEQUENCE_TD1;
	td2 = LCD_POWER_SEQUENCE_TD2;
	td3 = LCD_POWER_SEQUENCE_TD3;

	if (DISP_DI_DFPH == di_port) {
		KMS_DEBUG("lvds1 sw power sequence on\n");
		/* Secondary power hardware power sequence enable 0:off 1: on */
		via_chrome9_write_vga_io_bits(REG_CRD4, 0x0, BIT1);
		/* Software control power sequence ON*/
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x1, BIT0);
		/* Delay td0 msec. */
		via_chrome9_delay_msec(dev, td0);
		/* VDD ON*/
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x10, BIT4);
		/* Delay td1 msec. */
		via_chrome9_delay_msec(dev, td1);
		/* DATA ON */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x08, BIT3);
		/* Delay td2 msec. */
		via_chrome9_delay_msec(dev, td2);
		/* VEE ON (unused on vt3353)*/
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x04, BIT2);
		/* Delay td3 msec.*/
		via_chrome9_delay_msec(dev, td3);
		/* Back-Light ON */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x02, BIT1);

	} else {
		KMS_DEBUG("lvds0 sw power sequence on\n");
		/* Software control power sequence ON*/
		via_chrome9_write_vga_io_bits(REG_CR91, 0x0, BIT7);
		via_chrome9_write_vga_io_bits(REG_CR91, 0x1, BIT0);
		/* Delay td0 msec. */
		via_chrome9_delay_msec(dev, td0);
		/* VDD ON*/
		via_chrome9_write_vga_io_bits(REG_CR91, 0x10, BIT4);
		/* Delay td1 msec. */
		via_chrome9_delay_msec(dev, td1);
		/* DATA ON */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x08, BIT3);
		/* Delay td2 msec. */
		via_chrome9_delay_msec(dev, td2);
		/* VEE ON (unused on vt3353)*/
		via_chrome9_write_vga_io_bits(REG_CR91, 0x04, BIT2);
		/* Delay td3 msec. */
		via_chrome9_delay_msec(dev, td3);
		/* Back-Light ON */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x2, BIT1);
	}
}

/* Set LCD software power sequence off */
void via_chrome9_sw_power_sequence_off(struct drm_device *dev, u32 di_port)
{
	u32 td0 = 0, td1 = 0, td2 = 0, td3 = 0;

	td0 = LCD_POWER_SEQUENCE_TD0;
	td1 = LCD_POWER_SEQUENCE_TD1;
	td2 = LCD_POWER_SEQUENCE_TD2;
	td3 = LCD_POWER_SEQUENCE_TD3;

	if (DISP_DI_DFPH == di_port) {
		KMS_DEBUG("lvds1 sw power sequence off\n");
		/* Back-Light OFF */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x0, BIT1);
		/* Delay td3 msec. */
		via_chrome9_delay_msec(dev, td3);
		/* VEE OFF */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x0, BIT2);
		/* Delay td2 msec. */
		via_chrome9_delay_msec(dev, td2);
		/* DATA OFF */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x0, BIT3);
		/* Delay td1 msec. */
		via_chrome9_delay_msec(dev, td1);
		/* VDD OFF */
		via_chrome9_write_vga_io_bits(REG_CRD3, 0x0, BIT4);

	} else {
		KMS_DEBUG("lvds0 sw power sequence off\n");
		/* Back-Light OFF */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x0, BIT1);
		/* Delay td3 msec. */
		via_chrome9_delay_msec(dev, td3);
		/* VEE OFF (unused on vt3353)*/
		via_chrome9_write_vga_io_bits(REG_CR91, 0x0, BIT2);
		/* Delay td2 msec. */
		via_chrome9_delay_msec(dev, td2);
		/* DATA OFF */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x0, BIT3);
		/* Delay td1 msec. */
		via_chrome9_delay_msec(dev, td1);
		/* VDD OFF */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x0, BIT4);
	}
}

static void via_chrome9_enable_internal_lvds(struct drm_device *dev,
		u32 di_port, bool dual_channel)
{
	bool first_power_sequence_on = false;
	bool second_power_sequence_on = false;
	int chipset = dev->pci_device;

	if (DISP_DI_DFPH == di_port)
		second_power_sequence_on = true;
	else
		first_power_sequence_on = true;

	/* 1. Turn on LCD panel */
	if (first_power_sequence_on) {
		if (VX800_DEVICE == chipset) {
			via_chrome9_sw_power_sequence_on(dev, di_port);
		} else {
			/* Use first power sequence control: */
			/* Use hardware control power sequence. */
			via_chrome9_write_vga_io_bits(REG_CR91, 0, BIT0);
			/* Turn on back light and panel path. */
			via_chrome9_write_vga_io_bits(REG_CR91, 0, BIT6 + BIT7);
			/* Turn on hardware power sequence. */
			via_chrome9_write_vga_io_bits(REG_CR6A, 0x08, BIT3);
		}
	}

	if (second_power_sequence_on) {
		if (VX800_DEVICE == chipset) {
			via_chrome9_sw_power_sequence_on(dev, di_port);
		} else {
			/* Use hardware control power sequence. */
			via_chrome9_write_vga_io_bits(REG_CRD3, 0, BIT0);
			/* Turn on back light and panel path. */
			via_chrome9_write_vga_io_bits(REG_CRD3, 0, BIT6 + BIT7);
			/* Turn on hardware power sequence. */
			via_chrome9_write_vga_io_bits(REG_CRD4, 0x02, BIT1);
		}
	}

	/* 2. Power on LVDS channel. */
	if (dual_channel) {
		/* high resolution LCD (internal),
		power on both LVDS0 and LVDS1 */
		via_chrome9_write_vga_io_bits(REG_CRD2, 0x0, BIT6 + BIT7);
	} else {
		switch (di_port) {
		case DISP_DI_DFPL:
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x0, BIT7);
			break;
		case DISP_DI_DFPH:
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x0, BIT6);
			break;
		default:
			KMS_DEBUG(
				"via_chrome9_enable_internal_lvds: invalid di_port\n");
			break;
		}
	}
}


void via_chrome9_disable_internal_lvds(struct drm_device *dev,
		u32 di_port, bool dual_channel)
{
	bool first_power_sequence_on = false;
	bool second_power_sequence_on = false;
	int chipset = dev->pci_device;
	KMS_DEBUG("\n");
	if (DISP_DI_DFPH == di_port)
		second_power_sequence_on = true;
	else
		first_power_sequence_on = true;

	/* 1. Turn off LCD panel */
	if (first_power_sequence_on) {
		if (VX800_DEVICE == chipset) {
			via_chrome9_sw_power_sequence_off(dev, di_port);
		} else {
			/* Use first power sequence control: */
			/* Turn off power sequence. */
			via_chrome9_write_vga_io_bits(REG_CR6A, 0, BIT3);

			/* Turn off back light and panel path. */
			via_chrome9_write_vga_io_bits(REG_CR91, 0xC0,
				BIT6 + BIT7);
		}
	}

	if (second_power_sequence_on) {
		if (VX800_DEVICE == chipset) {
			via_chrome9_sw_power_sequence_off(dev, di_port);
		} else {
			/* Use second power sequence control: */
			/* Turn off power sequence. */
			via_chrome9_write_vga_io_bits(REG_CRD4, 0, BIT1);
			/* Turn off back light and panel path. */
			via_chrome9_write_vga_io_bits(REG_CRD3, 0xC0,
				BIT6 + BIT7);
		}
	}

	/* 2. Power off LVDS channel. */
	if (dual_channel) {
		/* high resolution LCD (internal),
		power off both LVDS0 and LVDS1 */
		via_chrome9_write_vga_io_bits(REG_CRD2, 0xC0,
			BIT7 + BIT6);
	} else {
		switch (di_port) {
		case DISP_DI_DFPL:
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x80, BIT7);
			break;
		case DISP_DI_DFPH:
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x40, BIT6);
			break;
		default:
			KMS_DEBUG(
				"via_chrome9_disable_internal_lvds:invalid di_port\n");
			break;
		}
	}
}

static void via_chrome9_lvds_dpms(struct drm_encoder *encoder, int mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	u32 di_port = via_chrome9_encoder->di_port;
	struct drm_device *dev = encoder->dev;
	int chipset = dev->pci_device;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_lcd_options *via_lcd_options = NULL;
	struct drm_connector *connector;
	struct via_chrome9_connector *via_chrome9_connector = NULL;
	bool dual_channel = false;
	enum drm_connector_status con_status = connector_status_disconnected;
	struct via_chrome9_crtc *via_chrome9_crtc;
	int iga_path;

	KMS_DEBUG("\n");
	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		if (encoder ==  connector->encoder) {
			via_chrome9_connector =
				to_via_chrome9_connector(connector);
			break;
		}
	}
	if (!via_chrome9_connector) {
		KMS_DEBUG(
			"this encoder is not used by any connector,dpms mode is %d\n",
			mode);
		return;
	} else {
		KMS_DEBUG("dpms mode is %d\n", mode);
	}
	if (via_chrome9_connector->detect)
		con_status = via_chrome9_connector->detect(connector, false);

	via_lcd_options = (struct via_lcd_options *)
		via_chrome9_connector->options;
	dual_channel = via_lcd_options->dual_channel;

	switch (mode) {
	case DRM_MODE_DPMS_ON:
		if (encoder->crtc == NULL)
			return;
		via_chrome9_crtc = to_via_chrome9_crtc(encoder->crtc);
		iga_path = via_chrome9_crtc->crtc_id;

		/* when using the EPIA-EX board, if we do not set this bit,
		light LCD will failed in nonRandR structure,
		So, when light LCD this bit is always setted */
		via_chrome9_write_vga_io_bits(REG_CR6A, 0x08, BIT3);
		if (di_port)
			via_chrome9_digital_port_pad_on(di_port,
				iga_path, chipset);
		if (p_priv->bios_is_spread_spectrum) {
			if ((VX800_DEVICE == chipset) ||
			(VX855_DEVICE == chipset) ||
			(VX900_DEVICE == chipset)) {
				/* GPIO-4/5 are used for spread spectrum,
				we must clear SR3D[7:6] to
				disable GPIO-4/5 output */
				via_chrome9_write_vga_io_bits(REG_SR3D,
					0x01, 0xC1);
			} else {
				via_chrome9_write_vga_io_bits(REG_SR2C,
					0x01, BIT0);
			}
			via_chrome9_write_vga_io_bits(REG_SR1E, 0x08, BIT3);
		}
		via_chrome9_enable_internal_lvds(dev, di_port, dual_channel);
		break;

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		if (con_status == connector_status_disconnected)
			return;
		via_chrome9_disable_internal_lvds(dev, di_port, dual_channel);
		if (di_port)
			via_chrome9_digital_port_pad_off(di_port);
		break;
	}
}

static void via_chrome9_lvds_prepare(struct drm_encoder *encoder)
{
	via_chrome9_lvds_dpms(encoder, DRM_MODE_DPMS_OFF);
}

bool via_chrome9_lvds_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct drm_crtc *crtc = encoder->crtc ;
	struct via_chrome9_crtc *via_chrome9_crtc = to_via_chrome9_crtc(crtc);
	int iga_path = via_chrome9_crtc->crtc_id;
	struct drm_device *dev = encoder->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_lcd_options *via_lcd_options = NULL;
	struct drm_connector *connector = NULL;
	struct via_chrome9_connector *via_chrome9_connector = NULL;
	bool dual_channel = false;
	bool center = false;
	unsigned int panel_width = 0;
	unsigned int panel_height = 0;
	u32 panel_index = 0;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		if (encoder ==  connector->encoder) {
			via_chrome9_connector =
				to_via_chrome9_connector(connector);
			break;
		}
	}
	if (!via_chrome9_connector) {
		KMS_DEBUG("this encoder is not used by any connector\n");
		return false;
	}
	via_chrome9_crtc->crtc_scale_status = 0;
	via_lcd_options = (struct via_lcd_options *)
		via_chrome9_connector->options;
	dual_channel = via_lcd_options->dual_channel;
	panel_width = via_lcd_options->physical_width;
	panel_height = via_lcd_options->physical_height;
	panel_index = via_lcd_options->panel_index;
	center = via_lcd_options->center;
	/* KMS_DEBUG("via_chrome9_lvds_mode_fixup,mode is:%dx%d\n",
		mode->hdisplay, mode->vdisplay);
	KMS_DEBUG("via_chrome9_lvds_mode_fixup,adjust mode is:%dx%d\n",
		mode->hdisplay, mode->vdisplay); */

	/* If mode is equal to panel physical size, don't change it
	Maybe we should discuss it! */
	if ((mode->hdisplay != panel_width) ||
		(mode->vdisplay != panel_height)) {
		/* Get build-in timing */
		struct display_timing_table *panel_crtc_table = NULL;
		panel_crtc_table = via_chrome9_get_driver_timing(LCD_MODE_TABLE,
			panel_index);
		if (!panel_crtc_table)
			return false;
		via_chrome9_fill_mode_by_driver_timing(panel_crtc_table,
			adjusted_mode);
	}

	drm_mode_set_crtcinfo(adjusted_mode, 0);
	/* Take care of 410 downscaling */
	if ((mode->hdisplay > panel_width) ||
		(mode->vdisplay > panel_height)) {
		if (p_priv->via_chrome9_chip_caps & CAPS_IGA2_DOWNSCALING)
			via_chrome9_crtc->crtc_scale_status = VIA_SCALING_HW;
	} else {
		if ((iga_path != IGA2) || center) {
			/* Do centering according to mode and adjusted_mode */
			via_chrome9_centering_timing(mode, adjusted_mode);
		} else {
			if (mode->hdisplay < panel_width)
				via_chrome9_crtc->crtc_scale_status |=
					VIA_HOR_EXPAND;
			if (mode->vdisplay < panel_height)
				via_chrome9_crtc->crtc_scale_status |=
					VIA_VER_EXPAND;
		}
	}
	return true;
}

void via_chrome9_lvds_msb_set(struct drm_encoder *encoder, bool is_msb)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	u32 di_port = via_chrome9_encoder->di_port;

	if (DISP_DI_DFPH == di_port) {
		if (is_msb)
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x00, BIT0);
		else
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x01, BIT0);

	} else if (DISP_DI_DFPL == di_port) {
		if (is_msb)
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x00, BIT1);
		else
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x02, BIT1);

	} else {
		KMS_DEBUG("The diport is invalid\n");
		return;
	}
}

static void via_chrome9_lvds_channel_set(struct drm_encoder *encoder,
		bool is_dual_channel)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	u32 di_port = via_chrome9_encoder->di_port;

	if (is_dual_channel) {
		via_chrome9_write_vga_io_bits(REG_CRD2, 0x20, BIT5 + BIT4);
		via_chrome9_write_vga_io_bits(REG_CR88, BIT4, BIT4);
	} else {
		if (DISP_DI_DFPH == di_port)
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x00, BIT5);
		else if (DISP_DI_DFPL == di_port)
			via_chrome9_write_vga_io_bits(REG_CRD2, 0x00,
				BIT4 + BIT5);
	}
}

void via_chrome9_lvds_no_dithering_set(bool is_no_dithering)
{
	if (is_no_dithering)
		via_chrome9_write_vga_io_bits(REG_CR88, 0, BIT0);
	else
		via_chrome9_write_vga_io_bits(REG_CR88, BIT0, BIT0);
}

static void via_chrome9_lvds_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct drm_device *dev = encoder->dev;
	struct via_lcd_options *via_lcd_options = NULL;
	struct drm_connector *connector;
	struct via_chrome9_connector *via_chrome9_connector = NULL;

	KMS_DEBUG("\n");
	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		if (encoder ==  connector->encoder) {
			via_chrome9_connector =
				to_via_chrome9_connector(connector);
			break;
		}
	}
	if (!via_chrome9_connector) {
		KMS_DEBUG("this encoder is not used by any connector\n");
		return;
	}
	via_lcd_options = (struct via_lcd_options *)
		via_chrome9_connector->options;

	via_chrome9_lvds_msb_set(encoder, via_lcd_options->msb);
	via_chrome9_lvds_channel_set(encoder, via_lcd_options->dual_channel);
	via_chrome9_lvds_no_dithering_set(via_lcd_options->no_dithering);

	return;
}

static void via_chrome9_lvds_commit(struct drm_encoder *encoder)
{
	via_chrome9_lvds_dpms(encoder, DRM_MODE_DPMS_ON);
}


static enum drm_connector_status via_chrome9_lvds_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	return connector_status_connected;
}

static const struct drm_encoder_helper_funcs via_chrome9_lvds_helper_funcs = {
	.dpms = via_chrome9_lvds_dpms,
	.mode_fixup = via_chrome9_lvds_mode_fixup,
	.prepare = via_chrome9_lvds_prepare,
	.mode_set = via_chrome9_lvds_mode_set,
	.commit = via_chrome9_lvds_commit,
	.detect = via_chrome9_lvds_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_lvds_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

/* calcute the BCH code and output 1 byte BCH code */
u8 via_chrome9_insert_bch_code(u8 *input, u32 length)
{
	u32 i, n;
	u32 reg[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	u32 matrix[56];
	u8  output = 0;

	/* initialize matrix */
	for (i = 0; i < length; i++) {
		matrix[0 + i * 8] = (input[i] >> 0) & BIT0;
		matrix[1 + i * 8] = (input[i] >> 1) & BIT0;
		matrix[2 + i * 8] = (input[i] >> 2) & BIT0;
		matrix[3 + i * 8] = (input[i] >> 3) & BIT0;
		matrix[4 + i * 8] = (input[i] >> 4) & BIT0;
		matrix[5 + i * 8] = (input[i] >> 5) & BIT0;
		matrix[6 + i * 8] = (input[i] >> 6) & BIT0;
		matrix[7 + i * 8] = (input[i] >> 7) & BIT0;
	}

	for (i = 0; i < length * 8; i++) {
		reg[0] = reg[8] ^ matrix[i];
		reg[8] = reg[7] ^ reg[0];
		reg[7] = reg[6] ^ reg[0];

		reg[6] = reg[5];
		reg[5] = reg[4];
		reg[4] = reg[3];
		reg[3] = reg[2];
		reg[2] = reg[1];
		reg[1] = reg[0];
	}

	n = 0;
	for (i = 1; i <= 4; i++) {
		if (reg[i] == 1)
			n = 2 * n + 1;
		else
			n = 2 * n;
	}

	output |= n << 4;

	n = 0;
	for (i = 5; i <= 8; i++) {
		if (reg[i] == 1)
			n = 2 * n + 1;
		else
			n = 2 * n;
	}

	output |= n;

	return output;
}

void via_chrome9_set_avi_info_frame(struct drm_display_mode *mode)
{
	struct via_hdmi_info_frame_rec  info_frame_packet;
	u32 i = 0;
	u8 check_sum = 0;
	u32 mm_c204, mm_c208, mm_c20c, mm_c210, mm_c214, mm_c218;
	u32 mm_c21c, mm_c220, mm_c224;
	struct via_cea_timing_info_rec *cea_timing_info =
		&via_cea_timing_table[mode->hskew - 1];
	info_frame_packet.packet_header[0] = 0x82;
	info_frame_packet.packet_header[1] = 0x02;
	info_frame_packet.packet_header[2] = 0x0D;
	info_frame_packet.ecc_code =
		via_chrome9_insert_bch_code(info_frame_packet.packet_header, 3);

	/* set Packet Bytes */
	for (i = 0; i < 28; i++)
		info_frame_packet.packet_byte[i] = 0;

	/* Remove the comment when upstream!! */
	/* S1  S0   |   Scan Information
	* -----------------------------
	*  0   1   |   Overscan
	*  1   0   |   Underscan
	*/
	info_frame_packet.packet_byte[1] |= 0x2;

	/* Remove the comment when upstream!! */
	/* R3  R2  R1  R0   |   Active Format Aspect Ratio
	* -----------------------------------------------
	*  1   0   0   0   |   Same as picture aspect ratio
	*  1   0   0   1   |   4:3
	*  1   0   1   0   |   16:9
	*  1   0   1   1   |   14:9
	*/
	info_frame_packet.packet_byte[2] |= 0x08;

	/* Remove the comment when upstream!! */
	/* M1  M0   |   Picture Aspect Ratio
	* ---------------------------------
	*  0   1   |    4:3
	*  1   0   |   16:9
	*/
	switch (cea_timing_info->ratio) {
	case RATIO_4_3:
		info_frame_packet.packet_byte[2] |= 0x10;
		break;

	case RATIO_16_9:
		info_frame_packet.packet_byte[2] |= 0x20;
		break;

	default:
		break;
	}

	/* Remove the comment when upstream!! */
	/* ITC bit */
	info_frame_packet.packet_byte[3] |= 0x80;

	/* Remove the comment when upstream!! */
	/* VIC code */
	info_frame_packet.packet_byte[4] = cea_timing_info->id;

	/* Remove the comment when upstream!! */
	/* count checksum */
	for (i = 1; i < 28; i++)
		check_sum += info_frame_packet.packet_byte[i];

	check_sum += (0x82 + 0x02 + 0x0D);

	check_sum = 0x100 - check_sum;
	info_frame_packet.packet_byte[0] = check_sum;

	/* Remove the comment when upstream!! */
	/* set bch_code */
	for (i = 0; i < 4; i++)
		info_frame_packet.bch_code[i] = via_chrome9_insert_bch_code(
			&(info_frame_packet.packet_byte[i * 7]), 7);

	mm_c204 = (info_frame_packet.ecc_code << 24) |
		(info_frame_packet.packet_header[2] << 16) |
		(info_frame_packet.packet_header[1] << 8) |
		info_frame_packet.packet_header[0];
	mm_c208 = (info_frame_packet.packet_byte[3] << 24) |
		(info_frame_packet.packet_byte[2] << 16) |
		(info_frame_packet.packet_byte[1] << 8) |
		info_frame_packet.packet_byte[0];
	mm_c20c = (info_frame_packet.bch_code[0] << 24) |
		(info_frame_packet.packet_byte[6] << 16) |
		(info_frame_packet.packet_byte[5] << 8) |
		info_frame_packet.packet_byte[4];
	mm_c210 = (info_frame_packet.packet_byte[10] << 24) |
		(info_frame_packet.packet_byte[9] << 16) |
		(info_frame_packet.packet_byte[8] << 8) |
		info_frame_packet.packet_byte[7];
	mm_c214 = (info_frame_packet.bch_code[1] << 24) |
		(info_frame_packet.packet_byte[13] << 16) |
		(info_frame_packet.packet_byte[12] << 8) |
		info_frame_packet.packet_byte[11];
	mm_c218 = (info_frame_packet.packet_byte[17] << 24) |
		(info_frame_packet.packet_byte[16] << 16) |
		(info_frame_packet.packet_byte[15] << 8) |
		info_frame_packet.packet_byte[14];
	mm_c21c = (info_frame_packet.bch_code[2] << 24) |
		(info_frame_packet.packet_byte[20] << 16) |
		(info_frame_packet.packet_byte[19] << 8) |
		info_frame_packet.packet_byte[18];
	mm_c220 = (info_frame_packet.packet_byte[24] << 24) |
		(info_frame_packet.packet_byte[23] << 16) |
		(info_frame_packet.packet_byte[22] << 8) |
		info_frame_packet.packet_byte[21];
	mm_c224 = (info_frame_packet.bch_code[3] << 24) |
		(info_frame_packet.packet_byte[27] << 16) |
		(info_frame_packet.packet_byte[26] << 8) |
		info_frame_packet.packet_byte[25];

	MMIO_WR(0xC204, mm_c204);
	MMIO_WR(0xC208, mm_c208);
	MMIO_WR(0xC20C, mm_c20c);
	MMIO_WR(0xC210, mm_c210);
	MMIO_WR(0xC214, mm_c214);
	MMIO_WR(0xC218, mm_c218);
	MMIO_WR(0xC21C, mm_c21c);
	MMIO_WR(0xC220, mm_c220);
	MMIO_WR(0xC224, mm_c224);
	MMIO_WR(0xC200, 0x80);
}

void via_chrome9_set_audio_info_frame(void)
{
	struct via_hdmi_info_frame_rec info_frame_packet;
	int i = 0;
	u8 check_sum = 0;
	u32 mm_c204, mm_c208, mm_c20c, mm_c210, mm_c214, mm_c218, mm_c21c;
	u32 mm_c220, mm_c224;

	info_frame_packet.packet_header[0] = 0x84;
	info_frame_packet.packet_header[1] = 0x01;
	info_frame_packet.packet_header[2] = 0x0A;
	info_frame_packet.ecc_code =
		via_chrome9_insert_bch_code(info_frame_packet.packet_header, 3);

	/* set Packet Bytes */
	for (i = 0; i < 28; i++)
		info_frame_packet.packet_byte[i] = 0;

	/* Remove the comment when upstream!! */
	/* CC2 CC1 CC0  |   Audio Channel Count
	* ---------------------------------------
	*  0   0   0   |   Refer to Stream Header
	*  0   0   1   |   2ch
	*  0   1   0   |   3ch
	*  0   1   1   |   4ch
	*/
	info_frame_packet.packet_byte[1] |= 0x0;

	/* Remove the comment when upstream!! */
	/* CT3 CT2 CT1 CT0  |   Audio Coding Type
	* -------------------------------------------
	*  0   0   0   0   |   Refer to Stream Header
	*  0   0   0   1   |   IEC 60958 PCM[30,31]
	*  0   0   1   0   |   AC-3
	*  0   0   1   1   |   MPEG1 (Layer 1 & 2)
	*/
	info_frame_packet.packet_byte[1] |= 0x00;

	/* Remove the comment when upstream!! */
	/* SS1 SS0  |   Sample Size
	* --------------------------
	*  0   0   |   Refer to Stream Header
	*  0   1   |   16 bit
	*  1   0   |   20 bit
	*  1   1   |   24 bit
	*/
	info_frame_packet.packet_byte[2] |= 0x00;

	/* Remove the comment when upstream!! */
	/* SF2 SF1 SF0  |   Sampling Frequency
	* ---------------------------------------
	*  0   0   0   |   Refer to Stream Header
	*  0   0   1   |   32 kHz
	*  0   1   0   |   44.1 kHz(CD)
	*  0   1   1   |   48 kHz
	*  1   0   0   |   88.2 kHz
	*  1   0   1   |   96 kHz
	*  1   1   0   |   176.4 kHz
	*  1   1   1   |   192 kHz
	*/
	info_frame_packet.packet_byte[2] |= 0x00;

	/* Remove the comment when upstream!! */
	/* count checksum */
	for (i = 1; i < 28; i++)
		check_sum += info_frame_packet.packet_byte[i];

	check_sum += (0x84 + 0x01 + 0x0A);
	check_sum = 0x100 - check_sum;
	info_frame_packet.packet_byte[0] = check_sum;

	/* Remove the comment when upstream!! */
	/* set BCHCode */
	for (i = 0; i < 4; i++)
		info_frame_packet.bch_code[i] = via_chrome9_insert_bch_code(
			&(info_frame_packet.packet_byte[i * 7]), 7);

	mm_c204 = (info_frame_packet.ecc_code << 24) |
		(info_frame_packet.packet_header[2] << 16) |
		(info_frame_packet.packet_header[1] << 8) |
		info_frame_packet.packet_header[0];
	mm_c208 = (info_frame_packet.packet_byte[3] << 24) |
		(info_frame_packet.packet_byte[2] << 16) |
		(info_frame_packet.packet_byte[1] << 8) |
		info_frame_packet.packet_byte[0];
	mm_c20c = (info_frame_packet.bch_code[0] << 24) |
		(info_frame_packet.packet_byte[6] << 16) |
		(info_frame_packet.packet_byte[5] << 8) |
		info_frame_packet.packet_byte[4];
	mm_c210 = (info_frame_packet.packet_byte[10] << 24) |
		(info_frame_packet.packet_byte[9] << 16) |
		(info_frame_packet.packet_byte[8] << 8) |
		info_frame_packet.packet_byte[7];
	mm_c214 = (info_frame_packet.bch_code[1] << 24) |
		(info_frame_packet.packet_byte[13] << 16) |
		(info_frame_packet.packet_byte[12] << 8) |
		info_frame_packet.packet_byte[11];
	mm_c218 = (info_frame_packet.packet_byte[17] << 24) |
		(info_frame_packet.packet_byte[16] << 16) |
		(info_frame_packet.packet_byte[15] << 8) |
		info_frame_packet.packet_byte[14];
	mm_c21c = (info_frame_packet.bch_code[2] << 24) |
		(info_frame_packet.packet_byte[20] << 16) |
		(info_frame_packet.packet_byte[19] << 8) |
		info_frame_packet.packet_byte[18];
	mm_c220 = (info_frame_packet.packet_byte[24] << 24) |
		(info_frame_packet.packet_byte[23] << 16) |
		(info_frame_packet.packet_byte[22] << 8) |
		info_frame_packet.packet_byte[21];
	mm_c224 = (info_frame_packet.bch_code[3] << 24) |
		(info_frame_packet.packet_byte[27] << 16) |
		(info_frame_packet.packet_byte[26] << 8) |
		info_frame_packet.packet_byte[25];

	MMIO_WR(0xC204, mm_c204);
	MMIO_WR(0xC208, mm_c208);
	MMIO_WR(0xC20C, mm_c20c);
	MMIO_WR(0xC210, mm_c210);
	MMIO_WR(0xC214, mm_c214);
	MMIO_WR(0xC218, mm_c218);
	MMIO_WR(0xC21C, mm_c21c);
	MMIO_WR(0xC220, mm_c220);
	MMIO_WR(0xC224, mm_c224);
	MMIO_WR(0xC200, 0x81);
}

void via_chrome9_set_info_frame_data(struct drm_display_mode *mode)
{
	via_chrome9_set_avi_info_frame(mode);
	via_chrome9_set_audio_info_frame();
}

u32 via_chrome9_get_sample_rate(void)
{
	u32 mm_c4d4, mm_c4d8;
	u32 hd_ac_sample_rate = 0;

	/* step 1. read out C4D8 */
	mm_c4d4 = MMIO_RD(0xC4D4) & 0x00FFFFFF;
	mm_c4d4 |=  0x01000000;      /* stream format */
	MMIO_WR(0xC4D4, mm_c4d4);

	mm_c4d8 = MMIO_RD(0xC4D8); /* [15:0] */

	/*
	* step 2. decode C4D8 by definition of C45C
	* C45C [14] Converter_Format_Base
	* BASE. Converter Format Sample Base Rate.
	* (PCM Format structure bit 14)
	*              0: 48kHz
	*              1: 44.1kHz
	* C45C [13:11] Converter_Format_Mult
	* MULT. Converter Format Sample Base Rate Multiple.
	* (PCM Format structure bits 13:11)
	*              000: 48kHz/ 44.1 kHz or lesss
	*              001: x2 (96kHz, 88.2kHz, 32kHz)
	*              010: x3 (144kHz)
	*              011: x4 (192kHz, 176.4kHz)
	*              100-111: Reserved
	* C45C [10:8] Converter_Format_Div
	* DIV. Converter Format Sample Base Rate Divisor.
	* (PCM Format structure bits 10:8)
	*              000: Divide by 1 (48kHz, 44.1 kHz)
	*              001: Divide by 2 (24kHz, 22.05kHz)
	*              010: Divide by 3 (16kHz, 32kHz)
	*              011: Divide by 4 (11.025kHz)
	*              100: Divide by 5 (9.6kHz)
	*              101: Divide by 6 (8kHz)
	*              110: Divide by 7
	*              111: Divide by 8 (6kHz)
	*/
	if (mm_c4d8 & 0x4000)
		hd_ac_sample_rate = 44100;
	else
		hd_ac_sample_rate = 48000;

	switch ((mm_c4d8 & 0x3800) >> 11) {
	case 1:
		hd_ac_sample_rate *= 2;
		break;
	case 2:
		hd_ac_sample_rate *= 3;
		break;
	case 3:
		hd_ac_sample_rate *= 4;
		break;
	default:
		break;
	}

	switch ((mm_c4d8 & 0x700) >> 8) {
	case 1:
		hd_ac_sample_rate /= 2;
		break;
	case 2:
		hd_ac_sample_rate /= 3;
		break;
	case 3:
		hd_ac_sample_rate /= 4;
		break;
	case 4:
		hd_ac_sample_rate /= 5;
		break;
	case 5:
		hd_ac_sample_rate /= 6;
		break;
	case 6:
		hd_ac_sample_rate /= 7;
		break;
	case 7:
		hd_ac_sample_rate /= 8;
		break;
	default:
		break;
	}

	return hd_ac_sample_rate;
}


void via_chrome9_set_hd_audio_parameter(struct drm_display_mode *mode)
{
	u32 mm_c424;
	u32 hd_ac_sample_rate, hd_ac_pixel_clock;

	hd_ac_pixel_clock = mode->clock * 1000;

	hd_ac_sample_rate = via_chrome9_get_sample_rate();

	/* C424[14:13] codec type. related with mm c618, mm c698 */
	mm_c424 = MMIO_RD(0xC424) & 0xFFFF9FFF;
	MMIO_WR(0xC424, mm_c424);

	/* calculate C400, C404, C410 and C414 */
	via_chrome9_set_HDAudio_sample_rate(
		hd_ac_sample_rate, hd_ac_pixel_clock);

	/* Remove the comment when upstream!! */
	/* fill cts & n according to Table7-1, 7-2, 7-3 of HDMI spec 1.3 */
	via_chrome9_set_HDAudio_CTS_N(hd_ac_sample_rate,
		hd_ac_pixel_clock);

}

/* Load CRTC  Pixel Timing registers */
void via_chrome9_load_crtc_pixel_timing(struct drm_display_mode *mode)
{
	int i;
	int reg_value = 0;
	u32 reg_mask = 0;
	u16 reg = 0;

	KMS_DEBUG("via_chrome9_load_crtc_pixel_timing\n");

	for (i = 0; i < 12; i++)  {
		switch (i) {
		case VIA_H_TOTAL_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_TOTAL_FORMULA
				(mode->crtc_htotal);
			reg = VIA_IGA1_PIX_H_TOTAL_REG;
			reg_mask = VIA_IGA1_PIX_H_TOTAL_MASK;
			break;
		case VIA_H_ADDR_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_ADDR_FORMULA
				((mode->crtc_hdisplay) << 16);
			reg = VIA_IGA1_PIX_H_ADDR_REG;
			reg_mask = VIA_IGA1_PIX_H_ADDR_MASK;
			break;
		case VIA_H_BLANK_START_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_BLANK_START_FORMULA
				(mode->crtc_hblank_start);
			reg = VIA_IGA1_PIX_H_BNK_ST_REG;
			reg_mask = VIA_IGA1_PIX_H_BNK_ST_MASK;
			break;
		case VIA_H_BLANK_END_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_BLANK_END_FORMULA
				((mode->crtc_hblank_end) << 16);
			reg = VIA_IGA1_PIX_H_BNK_END_REG;
			reg_mask = VIA_IGA1_PIX_H_BNK_END_MASK;
			break;
		case VIA_H_SYNC_START_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_SYNC_START_FORMULA
				(mode->crtc_hsync_start);
			reg = VIA_IGA1_PIX_H_SYNC_ST_REG;
			reg_mask = VIA_IGA1_PIX_H_SYNC_ST_MASK;
			break;
		case VIA_H_SYNC_END_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_HOR_SYNC_END_FORMULA
				((mode->crtc_hsync_end) << 16);
			reg = VIA_IGA1_PIX_H_SYNC_END_REG;
			reg_mask = VIA_IGA1_PIX_H_SYNC_END_MASK;
			break;
		case VIA_V_TOTAL_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_TOTAL_FORMULA
				(mode->crtc_vtotal);
			reg = VIA_IGA1_PIX_V_TOTAL_REG;
			reg_mask = VIA_IGA1_PIX_V_TOTAL_MASK;
			break;
		case VIA_V_ADDR_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_ADDR_FORMULA
				((mode->crtc_vdisplay) << 16);
			reg = VIA_IGA1_PIX_V_ADDR_REG;
			reg_mask = VIA_IGA1_PIX_V_ADDR_MASK;
			break;
		case VIA_V_BLANK_START_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_BLANK_START_FORMULA
				(mode->crtc_vblank_start);
			reg = VIA_IGA1_PIX_V_BNK_ST_REG;
			reg_mask = VIA_IGA1_PIX_V_BNK_ST_MASK;
			break;
		case VIA_V_BLANK_END_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_BLANK_END_FORMULA
				((mode->crtc_vblank_end) << 16);
			reg = VIA_IGA1_PIX_V_BNK_END_REG;
			reg_mask = VIA_IGA1_PIX_V_BNK_END_MASK;
			break;
		case VIA_V_SYNC_START_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_SYNC_START_FORMULA
				(mode->crtc_vsync_start);
			reg = VIA_IGA1_PIX_V_SYNC_ST_REG;
			reg_mask = VIA_IGA1_PIX_V_SYNC_ST_MASK;
			break;
		case VIA_V_SYNC_END_INDEX:
			reg_value = VIA_IGA1_PIXELTIMING_VER_SYNC_END_FORMULA
				((mode->crtc_vsync_end) << 12);
			reg = VIA_IGA1_PIX_V_SYNC_END_REG;
			reg_mask = VIA_IGA1_PIX_V_SYNC_END_MASK;
			break;
		}
		MMIO_WR_MASK(reg, reg_value, reg_mask);
	}

	if (mode->flags & DRM_MODE_FLAG_INTERLACE) {
		reg_value = VIA_IGA1_PIXELTIMING_HVSYNC_OFFSET_END_FORMULA
			(mode->crtc_htotal, mode->crtc_hsync_start);
		MMIO_WR_MASK(VIA_IGA1_PIX_HALF_LINE_REG, reg_value,
			VIA_IGA1_PIX_HALF_LINE_MASK);
		via_chrome9_write_vga_io_bits(REG_CR32, BIT2, BIT2);
		/**
		 * According to information from HW team,
		 * we need to set 0xC280[1] = 1 (HDMI function enable)
		 * or 0xC640[0] = 1 (DP1 enable)
		 * to let the half line function work.
		 * Otherwise, the clock for interlace mode
		 * will not correct.
		 * This is a special setting for 410.
		 */
		MMIO_WR_MASK(0xC280, BIT1, BIT1);
	} else {
		MMIO_WR_MASK(VIA_IGA1_PIX_HALF_LINE_REG, 0x0,
			VIA_IGA1_PIX_HALF_LINE_MASK);
		via_chrome9_write_vga_io_bits(REG_CR32, 0x0, BIT2);
	}
	via_chrome9_write_vga_io_bits(REG_CRFD, BIT5 + BIT6, BIT5 + BIT6);
}


/* caculate the cetering timing using adjusted_mode and four bordles.*/
static void via_chrome9_get_centering_timing(
		struct drm_display_mode *adjusted_mode,
		u32 left_border, u32 right_border,
		u32 top_border, u32 bottom_border)
{
	u32 panelHSyncTime = 0;
	u32 panelVSyncTime = 0;
	u32 panelHBlankStart = 0;
	u32 panelVBlankStart = 0;
	u32 newHBlankStart = 0;
	u32 newVBlankStart = 0;

	if (adjusted_mode->flags & DRM_MODE_FLAG_INTERLACE) {
		/* when interlace mode,
		we should consider halve vertical timings. */
		panelHSyncTime = adjusted_mode->hsync_end -
			adjusted_mode->hsync_start;
		panelVSyncTime = adjusted_mode->vsync_end / 2 -
			adjusted_mode->vsync_start / 2;
		panelHBlankStart = adjusted_mode->hdisplay;
		panelVBlankStart = adjusted_mode->vdisplay / 2;
		newHBlankStart = adjusted_mode->hdisplay - left_border;
		newVBlankStart = adjusted_mode->vdisplay / 2 - top_border;

		adjusted_mode->hdisplay =
			adjusted_mode->hdisplay - left_border - right_border;
		adjusted_mode->hsync_start =
			(adjusted_mode->hsync_start - panelHBlankStart) +
			newHBlankStart;
		adjusted_mode->hsync_end =
			adjusted_mode->hsync_start + panelHSyncTime;

		adjusted_mode->vdisplay = adjusted_mode->vdisplay / 2 -
			top_border - bottom_border;
		adjusted_mode->vsync_start =
			(adjusted_mode->vsync_start / 2 - panelVBlankStart) +
			newVBlankStart;
		adjusted_mode->vsync_end =
			adjusted_mode->vsync_start + panelVSyncTime;

	} else {
		panelHSyncTime =
			adjusted_mode->hsync_end - adjusted_mode->hsync_start;
		panelVSyncTime =
			adjusted_mode->vsync_end - adjusted_mode->vsync_start;
		panelHBlankStart = adjusted_mode->hdisplay;
		panelVBlankStart = adjusted_mode->vdisplay;
		newHBlankStart = adjusted_mode->hdisplay - left_border;
		newVBlankStart = adjusted_mode->vdisplay - top_border;

		KMS_DEBUG("adjusted_mode: hd = %d, hss = %d, hse = %d.\n",
			adjusted_mode->hdisplay,
			adjusted_mode->hsync_start,
			adjusted_mode->hsync_end);
		KMS_DEBUG("adjusted_mode: vd = %d, vss = %d, vse = %d.\n",
			adjusted_mode->vdisplay,
			adjusted_mode->vsync_start,
			adjusted_mode->vsync_end);

		adjusted_mode->hdisplay =
			adjusted_mode->hdisplay - left_border - right_border;
		adjusted_mode->hsync_start =
			(adjusted_mode->hsync_start - panelHBlankStart) +
			newHBlankStart;
		adjusted_mode->hsync_end =
			adjusted_mode->hsync_start + panelHSyncTime;

		adjusted_mode->vdisplay =
			adjusted_mode->vdisplay - top_border - bottom_border;
		adjusted_mode->vsync_start =
			(adjusted_mode->vsync_start - panelVBlankStart) +
			newVBlankStart;
		adjusted_mode->vsync_end =
			adjusted_mode->vsync_start + panelVSyncTime;

	}

	/*Adjust crtc H and V*/
	adjusted_mode->crtc_hdisplay = adjusted_mode->hdisplay;
	adjusted_mode->crtc_hblank_start = newHBlankStart;
	adjusted_mode->crtc_hblank_end =
		adjusted_mode->crtc_htotal - left_border;
	adjusted_mode->crtc_hsync_start = adjusted_mode->hsync_start;
	adjusted_mode->crtc_hsync_end = adjusted_mode->hsync_end;

	adjusted_mode->crtc_vdisplay = adjusted_mode->vdisplay;
	adjusted_mode->crtc_vblank_start = newVBlankStart;
	adjusted_mode->crtc_vblank_end =
		adjusted_mode->crtc_vtotal - top_border;
	adjusted_mode->crtc_vsync_start = adjusted_mode->vsync_start;
	adjusted_mode->crtc_vsync_end = adjusted_mode->vsync_end;

	KMS_DEBUG(
		"adjusted_mode_crtc: hd = %d, hbs = %d, hbe = %d, hss = %d, hse = %d.",
		adjusted_mode->crtc_hdisplay,
		adjusted_mode->crtc_hblank_start,
		adjusted_mode->crtc_hblank_end,
		adjusted_mode->crtc_hsync_start,
		adjusted_mode->crtc_hsync_end);

	KMS_DEBUG(
		"adjusted_mode_crtc: vd = %d, vbs = %d, vbe = %d, vss = %d, vse = %d.",
		adjusted_mode->crtc_vdisplay,
		adjusted_mode->crtc_vblank_start,
		adjusted_mode->crtc_vblank_end,
		adjusted_mode->crtc_vsync_start,
		adjusted_mode->crtc_vsync_end);

	}


void via_chrome9_set_integrated_hdmi_mode(struct drm_display_mode *mode,
		int iga_path)
{
	u32 reg_c280, reg_c284;
	int delay, max_packet;
	/* Infoframe setting */
	via_chrome9_set_info_frame_data(mode);
	via_chrome9_set_hd_audio_parameter(mode);
	/* 135MHz ~ 270MHz */
	if (mode->clock >= 135000)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x00000000, 0xC0000000);
	/* 67.5MHz ~ <135MHz */
	else if (mode->clock >= 67500)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x40000000, 0xC0000000);
	/* 33.75MHz ~ <67.5MHz */
	else if (mode->clock >= 33750)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x80000000, 0xC0000000);
	/* 25MHz ~ <33.75MHz */
	else
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0xC0000000, 0xC0000000);

	/* touch C282 when init HDMI by mode 720x576, 720x480, or other modes */
	if ((mode->hdisplay == 720) && (mode->vdisplay == 576))
		MMIO_WR(0xC280, 0x18232402);
	else if ((mode->hdisplay == 720) && (mode->vdisplay == 480))
		MMIO_WR(0xC280, 0x181f2402);
	else
		MMIO_WR(0xC280, 0x18330002);

	/* init C280 */
	reg_c280 = 0x18000002;
	/* sync polarity */
	if (mode->flags & DRM_MODE_FLAG_NHSYNC)
		reg_c280 |= BIT10;

	if (mode->flags & DRM_MODE_FLAG_NVSYNC)
		reg_c280 |= BIT13;

	/* regC280[4]: DVI mode enable: 0: HDMI mode, 1: DVI mode */
	reg_c280 &= ~0x40;
	/* caculate correct delay of regC280[22:16] */
	if ((mode->crtc_hsync_start - mode->crtc_hdisplay) > (58 - 11))
		delay = 0;
	else
		delay = 58 -
			(mode->crtc_hsync_start - mode->crtc_hdisplay) - 11;

	/* caculate max_packet */
	max_packet =
		(mode->crtc_hblank_end - mode->crtc_hsync_start -
		16 - 11 - delay) / 32;
	if (0 == delay)
		delay = mode->crtc_hblank_end - mode->crtc_hsync_start -
			(32 * max_packet + 16 + 11);

	reg_c280 |= (delay << 16);
	MMIO_WR(0xC280, reg_c280);
	/* C284 */
	reg_c284 = 0x00000102 | (max_packet << 28);
	/* power down to reset */
	MMIO_WR_MASK(0xC740, 0x00000000, 0x06000000);
	/* enable DP data pass */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x00000001, 0x00000001);
	/* select HDMI mode */
	MMIO_WR_MASK(0xC748, 0, BIT0);
	/* enable HDMI with HDMI mode */
	MMIO_WR_MASK(0xC280, 0x0, 0x40);
	/* select AC mode */
	MMIO_WR_MASK(0xC74C, 0x40, 0x40);
	/* enable InfoFrame */
	MMIO_WR(0xC284, reg_c284);
	/* set status of Lane0~3 */
	MMIO_WR_MASK(0xC744, 0x00FFFF82, 0x00FFFF82);
	MMIO_WR(0xC0B4, 0x12000000);
	/* enable audio packet */
	MMIO_WR_MASK(0xC294, 0x10000000, 0x10000000);
	/* enable InfoFrame */
	MMIO_WR(0xC284, reg_c284);
	MMIO_WR_MASK(0xC740, 0x1E4CBE7F, 0x3FFFFFFF);
	MMIO_WR_MASK(0xC748, 0x84509180, 0x001FFFFF);
	/* Select PHY Function as HDMI */
	/* Select HDTV0 source */
	if (IGA1 == iga_path)
		via_chrome9_write_vga_io_bits(REG_CRFF, BIT0, BIT1 + BIT0);
	else
		via_chrome9_write_vga_io_bits(REG_CRFF, BIT1 + BIT0,
			BIT1 + BIT0);
}

void via_chrome9_set_integrated_dvi_mode(
		struct drm_display_mode *mode,
		int iga_path)
{
	/* 135MHz ~ 270MHz */
	if (mode->clock >= 135000)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x00000000, 0xC0000000);
	/* 67.5MHz ~ <135MHz */
	else if (mode->clock >= 67500)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x40000000, 0xC0000000);
	/* 33.75MHz ~ <67.5MHz */
	else if (mode->clock >= 33750)
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x80000000, 0xC0000000);
	/* 25MHz ~ <33.75MHz */
	else
		MMIO_WR_MASK(DP_EPHY_PLL_REG, 0xC0000000, 0xC0000000);
	/* Power down TPLL to reset */
	MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x00000000, 0x06000000);
	/* Enable DP data pass */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x00000001, 0x00000001);
	/* Select EPHY as HDMI mode */
	MMIO_WR_MASK(DP_EPHY_MISC_PWR_REG, 0, BIT0);
	/* Enable HDMI with DVI mode */
	MMIO_WR_MASK(0xC280, 0x40, 0x40);
	/* select AC mode */
	MMIO_WR_MASK(0xC74C, 0x40, 0x40);
	/* Set status of Lane0~3 */
	MMIO_WR_MASK(0xC744, 0x00FFFF00, 0x00FFFF00);
	/* Disable InfoFrame */
	MMIO_WR_MASK(0xC284, 0x00000000, 0x00000002);
	/* EPHY Control Register */
	MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x1EC46E6F, 0x3FFFFFFF);
	/* Select PHY Function as HDMI */
	via_chrome9_write_vga_io_bits(REG_CRFF, BIT0, BIT0);
	/* Select HDTV0 source */
	if (IGA1 == iga_path)
		via_chrome9_write_vga_io_bits(REG_CRFF, 0, BIT1);
	else
		via_chrome9_write_vga_io_bits(REG_CRFF, BIT1, BIT1);

	/* in 640x480 case, MPLL is different */
	/* For VT3410 internal transmitter 640x480 issue */
	if (mode->hdisplay == 640 && mode->vdisplay == 480) {
		MMIO_WR(DP_EPHY_PLL_REG, 0xD8C29E6F);
		MMIO_WR(DP_EPHY_PLL_REG, 0xDEC29E6F);
	}
}

static void via_chrome9_internal_hdmi_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct drm_device *dev = encoder->dev;
	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	int iga_path = via_chrome9_crtc->crtc_id;

	/* Important: Use hskew to indicate cea mode id. Because the others
	parameters will be clear by Xorg. I have no choice, there is only this
	parameter can be used. This should be fixed when we found a better
	solution.	*/
	struct via_cea_timing_info_rec *cea_timing_info =
		&via_cea_timing_table[adjusted_mode->hskew - 1];

	via_chrome9_set_integrated_hdmi_mode(adjusted_mode, iga_path);
	if (IGA1 == iga_path)
		via_chrome9_load_crtc_pixel_timing(adjusted_mode);

	/* Set Hsync Offset, delay one clock (To meet 861-D spec.)*/
	via_chrome9_write_vga_io_bits(REG_CR8A, 0x01, 0x7);

	/* If CR8A +1, HSyc must -1 */
	via_chrome9_write_vga_io(REG_CR56,
		via_chrome9_read_vga_io(REG_CR56) - 1);
	via_chrome9_write_vga_io(REG_CR57,
		via_chrome9_read_vga_io(REG_CR57) - 1);

	if (adjusted_mode->flags & DRM_MODE_FLAG_INTERLACE) {
		if (IGA2 == iga_path) {
			switch (dev->pci_device) {
			case VX855_DEVICE:
				via_chrome9_write_vga_io_bits(REG_CRFB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRFC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				break;

			case VX900_DEVICE:
				via_chrome9_write_vga_io_bits(REG_CRAB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRAC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				break;

			default:
				via_chrome9_write_vga_io_bits(REG_CRFB,
					cea_timing_info->vsync_offset & 0xFF,
					0xFF);
				via_chrome9_write_vga_io_bits(REG_CRFC,
					(cea_timing_info->vsync_offset &
					0x700) >> 8,
					0x07);
				break;
			}
		}
	} else { /* non-interlace, clear interlace setting. */
		if (IGA2 == iga_path) {
			via_chrome9_write_vga_io_bits(REG_CRFB, 0, 0xFF);
			via_chrome9_write_vga_io_bits(REG_CRFC, 0, 0x07);
		}
	}
	/* via_chrome9_patch_dot_clock(encoder, adjusted_mode); */
}

static void via_chrome9_hdmi_enc_dpms(struct drm_encoder *encoder, int mode)
{
	struct drm_device *dev = encoder->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	switch (mode) {
	case DRM_MODE_DPMS_ON:
		KMS_DEBUG("HDMI on\n");
		/* enable bangap */
		MMIO_WR_MASK(0xC740,  0x1, 0x1);
		/* enable video */
		MMIO_WR_MASK(0xC640, 0x8, 0x8);
		/* enable HDMI */
		MMIO_WR_MASK(0xC280, 0x2, 0x2);

		/* turn on the cts patch timer. */
		CTS_PATCH_ON();

		break;

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		KMS_DEBUG("HDMI off\n");
		if (p_priv->is_ext_hdmi_interlace_mode) {
			/* HDMI can not be disable, that is 0xC280[1]=1b
			*  to let the half line function work.
			* Otherwise, the clock for interlace mode will not
			* correct.
			* This is a special setting for 410. */
		} else {
			/* disable HDMI */
			MMIO_WR_MASK(0xC280, 0x0, 0x2);
		}

		/* turn off the cts patch timer. */
		CTS_PATCH_OFF();

		break;
	}

	return;
}

bool via_hdmi_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct drm_device *dev = encoder->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_hdmi_options *via_hdmi_options = NULL;
	struct drm_connector *connector;
	struct via_chrome9_connector *via_chrome9_connector = NULL;

	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	int iga_path = via_chrome9_crtc->crtc_id;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		if (encoder ==  connector->encoder) {
			via_chrome9_connector =
				to_via_chrome9_connector(connector);
			break;
		}
	}
	if (!via_chrome9_connector) {
		KMS_DEBUG("this encoder is not used by any connector\n");
		return false;
	}

	/**
	 * initial crtc_scale_status to 0
	 * we should init it to 0 because when we use xrandr, switch
	 * downscale to non-scale, the igastatus VIA_SCALING_HW
	 * must be cleared, or there will be wrong.
	 */
	via_chrome9_crtc->crtc_scale_status = 0;

	via_hdmi_options = (struct via_hdmi_options *)
		via_chrome9_connector->options;

	via_chrome9_get_centering_timing(adjusted_mode,
		via_hdmi_options->left_border, via_hdmi_options->right_border,
		via_hdmi_options->top_border, via_hdmi_options->bottom_border);

	if ((0 != via_hdmi_options->left_border) ||
	    (0 != via_hdmi_options->right_border) ||
	    (0 != via_hdmi_options->top_border) ||
	    (0 != via_hdmi_options->bottom_border)) {
		if ((IGA2 == iga_path) && (p_priv->via_chrome9_chip_caps
			& CAPS_IGA2_DOWNSCALING)) {
			via_chrome9_crtc->crtc_scale_status = VIA_SCALING_HW;
		} else if ((IGA1 == iga_path) && (p_priv->via_chrome9_chip_caps
			& CAPS_IGA1_DOWNSCALING)) {
			/* TODO: we need to support overscan
			adjust function on IGA1. */
			/* via_chrome9_crtc->crtc_scale_status =
			VIA_SCALING_HW; */
		} else {
			KMS_DEBUG(
				"Error mode! This mode should be filtered.\n");
		}
	}

	return true;
}

static bool via_chrome9_internal_hdmi_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	return via_hdmi_mode_fixup(encoder, mode, adjusted_mode);
}

static bool via_chrome9_hdmi_enc_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	struct via_hdmi_encoder_private_info *via_hdmi_enc_info = NULL;
	via_hdmi_enc_info = (struct via_hdmi_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	via_chrome9_crtc->crtc_scale_status = 0;
	KMS_DEBUG("%d\n", via_hdmi_enc_info->connector_type);

	if (VIA_CHROME9_DEVICE_CONNECTOR_HDMI ==
		via_hdmi_enc_info->connector_type)
		return via_chrome9_internal_hdmi_mode_fixup(
			encoder, mode, adjusted_mode);
	else if (VIA_CHROME9_DEVICE_CONNECTOR_DVI_D ==
		via_hdmi_enc_info->connector_type)
		return true;

	return false;
}


static void via_chrome9_hdmi_enc_prepare(struct drm_encoder *encoder)
{
	via_chrome9_hdmi_enc_dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_hdmi_enc_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	struct via_hdmi_encoder_private_info *via_hdmi_enc_info = NULL;
	int iga_path = via_chrome9_crtc->crtc_id;
	via_hdmi_enc_info = (struct via_hdmi_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	if (VIA_CHROME9_DEVICE_CONNECTOR_HDMI ==
		via_hdmi_enc_info->connector_type)
		via_chrome9_internal_hdmi_mode_set(
			encoder, mode, adjusted_mode);
	else if (VIA_CHROME9_DEVICE_CONNECTOR_DVI_D ==
		via_hdmi_enc_info->connector_type)
		via_chrome9_set_integrated_dvi_mode(
			adjusted_mode, iga_path);

	KMS_DEBUG("%d\n", via_hdmi_enc_info->connector_type);
	return;
}

static void via_chrome9_hdmi_enc_commit(struct drm_encoder *encoder)
{
	via_chrome9_hdmi_enc_dpms(encoder, DRM_MODE_DPMS_ON);
}

static enum drm_connector_status via_chrome9_hdmi_enc_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	return connector_status_connected;
}

static const struct drm_encoder_helper_funcs
	via_chrome9_hdmi_enc_helper_funcs = {
	.dpms = via_chrome9_hdmi_enc_dpms,
	.mode_fixup = via_chrome9_hdmi_enc_mode_fixup,
	.prepare = via_chrome9_hdmi_enc_prepare,
	.mode_set = via_chrome9_hdmi_enc_mode_set,
	.commit = via_chrome9_hdmi_enc_commit,
	.detect = via_chrome9_hdmi_enc_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_hdmi_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

void via_chrome9_enable_dp_1(void)
{
	/* Must Enable DP data pass first */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x01, 0x01);
	MMIO_WR_MASK(DP_EPHY_PLL_REG, 0x0, 0xC0000000);
	/* if DP mode, video input selection - sp1 */
	MMIO_WR(DP_VIDEO_CTRL_REG, (MMIO_RD(DP_VIDEO_CTRL_REG) & 0x3FFC0001));
	/* enable DP and video */
	MMIO_WR(DP_ENABLE_IF_REG, MMIO_RD(DP_ENABLE_IF_REG) | 0x00000009);
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	KMS_DEBUG("enable dp1\n");
}

void via_chrome9_disable_dp_1(void)
{
	/* disable DP */
	MMIO_WR(DP_ENABLE_IF_REG, (MMIO_RD(DP_ENABLE_IF_REG) & 0xFFFFFFFE));
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	KMS_DEBUG("disable dp1\n");
}

void via_chrome9_dp_1_hor_width_and_tu(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int tu_size = 48;
	int width;
	int link_speed;

	link_speed = via_dp_encoder_private_info->link_speed;
	via_dp_encoder_private_info->tu_ratio  =
		via_chrome9_calculate_tu(link_speed, mode->clock, 24);
	width = (mode->crtc_hdisplay / 4) - 1;
	temp = via_dp_encoder_private_info->tu_ratio << 17 |
		(tu_size - 1) << 11 | width;
	MMIO_WR(DP_HWIDTH_TUSIZE_REG, temp);
}

void via_chrome9_dp_1_hor_line_duration(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int link_clock;
	int htotal;
	int hblank;
	/* units of Dclk and Ls_clk are MHz */
	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		link_clock = 162;
		break;
	case SPEED_270MHz:
		link_clock = 270;
		break;
	default:
		link_clock = 270;
		break;
	}
	hblank = ((((mode->crtc_htotal - mode->crtc_hdisplay) * link_clock) /
		(mode->clock / 1000)) - 15) & 0x00000FFF;
	htotal =
		((link_clock * mode->crtc_htotal / (mode->clock / 1000)) - 15) &
		0x00007FFF;

	temp = hblank << 15 | htotal;
	MMIO_WR(DP_HLINE_DUR_REG, temp);
}

void via_chrome9_dp_1_m_vid(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int link_clock;
	/* units of Dclk and Ls_clk are MHz */
	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		link_clock = 162;
		break;
	case SPEED_270MHz:
		link_clock = 270;
		break;
	default:
		link_clock = 270;
		break;
	}
	temp = ((mode->clock / 1000) * 32768 / link_clock) & 0x19FFFFFF;
	/* 8 bits, MMC6CC[31:29]=001b */
	temp |=  0x20000000;
	temp &=  0xFEFFFFFF;
	/* [24]=0: Asynchronous mode */
	MMIO_WR(DP_MVID_MISC0_REG, temp);
	/* Generate MVID in Asynchronous mode */
	MMIO_WR_MASK(DP_ENABLE_IF_REG, 0x02000000, 0x02000000);
}

void via_chrome9_dp_1_hor_ver_total(struct drm_display_mode *mode)
{
	u32 temp;

	temp = mode->crtc_vtotal << 16 | mode->crtc_htotal;
	MMIO_WR(DP_H_ATTR_REG, temp);
}

void via_chrome9_dp_1_hor_ver_offset(struct drm_display_mode *mode)
{
	u32 temp;

	temp = (mode->crtc_vtotal - mode->crtc_vsync_start) << 16 |
		(mode->crtc_htotal - mode->crtc_hsync_start);
	MMIO_WR(DP_HV_START_REG, temp);
}

void via_chrome9_dp_1_hor_ver_sync_width_and_polarity(
		struct drm_display_mode *mode)
{
	u32 temp;
	int hsync_polarity, vsync_polarity, vsync_interval, hsync_interval;

	if (mode->flags & DRM_MODE_FLAG_PHSYNC)
		hsync_polarity = POSITIVE;

	if (mode->flags & DRM_MODE_FLAG_NHSYNC)
		hsync_polarity = NEGATIVE;

	if (mode->flags & DRM_MODE_FLAG_PVSYNC)
		vsync_polarity = POSITIVE;

	if (mode->flags & DRM_MODE_FLAG_NVSYNC)
		vsync_polarity = NEGATIVE;

	vsync_interval = mode->crtc_vsync_end - mode->crtc_vsync_start;
	hsync_interval = mode->crtc_hsync_end - mode->crtc_hsync_start;

	temp = vsync_polarity << 31 | vsync_interval << 16 |
		hsync_polarity << 15 | hsync_interval;
	MMIO_WR(DP_POLARITY_WIDTH_REG, temp);
}

void via_chrome9_dp_1_hor_ver_active(struct drm_display_mode *mode)
{
	u32 temp;

	temp = mode->crtc_vdisplay << 16 | mode->crtc_hdisplay;
	MMIO_WR(DP_ACITVE_WH_REG, temp);
}

void via_chrome9_dp_1_n_vid(void)
{
	/* [23:0] Stream Clock Recovery :N */
	MMIO_WR_MASK(DP_ATTR_DATA_REG, 32768, 0x00FFFFFF);
}

void via_chrome9_dp_1_delay_reg(void)
{
	/* Default: [29:24] = 20h for delay */
	MMIO_WR_MASK(DP_VIDEO_CTRL_REG, 0x20000000, 0x3F000000);
}

void via_chrome9_dp_1_setup_main_link(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;

	KMS_DEBUG("via_chrome9_dp_setup_main_link\n");

	if (via_dp_encoder_private_info->link_speed) {
		/* Field ID flag not invert */
		MMIO_WR_MASK(DP_ENABLE_IF_REG, 0x0, BIT1);
		via_chrome9_dp_1_hor_width_and_tu(encoder, mode);
		via_chrome9_dp_1_hor_line_duration(encoder, mode);
		via_chrome9_dp_1_m_vid(encoder, mode);
		via_chrome9_dp_1_hor_ver_total(mode);
		via_chrome9_dp_1_hor_ver_offset(mode);
		via_chrome9_dp_1_hor_ver_sync_width_and_polarity(mode);
		via_chrome9_dp_1_hor_ver_active(mode);
		via_chrome9_dp_1_n_vid();
		via_chrome9_dp_1_delay_reg();

		KMS_DEBUG("End setup main link\n");
	}
}

static void via_chrome9_DP_1_enc_dpms(struct drm_encoder *encoder, int mode)
{
	switch (mode) {
	case DRM_MODE_DPMS_ON:
		via_chrome9_enable_dp_1();
		break;

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		via_chrome9_disable_dp_1();
		break;
	}
	return;
}

static bool via_chrome9_DP_1_enc_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	/* implement later if necessary */
	struct drm_crtc *crtc = encoder->crtc;
	struct via_chrome9_crtc *via_chrome9_crtc = to_via_chrome9_crtc(crtc);
	via_chrome9_crtc->crtc_scale_status = 0;
	return true;
}

static void via_chrome9_DP_1_enc_prepare(struct drm_encoder *encoder)
{
	via_chrome9_DP_1_enc_dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_DP_1_enc_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_crtc *via_chrome9_crtc =
		to_via_chrome9_crtc(encoder->crtc);
	int iga_path = via_chrome9_crtc->crtc_id;
	/* Important: Use hskew to indicate cea mode id. Because the others
	parameters will be clear by Xorg. I have no choice, there is only this
	parameter can be used. This should be fixed when we found a better
	solution.	*/
	/* disable video */
	MMIO_WR(DP_ENABLE_IF_REG, MMIO_RD(DP_ENABLE_IF_REG) & 0xFFFFFFF7);
	/* select DP mode */
	MMIO_WR_MASK(DP_EPHY_MISC_PWR_REG, 0x00000001, 0x00000001);
	/* DP channel selection */
	if (IGA1 == iga_path)
		via_chrome9_write_vga_io_bits(REG_CRFF, 0x00, BIT1);
	else
		via_chrome9_write_vga_io_bits(REG_CRFF, BIT1, BIT1);

	via_chrome9_dp_1_setup_main_link(encoder, adjusted_mode);
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	KMS_DEBUG("\n");
	return;
}

static void via_chrome9_DP_1_enc_commit(struct drm_encoder *encoder)
{
	via_chrome9_DP_1_enc_dpms(encoder, DRM_MODE_DPMS_ON);
}

static enum drm_connector_status via_chrome9_DP_1_enc_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	return connector_status_connected;
}

static const struct drm_encoder_helper_funcs
	via_chrome9_DP_1_enc_helper_funcs = {
	.dpms = via_chrome9_DP_1_enc_dpms,
	.mode_fixup = via_chrome9_DP_1_enc_mode_fixup,
	.prepare = via_chrome9_DP_1_enc_prepare,
	.mode_set = via_chrome9_DP_1_enc_mode_set,
	.commit = via_chrome9_DP_1_enc_commit,
	.detect = via_chrome9_DP_1_enc_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_DP_1_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

void via_chrome9_enable_dp_2(void)
{
	/* Must Enable DP data pass first */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x01, 0x01);
	/* if DP mode, video input selection - sp1 */
	MMIO_WR(DP2_VIDEO_CTRL_REG, (MMIO_RD(DP2_VIDEO_CTRL_REG) & 0x3FFC0001));
	/* enable DP and video */
	MMIO_WR(DP2_ENABLE_IF_REG, MMIO_RD(DP2_ENABLE_IF_REG) | 0x00000009);
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);

	KMS_DEBUG("enable dp2\n");
}

void via_chrome9_disable_dp_2(void)
{
	/* disable DP and video */
	MMIO_WR(DP2_ENABLE_IF_REG, MMIO_RD(DP2_ENABLE_IF_REG) & 0xFFFFFFF6);
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);

	KMS_DEBUG("disable dp2\n");
}

void via_chrome9_dp_2_hor_width_and_tu(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int tu_size = 48;
	int width;
	int link_speed;

	link_speed = via_dp_encoder_private_info->link_speed;
	via_dp_encoder_private_info->tu_ratio =
		via_chrome9_calculate_tu(link_speed, mode->clock, 24);
	width = (mode->crtc_hdisplay / 4) - 1;
	temp = via_dp_encoder_private_info->tu_ratio << 17 |
		(tu_size - 1) << 11 | width;
	MMIO_WR(DP2_HWIDTH_TUSIZE_REG, temp);
}

void via_chrome9_dp_2_hor_line_duration(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int link_clock;
	int htotal;
	int hblank;
	/* units of Dclk and Ls_clk are MHz */
	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		link_clock = 162;
		break;
	case SPEED_270MHz:
		link_clock = 270;
		break;
	default:
		link_clock = 270;
		break;
	}
	hblank = ((((mode->crtc_htotal - mode->crtc_hdisplay) * link_clock) /
		(mode->clock / 1000)) - 15) & 0x00000FFF;
	htotal =
		((link_clock * mode->crtc_htotal / (mode->clock / 1000)) - 15) &
		0x00007FFF;

	temp = hblank << 15 | htotal;
	MMIO_WR(DP2_HLINE_DUR_REG, temp);
}

void via_chrome9_dp_2_m_vid(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;
	u32 temp;
	int link_clock;

	/* units of Dclk and Ls_clk are MHz */
	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		link_clock = 162;
		break;
	case SPEED_270MHz:
		link_clock = 270;
		break;
	default:
		link_clock = 270;
		break;
	}
	temp = ((mode->clock / 1000) * 32768 / link_clock) & 0x19FFFFFF;
	/* 8 bits, MMC6CC[31:29]=001b */
	temp |=  0x20000000;
	temp &=  0xFEFFFFFF;
	/* [24]=0: Asynchronous mode */
	MMIO_WR(DP2_MVID_MISC0_REG, temp);
	/* Generate MVID in Asynchronous mode */
	MMIO_WR_MASK(DP2_ENABLE_IF_REG, 0x02000000, 0x02000000);
}

void via_chrome9_dp_2_hor_ver_total(struct drm_display_mode *mode)
{
	u32 temp;

	temp = mode->crtc_vtotal << 16 | mode->crtc_htotal;
	MMIO_WR(DP2_H_ATTR_REG, temp);
}

void via_chrome9_dp_2_hor_ver_offset(struct drm_display_mode *mode)
{
	u32 temp;

	temp = (mode->crtc_vtotal - mode->crtc_vsync_start) << 16 |
		(mode->crtc_htotal - mode->crtc_hsync_start);
	MMIO_WR(DP2_HV_START_REG, temp);
}

void via_chrome9_dp_2_hor_ver_sync_width_and_polarity(
		struct drm_display_mode *mode)
{
	u32 temp;
	int hsync_polarity, vsync_polarity, vsync_interval, hsync_interval;

	if (mode->flags & DRM_MODE_FLAG_PHSYNC)
		hsync_polarity = POSITIVE;

	if (mode->flags & DRM_MODE_FLAG_NHSYNC)
		hsync_polarity = NEGATIVE;

	if (mode->flags & DRM_MODE_FLAG_PVSYNC)
		vsync_polarity = POSITIVE;

	if (mode->flags & DRM_MODE_FLAG_NVSYNC)
		vsync_polarity = NEGATIVE;

	vsync_interval = mode->crtc_vsync_end - mode->crtc_vsync_start;
	hsync_interval = mode->crtc_hsync_end - mode->crtc_hsync_start;

	temp = vsync_polarity << 31 | vsync_interval << 16 |
		hsync_polarity << 15 | hsync_interval;
	MMIO_WR(DP2_POLARITY_WIDTH_REG, temp);
}

void via_chrome9_dp_2_hor_ver_active(struct drm_display_mode *mode)
{
	u32 temp;

	temp = mode->crtc_vdisplay << 16 | mode->crtc_hdisplay;
	MMIO_WR(DP2_ACITVE_WH_REG, temp);
}

void via_chrome9_dp_2_n_vid(void)
{
	/* [23:0] Stream Clock Recovery :N */
	MMIO_WR_MASK(DP2_NVID_MISC0_REG, 32768, 0x00FFFFFF);
}

void via_chrome9_dp_2_delay_reg(void)
{
	/* Default: [29:24] = 20h for delay */
	MMIO_WR_MASK(DP2_VIDEO_CTRL_REG, 0x20000000, 0x3F000000);
}

void via_chrome9_dp_2_setup_main_link(struct drm_encoder *encoder,
		struct drm_display_mode *mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	struct via_dp_encoder_private_info *via_dp_encoder_private_info =
		(struct via_dp_encoder_private_info *)
		via_chrome9_encoder->encoder_private;

	KMS_DEBUG("via_chrome9_dp_2_setup_main_link\n");

	if (via_dp_encoder_private_info->link_speed) {
		/* Field ID flag not invert */
		MMIO_WR_MASK(DP_ENABLE_IF_REG, 0x0, BIT1);
		via_chrome9_dp_2_hor_width_and_tu(encoder, mode);
		via_chrome9_dp_2_hor_line_duration(encoder, mode);
		via_chrome9_dp_2_m_vid(encoder, mode);
		via_chrome9_dp_2_hor_ver_total(mode);
		via_chrome9_dp_2_hor_ver_offset(mode);
		via_chrome9_dp_2_hor_ver_sync_width_and_polarity(mode);
		via_chrome9_dp_2_hor_ver_active(mode);
		via_chrome9_dp_2_n_vid();
		via_chrome9_dp_2_delay_reg();

		KMS_DEBUG("End setup main link\n");
	}
}

static void via_chrome9_DP_2_enc_dpms(struct drm_encoder *encoder, int mode)
{
	switch (mode) {
	case DRM_MODE_DPMS_ON:
		via_chrome9_enable_dp_2();
		break;

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		via_chrome9_disable_dp_2();
		break;
	}
	return;
}

static bool via_chrome9_DP_2_enc_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	/* implement later if necessary */
	struct drm_crtc *crtc = encoder->crtc;
	struct via_chrome9_crtc *via_chrome9_crtc = to_via_chrome9_crtc(crtc);
	via_chrome9_crtc->crtc_scale_status = 0;
	return true;
}

static void via_chrome9_DP_2_enc_prepare(struct drm_encoder *encoder)
{
	via_chrome9_DP_2_enc_dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_DP_2_enc_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_crtc *via_chrome9_crtc =
					to_via_chrome9_crtc(encoder->crtc);
	int iga_path = via_chrome9_crtc->crtc_id;
	/* disable video */
	MMIO_WR(DP2_ENABLE_IF_REG, MMIO_RD(DP2_ENABLE_IF_REG) & 0xFFFFFFF7);
	/* DP channel selection */
	if (IGA1 == iga_path)
		via_chrome9_write_vga_io_bits(REG_CRFF, 0x00, BIT2);
	else
		via_chrome9_write_vga_io_bits(REG_CRFF, BIT2, BIT2);

	via_chrome9_dp_2_setup_main_link(encoder, adjusted_mode);
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
	return;
}

static void via_chrome9_DP_2_enc_commit(struct drm_encoder *encoder)
{
	via_chrome9_DP_2_enc_dpms(encoder, DRM_MODE_DPMS_ON);
}

static enum drm_connector_status via_chrome9_DP_2_enc_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	return connector_status_connected;
}

static const struct drm_encoder_helper_funcs
	via_chrome9_DP_2_enc_helper_funcs = {
	.dpms = via_chrome9_DP_2_enc_dpms,
	.mode_fixup = via_chrome9_DP_2_enc_mode_fixup,
	.prepare = via_chrome9_DP_2_enc_prepare,
	.mode_set = via_chrome9_DP_2_enc_mode_set,
	.commit = via_chrome9_DP_2_enc_commit,
	.detect = via_chrome9_DP_2_enc_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_DP_2_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

static void via_chrome9_tmds_dpms(struct drm_encoder *encoder, int mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	u32 di_port = via_chrome9_encoder->di_port;
	struct drm_device *dev = encoder->dev;
	int chipset = dev->pci_device;
	struct drm_connector *connector;
	struct via_chrome9_connector *via_chrome9_connector = NULL;
	struct via_chrome9_crtc *via_chrome9_crtc;
	enum drm_connector_status con_status = connector_status_disconnected;
	int iga_path;

	KMS_DEBUG("\n");
	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		if (encoder ==  connector->encoder) {
			via_chrome9_connector =
				to_via_chrome9_connector(connector);
			break;
		}
	}
	if (!via_chrome9_connector) {
		KMS_DEBUG(
			"this encoder is not used by any connector,dpms mode is %d\n",
			mode);
		return;
	} else {
		KMS_DEBUG("dpms mode is %d\n", mode);
	}

	if (via_chrome9_connector->detect)
		con_status = via_chrome9_connector->detect(connector, false);

	switch (mode) {
	case DRM_MODE_DPMS_ON:
	{
		if (encoder->crtc == NULL)
			return;
		via_chrome9_crtc = to_via_chrome9_crtc(encoder->crtc);
		iga_path = via_chrome9_crtc->crtc_id;
		via_chrome9_write_vga_io_bits(REG_CRD2, BIT4, BIT4);
		via_chrome9_write_vga_io_bits(REG_CR91, 0x00, BIT0);
		via_chrome9_digital_port_pad_on(di_port, iga_path, chipset);
		/* Internal TMDS only use DFP_L */
		/* Turn on DVI panel path(Only for internal), otherwise,
		the screen of DVI will be black. */
		via_chrome9_write_vga_io_bits(REG_CR91, 0x00, BIT7);
		/* Power on TMDS */
		via_chrome9_write_vga_io_bits(REG_CRD2, 0x00, BIT3);
		break;
	}
	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		if (con_status == connector_status_disconnected)
			return;
		via_chrome9_write_vga_io_bits(REG_CR91, 0x80, BIT7);
		/* Power off TMDS */
		via_chrome9_write_vga_io_bits(REG_CRD2, 0x08, BIT3);
		via_chrome9_digital_port_pad_off(di_port);
		break;
	}
	return;
}

static bool via_chrome9_tmds_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	/* implement later if necessary */
	struct drm_crtc *crtc = encoder->crtc;
	struct via_chrome9_crtc *via_chrome9_crtc = to_via_chrome9_crtc(crtc);
	via_chrome9_crtc->crtc_scale_status = 0;
	return true;
}

static void via_chrome9_tmds_prepare(struct drm_encoder *encoder)
{
	via_chrome9_tmds_dpms(encoder, DRM_MODE_DPMS_OFF);
}

static void via_chrome9_tmds_mode_set(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_encoder *via_chrome9_encoder =
		to_via_chrome9_encoder(encoder);
	u32 di_port = via_chrome9_encoder->di_port;

	via_chrome9_set_digital_port_sync_polarity(di_port, adjusted_mode);
	return;
}

static void via_chrome9_tmds_commit(struct drm_encoder *encoder)
{
	via_chrome9_tmds_dpms(encoder, DRM_MODE_DPMS_ON);
}

static enum drm_connector_status via_chrome9_tmds_detect(
		struct drm_encoder *encoder,
		struct drm_connector *connector)
{
	return connector_status_connected;
}

static const struct drm_encoder_helper_funcs via_chrome9_tmds_helper_funcs = {
	.dpms = via_chrome9_tmds_dpms,
	.mode_fixup = via_chrome9_tmds_mode_fixup,
	.prepare = via_chrome9_tmds_prepare,
	.mode_set = via_chrome9_tmds_mode_set,
	.commit = via_chrome9_tmds_commit,
	.detect = via_chrome9_tmds_detect,
	.disable = via_chrome9_encoder_disable,
};

static const struct drm_encoder_funcs via_chrome9_tmds_enc_funcs = {
	.destroy = via_chrome9_enc_destroy,
};

void via_chrome9_add_encoder(
		struct drm_device *dev, int encoder_id,
		enum via_chrome9_encoder_type type,
		uint32_t supported_connect_types)
{
	struct drm_encoder *encoder;
	struct via_chrome9_encoder *via_chrome9_encoder;
	struct via_dp_encoder_private_info
		*via_dp_1_encoder_private_info = NULL;
	struct via_dp_encoder_private_info
		*via_dp_2_encoder_private_info = NULL;
	struct via_hdmi_encoder_private_info
		*via_hdmi_encoder_private_info = NULL;
	int encoder_index;

	list_for_each_entry(encoder, &dev->mode_config.encoder_list, head) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		if ((via_chrome9_encoder->encoder_id == encoder_id) &&
			(via_chrome9_encoder->encoder_type == type)) {
			via_chrome9_encoder->supported_connect_types |=
				supported_connect_types;
			return;
		}
	}

	via_chrome9_encoder = kzalloc(sizeof(struct via_chrome9_encoder),
		GFP_KERNEL);
	if (!via_chrome9_encoder) {
		DRM_INFO("allocate the encoder error\n");
		return;
	}

	via_chrome9_encoder->encoder_id = encoder_id;
	via_chrome9_encoder->supported_connect_types = supported_connect_types;
	via_chrome9_encoder->encoder_type = type;

	encoder = &via_chrome9_encoder->base;
	encoder->possible_crtcs = 0x3;

	switch (via_chrome9_encoder->encoder_id) {
	case VIA_CHROME9_DEVICE_ENCODER_DAC:
		encoder_index = VIA_CHROME9_ENCODER_DAC_INDEX;
		via_chrome9_encoder->di_port = DISP_DI_NONE;
		drm_encoder_init(dev, encoder,
			&via_chrome9_dac_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_dac_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_LVDS:
		encoder_index = VIA_CHROME9_ENCODER_LVDS_INDEX;
		encoder->possible_crtcs = 0x2;
		via_chrome9_encoder->di_port = DISP_DI_DFPL;
		drm_encoder_init(dev, encoder,
			&via_chrome9_lvds_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_lvds_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_HDMI:
		via_hdmi_encoder_private_info = kzalloc(
			sizeof(struct via_hdmi_encoder_private_info),
			GFP_KERNEL);
		if (!via_hdmi_encoder_private_info) {
			DRM_INFO("allocate the hdmi private info error\n");
			return;
		} else {
			via_chrome9_encoder->encoder_private =
				(void *)via_hdmi_encoder_private_info;
		}
		via_hdmi_encoder_private_info->dp_1_ephy_mode = via_unknow_mode;
		/* need to check other hdmi enabled or not,
		and modify the possiable crtc,
		the first hdmi default is 0x2 and the second hdmi is 0x1 */
		encoder_index = VIA_CHROME9_ENCODER_HDMI_INDEX;
		encoder->possible_crtcs = 0x2;
		via_chrome9_encoder->di_port = DISP_DI_NONE;
		drm_encoder_init(dev, encoder,
			&via_chrome9_hdmi_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_hdmi_enc_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_DP_1:
		via_dp_1_encoder_private_info = kzalloc(
			sizeof(struct via_dp_encoder_private_info),
			GFP_KERNEL);
		if (!via_dp_1_encoder_private_info) {
			DRM_INFO("allocate the dp1 private info error\n");
			return;
		} else {
			via_chrome9_encoder->encoder_private =
				(void *)via_dp_1_encoder_private_info;
		}
		via_dp_1_encoder_private_info->dp_1_ephy_mode = via_unknow_mode;
		encoder_index = VIA_CHROME9_ENCODER_DP_1_INDEX;
		drm_encoder_init(dev, encoder,
			&via_chrome9_DP_1_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_DP_1_enc_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_DP_2:
		via_dp_2_encoder_private_info = kzalloc(
			sizeof(struct via_dp_encoder_private_info),
			GFP_KERNEL);
		if (!via_dp_2_encoder_private_info) {
			DRM_INFO("allocate the dp2 private info error\n");
			return;
		} else {
			via_chrome9_encoder->encoder_private =
				(void *)via_dp_2_encoder_private_info;
		}
		encoder_index = VIA_CHROME9_ENCODER_DP_2_INDEX;
		drm_encoder_init(dev, encoder,
			&via_chrome9_DP_2_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_DP_2_enc_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_TMDS:
		encoder_index = VIA_CHROME9_ENCODER_TMDS_INDEX;
		via_chrome9_encoder->di_port = DISP_DI_DFPL;
		drm_encoder_init(dev, encoder,
			&via_chrome9_tmds_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_tmds_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_LVDS_2:
		encoder_index = VIA_CHROME9_ENCODER_LVDS_2_INDEX;
		encoder->possible_crtcs = 0x2;
		via_chrome9_encoder->di_port = DISP_DI_DFPH;
		drm_encoder_init(dev, encoder,
			&via_chrome9_lvds_enc_funcs,
			via_chrome9_encoder_convert[encoder_index]);
		drm_encoder_helper_add(encoder,
			&via_chrome9_lvds_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_ENCODER_TV:
		break;
	default:
		DRM_INFO("invalid encoder type\n");
		break;
	}

}

/*
 * add the internal encoder
 */
int via_chrome9_internal_encoder_init(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct internal_encoder_list encoder_prementer;
	int encoder_type, supported_connect_types, i;

	for (i = 0; i < ARRAY_SIZE(via_chrome9_internal_encoder_list); i++) {
		encoder_prementer = via_chrome9_internal_encoder_list[i];
		if (!(p_priv->mode_info.chip_index &
			encoder_prementer.support_chip_id))
			continue;
		encoder_type = encoder_prementer.encoder_type;

		KMS_DEBUG(" i=%x  encoder_type=%x  name=%s\n",
			i, encoder_type, encoder_prementer.name);

		switch (encoder_type) {
		case VIA_CHROME9_DEVICE_ENCODER_DAC:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_CRT;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_CRT,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_LVDS:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_LCD;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_LCD,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_HDMI:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_HDMI |
				VIA_CHROME9_DEVICE_CONNECTOR_HDMI_DVI_D;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_HDMI,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_HDMI_DVI_D,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_DP_1:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_DP_1;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_DP_1,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_DP_2:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_DP_2;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_DP_2,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_TMDS:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_DVI_D,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_LVDS_2:
			supported_connect_types =
				VIA_CHROME9_DEVICE_CONNECTOR_LCD_2;
			via_chrome9_add_encoder(dev, encoder_type,
				VIA_CHROME9_INTERNAL_ENCODER,
				supported_connect_types);
			via_chrome9_add_connector(dev,
				VIA_CHROME9_DEVICE_CONNECTOR_LCD_2,
				VIA_CHROME9_INTERNAL_CONNECTOR);
			break;
		case VIA_CHROME9_DEVICE_ENCODER_TV:
			break;
		default:
			DRM_INFO("unknow encoder type\n");
			break;
		}
	}
	return 0;
}
