/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/list.h>
#include <linux/string.h>

#include "via_chrome9_drv.h"
#include "via_chrome9_i2c.h"
#include "via_chrome9_external_card.h"


static struct via_chrome9_external_card_ops vt1625_ops;

static DEFINE_MUTEX(cards_mutex);
static LIST_HEAD(card_list);
static struct drm_device *via_chrome9_drm_dev;
static const struct i2c_device_id via_chrome9_external_card_ids[] = {
	{ "ad9389", 0 },
	{ "sil164", 0 },
	{ "vt1625", 0 },
	{ "vt3192", 0 },
	{ "vt3345", 0 },
	{ "ch7301_ec", 0 },
	{ "ch7301_ea", 0 },
	{}
};

struct same_slave_card_table {
	u16 reg;
	u8 id;
	char name[10];
};

struct card_table_info {
	int slave_addr;
	int num;
	struct same_slave_card_table table[2];
};

static struct card_table_info via_chrome9_external_card_infos[] = {
	#include "external_card_info.h"
};

#define NUM_OF_EXTERN_CARD_ADDR ARRAY_SIZE(via_chrome9_external_card_infos)

static const short via_chrome9_external_card_addrs[] = {0x08, 0x39,
	0x38, 0x40, 0x76, 0x75, I2C_CLIENT_END};

/**************************************************************************/
/*                                   VT3353 DPA Setting                   */
/**************************************************************************/

/* DVP0, DVP0DataDriving, DVP0ClockDriving,
DVP1, DVP1Driving, DFPHigh, DFPLow */
struct via_chrome9_gfx_dpa_value vt3353_default_dpa[] = {
	/* CR96, SR2A[5], SR1B[1], SR2A[4], SR1E[2], CR9B, SR65, CR97, CR99 */
	{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } };

struct via_chrome9_gfx_dpa_value vt3353_clk_50_70m_dpa[] = {
	/* CR96, SR2A[5], SR1B[1], SR2A[4], SR1E[2], CR9B, SR65, CR97, CR99 */
	{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00 } };

struct via_chrome9_gfx_dpa_value vt3353_clk_100_150m_dpa[] = {
	/* CR96, SR2A[5], SR1B[1], SR2A[4], SR1E[2], CR9B, SR65, CR97, CR99 */
	{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00 } };

struct via_chrome9_gfx_dpa_info_table gfx_dpa_vt3353_vt1636[] = {
	{ DPA_CLK_RANGE_30M, vt3353_default_dpa },
	{ DPA_CLK_RANGE_30_50M, vt3353_default_dpa },
	{ DPA_CLK_RANGE_50_70M, vt3353_clk_50_70m_dpa },
	{ DPA_CLK_RANGE_70_100M, vt3353_default_dpa },
	{ DPA_CLK_RANGE_100_150M, vt3353_clk_100_150m_dpa },
	{ DPA_CLK_RANGE_150M, vt3353_default_dpa }
};

struct via_chrome9_gfx_dpa_index_table gfx_dpa_index_table[] = {
	{ VX800_DEVICE, NULL, NULL, gfx_dpa_vt3353_vt1636, NULL },
	{ VX855_DEVICE, NULL, NULL, gfx_dpa_vt3353_vt1636, NULL },
	{ VX900_DEVICE, NULL, NULL, gfx_dpa_vt3353_vt1636, NULL },
	{ 0xffffffff, NULL, NULL, NULL, NULL }
};
/* for lcd dpa. compute clock range */
int via_chrome9_get_clk_range_index(u32 clk)
{
	if (clk < DPA_CLK_30M)
		return DPA_CLK_RANGE_30M;
	else if ((DPA_CLK_30M < clk) && (clk < DPA_CLK_50M))
		return DPA_CLK_RANGE_30_50M;
	else if ((DPA_CLK_50M < clk) && (clk < DPA_CLK_70M))
		return DPA_CLK_RANGE_50_70M;
	else if ((DPA_CLK_70M < clk) && (clk < DPA_CLK_100M))
		return DPA_CLK_RANGE_70_100M;
	else if ((DPA_CLK_100M < clk) && (clk < DPA_CLK_150M))
		return DPA_CLK_RANGE_100_150M;
	else
		return DPA_CLK_RANGE_150M;
}

void via_chrome9_set_dpa(u32 port,
		struct via_chrome9_gfx_dpa_value *gfx_dpa_value)
{
	KMS_DEBUG("via_chrome9_set_dpa, use port %x.\n", port);

	switch (port) {
	case DISP_DI_DVP0:
		/* DVP0 Clock Polarity and Adjust: */
		via_chrome9_write_vga_io_bits(REG_CR96,
			gfx_dpa_value->dvp0, 0x0F);
		/* DVP0 Clock and Data Pads Driving: */
		via_chrome9_write_vga_io_bits(REG_SR1E,
			gfx_dpa_value->dvp0_clock_dri_s, BIT2);
		via_chrome9_write_vga_io_bits(REG_SR2A,
			gfx_dpa_value->dvp0_clock_dri_s1, BIT4);
		via_chrome9_write_vga_io_bits(REG_SR1B,
			gfx_dpa_value->dvp0_data_dri_s, BIT1);
		via_chrome9_write_vga_io_bits(REG_SR2A,
			gfx_dpa_value->dvp0_data_dri_s1, BIT5);
		break;
	case DISP_DI_DVP1:
		/* DVP1 Clock Polarity and Adjust: */
		via_chrome9_write_vga_io_bits(REG_CR9B,
			gfx_dpa_value->dvp1, 0x0F);
		/* DVP1 Clock and Data Pads Driving: */
		via_chrome9_write_vga_io_bits(REG_SR65,
			gfx_dpa_value->dvp1_driving, 0x0F);
		break;
	case DISP_DI_DFPH:
		via_chrome9_write_vga_io_bits(REG_CR97,
			gfx_dpa_value->dfp_high, 0x0F);
		break;
	case DISP_DI_DFPL:
		via_chrome9_write_vga_io_bits(REG_CR99,
			gfx_dpa_value->dfp_low, 0x0F);
		break;
	case DISP_DI_DFP:
		via_chrome9_write_vga_io_bits(REG_CR97,
			gfx_dpa_value->dfp_high, 0x0F);
		via_chrome9_write_vga_io_bits(REG_CR99,
			gfx_dpa_value->dfp_low, 0x0F);
		break;
	default:
		break;
	}
}

static void via_chrome9_patch_dvp1_dpa(int chipset, char *sub_chip_name)
{
	switch (chipset) {
	case VX900_DEVICE:
		if (!strcmp(sub_chip_name, "sil164") ||
		 !strcmp(sub_chip_name, "vt3192") ||
		 !strcmp(sub_chip_name, "ad9389") ||
		 !strcmp(sub_chip_name, "ch7301_ec") ||
		 !strcmp(sub_chip_name, "ch7301_ea"))
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x0A, 0x0F);
		break;

	default:
		break;
	}
}

static void via_chrome9_load_default_dpa_setting(int chipset, u32 port,
		struct drm_display_mode *mode,
		char *sub_chip_name)
{
	switch (port) {
	case DISP_DI_DVP1:
		via_chrome9_patch_dvp1_dpa(chipset, sub_chip_name);
		break;

	case DISP_DI_DVP0:
	case DISP_DI_DFPL:
		break;

	default:
		break;
	}
}


static int via_chrome9_get_external_card_device_info(
		struct i2c_client *client,
		char *name)
{
	int i, num = 0;
	struct same_slave_card_table *table = NULL;
	u8 data = 0;
	struct via_chrome9_i2c_chan *chan = container_of(client->adapter,
		struct via_chrome9_i2c_chan, adapter);

	for (i = 0; i < NUM_OF_EXTERN_CARD_ADDR; i++) {
		if (client->addr ==
			via_chrome9_external_card_infos[i].slave_addr) {
			table = via_chrome9_external_card_infos[i].table;
			num = via_chrome9_external_card_infos[i].num;
			break;
		}
	}

	/* speacilize for ad9389 */
	if (client->addr == 0x39) {
		strcpy(name, table[0].name);
		return 0;
	}

	for (i = 0; i < num; i++) {
		via_chrome9_i2c_readbytes(chan,
			client->addr, table[i].reg, &data, 1);
		if (data == table[i].id)
			break;
	}
	if (i == num)
		return -1;

	strcpy(name, table[i].name);

	return 0;
}

static void via_chrome9_set_external_card_i2c_info(
		struct i2c_board_info *info,
		char *name)
{
	strcpy(info->type, name);
}

static int via_chrome9_external_card_i2c_detect(struct i2c_client *client,
		struct i2c_board_info *info)
{
	char name[10];
	int ret;
	KMS_DEBUG("enter\n");

	ret = via_chrome9_get_external_card_device_info(client, name);
	if (ret)
		return ret;

	via_chrome9_set_external_card_i2c_info(info, name);

	return 0;
}

void via_chrome9_load_user_gfx_dpa_setting(u32 port,
		struct via_gfx_dpa *via_gfx_dpa)
{
	switch (port) {
	case DISP_DI_DVP0:
		if (via_gfx_dpa->is_clk_polarity_used)
			via_chrome9_write_vga_io_bits(REG_CR96,
				via_gfx_dpa->clk_polarity << 3, BIT3);
		if (via_gfx_dpa->is_clk_adjust_used)
			via_chrome9_write_vga_io_bits(REG_CR96,
				via_gfx_dpa->clk_adjust, 0x07);
		if (via_gfx_dpa->is_clk_driving_sel_used) {
			/* pVal->clkDrivingSel[1:0],
			Bit0's value is S0, bit1's value is S1 */
			via_chrome9_write_vga_io_bits(REG_SR1E,
				via_gfx_dpa->clk_driving_sel << 2, BIT2);
			via_chrome9_write_vga_io_bits(REG_SR2A,
				via_gfx_dpa->clk_driving_sel << 4, BIT4);
		}
		if (via_gfx_dpa->is_data_driving_sel_used) {
			/* pVal->clkDrivingSel[1:0],
			Bit0's value is S0, bit1's value is S1 */
			via_chrome9_write_vga_io_bits(REG_SR1B,
				via_gfx_dpa->data_driving_sel << 1, BIT1);
			via_chrome9_write_vga_io_bits(REG_SR2A,
				via_gfx_dpa->data_driving_sel << 5, BIT5);
		}
		break;
	case DISP_DI_DVP1:
		if (via_gfx_dpa->is_clk_polarity_used)
			via_chrome9_write_vga_io_bits(REG_CR9B,
				via_gfx_dpa->clk_polarity << 3, BIT3);
		if (via_gfx_dpa->is_clk_adjust_used)
			via_chrome9_write_vga_io_bits(REG_CR9B,
				via_gfx_dpa->clk_adjust, 0x07);
		if (via_gfx_dpa->is_clk_driving_sel_used)
			via_chrome9_write_vga_io_bits(REG_SR65,
				via_gfx_dpa->clk_driving_sel << 2, 0x0C);
		if (via_gfx_dpa->is_data_driving_sel_used)
			via_chrome9_write_vga_io_bits(REG_SR65,
				via_gfx_dpa->data_driving_sel, 0x03);
		break;

	case DISP_DI_DFPH:
		if (via_gfx_dpa->is_clk_polarity_used)
			via_chrome9_write_vga_io_bits(REG_CR97,
				via_gfx_dpa->clk_polarity << 3, BIT3);
		if (via_gfx_dpa->is_clk_adjust_used)
			via_chrome9_write_vga_io_bits(REG_CR97,
				via_gfx_dpa->clk_adjust, 0x07);
		break;

	case DISP_DI_DFPL:
		if (via_gfx_dpa->is_clk_polarity_used)
			via_chrome9_write_vga_io_bits(REG_CR99,
				via_gfx_dpa->clk_polarity << 3, BIT3);
		if (via_gfx_dpa->is_clk_adjust_used)
			via_chrome9_write_vga_io_bits(REG_CR99,
				via_gfx_dpa->clk_adjust, 0x07);
		break;

	case DISP_DI_DFP:
		if (via_gfx_dpa->is_clk_polarity_used) {
			via_chrome9_write_vga_io_bits(REG_CR97,
				via_gfx_dpa->clk_polarity << 3, BIT3);
			via_chrome9_write_vga_io_bits(REG_CR99,
				via_gfx_dpa->clk_polarity << 3, BIT3);
		}
		if (via_gfx_dpa->is_clk_adjust_used) {
			via_chrome9_write_vga_io_bits(REG_CR97,
				via_gfx_dpa->clk_adjust, 0x07);
			via_chrome9_write_vga_io_bits(REG_CR99,
				via_gfx_dpa->clk_adjust, 0x07);
		}
		break;
	default:
		break;
	}
}

static void via_chrome9_load_lvds_gfx_default_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	int i, clk_range_index;
	int clock = mode->clock * 1000;
	struct via_chrome9_gfx_dpa_info_table *gfx_dpa_info_table = NULL;
	struct via_chrome9_gfx_dpa_value *gfx_dpa_value = NULL;
	struct drm_device *dev = card->drm_dev;

	for (i = 0; gfx_dpa_index_table[i].chipset != -1; i++) {
		if (gfx_dpa_index_table[i].chipset == dev->pci_device)
			break;
	}

	if (-1 == gfx_dpa_index_table[i].chipset)
		return;

	clk_range_index = via_chrome9_get_clk_range_index(clock);
	gfx_dpa_info_table = gfx_dpa_index_table[i].vt1636_dpa_tbl;

	if (gfx_dpa_info_table)
		gfx_dpa_value =
			gfx_dpa_info_table[clk_range_index].gfx_dpa_value;

	if (gfx_dpa_value)
		via_chrome9_set_dpa(card->di_port, gfx_dpa_value);

}

/* set lvds subchip dpa setting*/
void via_chrome9_load_user_lvds_dpa_setting(
		struct via_chrome9_external_card *card,
		struct via_lvds_dpa *via_lvds_dpa)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	if (!strcmp(card->name, "vt3345")) {
		if (via_lvds_dpa->is_vt_1636_clk_sel_st1_used)
			via_chrome9_external_card_write_mask(client,
				0x09, via_lvds_dpa->vt1636_clk_sel_st1, 0x1F);
		if (via_lvds_dpa->is_vt_1636_clk_sel_st2_used)
			via_chrome9_external_card_write_mask(client,
				0x08, via_lvds_dpa->vt1636_clk_sel_st2, 0x0F);
	}
}

static void via_load_hdmi_default_dpa_setting(u32 chipset, u32 di_port)
{
	if (di_port == DISP_DI_DVP1) {
		switch (chipset) {
		case VX800_DEVICE:
			via_chrome9_write_vga_io_bits(REG_SR65, 0x0B, 0x0F);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x0F, 0x0F);
			break;

		case VX855_DEVICE:
			via_chrome9_write_vga_io_bits(REG_SR65, 0x0B, 0x0F);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x0F, 0x0F);
			break;

		case VX900_DEVICE:
			/* Sync VBIOS DPA setting */
			via_chrome9_write_vga_io_bits(REG_SR65, 0x09, 0x0F);
			via_chrome9_write_vga_io_bits(REG_CR9B, 0x09, 0x0F);
			break;

		default:
			break;
		}
	}
}

void via_chrome9_vga_gfx_dpa_setting(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	u32 di_port = card->di_port;
	struct drm_device *drm_dev = card->drm_dev;
	u32 chip_set = drm_dev->pci_device;
	char *sub_chip_name = card->name;
	struct via_vga_options *via_vga_options = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_connector *current_con = card->current_con;
	struct via_chrome9_connector *via_con = NULL;

	if (!current_con) {
		KMS_DEBUG("%s encoder has no connector\n", card->name);
		return;
	} else {
		via_con = to_via_chrome9_connector(current_con);
		via_vga_options = (struct via_vga_options *)via_con->options;
	}

	via_gfx_dpa.clk_adjust = via_vga_options->clk_adjust;
	via_gfx_dpa.clk_driving_sel = via_vga_options->clk_driving_sel;
	via_gfx_dpa.clk_polarity = via_vga_options->clk_polarity;
	via_gfx_dpa.data_driving_sel = via_vga_options->data_driving_sel;
	via_gfx_dpa.is_clk_adjust_used = via_vga_options->is_clk_adjust_used;
	via_gfx_dpa.is_clk_driving_sel_used =
		via_vga_options->is_clk_driving_sel_used;
	via_gfx_dpa.is_clk_polarity_used =
		via_vga_options->is_clk_polarity_used;
	via_gfx_dpa.is_data_driving_sel_used =
		via_vga_options->is_data_driving_sel_used;
	via_chrome9_load_default_dpa_setting(chip_set, di_port, mode,
		sub_chip_name);
	via_chrome9_load_user_gfx_dpa_setting(di_port, &via_gfx_dpa);
}


void via_chrome9_lvds_gfx_dpa_setting(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	u32 di_port = card->di_port;
	struct via_lcd_options *via_lcd_options = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct via_lvds_dpa via_lvds_dpa;
	struct drm_connector *current_con = card->current_con;
	struct via_chrome9_connector *via_con = NULL;

	if (!current_con) {
		KMS_DEBUG("%s encoder has no connector\n", card->name);
		return;
	} else {
		via_con = to_via_chrome9_connector(current_con);
		via_lcd_options = (struct via_lcd_options *)via_con->options;
	}

	via_gfx_dpa.clk_adjust = via_lcd_options->clk_adjust;
	via_gfx_dpa.clk_driving_sel = via_lcd_options->clk_driving_sel;
	via_gfx_dpa.clk_polarity = via_lcd_options->clk_polarity;
	via_gfx_dpa.data_driving_sel = via_lcd_options->data_driving_sel;
	via_gfx_dpa.is_clk_adjust_used = via_lcd_options->is_clk_adjust_used;
	via_gfx_dpa.is_clk_driving_sel_used =
		via_lcd_options->is_clk_driving_sel_used;
	via_gfx_dpa.is_clk_polarity_used =
		via_lcd_options->is_clk_polarity_used;
	via_gfx_dpa.is_data_driving_sel_used =
		via_lcd_options->is_data_driving_sel_used;

	via_lvds_dpa.is_vt_1636_clk_sel_st1_used =
		via_lcd_options->is_vt_1636_clk_sel_st1_used;
	via_lvds_dpa.is_vt_1636_clk_sel_st2_used =
		via_lcd_options->is_vt_1636_clk_sel_st2_used;
	via_lvds_dpa.vt1636_clk_sel_st1 = via_lcd_options->vt1636_clk_sel_st1;
	via_lvds_dpa.vt1636_clk_sel_st2 = via_lcd_options->vt1636_clk_sel_st2;

	via_chrome9_load_lvds_gfx_default_dpa_setting(card, mode);
	via_chrome9_load_user_gfx_dpa_setting(di_port, &via_gfx_dpa);
	via_chrome9_load_user_lvds_dpa_setting(card, &via_lvds_dpa);
}


void via_chrome9_dvi_gfx_dpa_setting(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	u32 di_port = card->di_port;
	struct drm_device *drm_dev = card->drm_dev;
	u32 chip_set = drm_dev->pci_device;
	char *sub_chip_name = card->name;
	struct via_dvi_options *via_dvi_options = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_connector *current_con = card->current_con;
	struct via_chrome9_connector *via_con = NULL;

	if (!current_con) {
		KMS_DEBUG("%s encoder has no connector\n", card->name);
		return;
	} else {
		via_con = to_via_chrome9_connector(current_con);
		via_dvi_options = (struct via_dvi_options *)via_con->options;
	}

	via_gfx_dpa.clk_adjust = via_dvi_options->clk_adjust;
	via_gfx_dpa.clk_driving_sel = via_dvi_options->clk_driving_sel;
	via_gfx_dpa.clk_polarity = via_dvi_options->clk_polarity;
	via_gfx_dpa.data_driving_sel = via_dvi_options->data_driving_sel;
	via_gfx_dpa.is_clk_adjust_used = via_dvi_options->is_clk_adjust_used;
	via_gfx_dpa.is_clk_driving_sel_used =
		via_dvi_options->is_clk_driving_sel_used;
	via_gfx_dpa.is_clk_polarity_used =
		via_dvi_options->is_clk_polarity_used;
	via_gfx_dpa.is_data_driving_sel_used =
		via_dvi_options->is_data_driving_sel_used;
	via_chrome9_load_default_dpa_setting(chip_set, di_port, mode,
		sub_chip_name);
	via_chrome9_load_user_gfx_dpa_setting(di_port, &via_gfx_dpa);
}

void via_chrome9_hdmi_gfx_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	u32 di_port = card->di_port;
	struct drm_device *drm_dev = card->drm_dev;
	u32 chip_set = drm_dev->pci_device;
	struct via_hdmi_options *via_hdmi_options = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_connector *current_con = card->current_con;
	struct via_chrome9_connector *via_con = NULL;

	if (!current_con) {
		KMS_DEBUG("%s encoder has no connector\n", card->name);
		return;
	} else {
		via_con = to_via_chrome9_connector(current_con);
		via_hdmi_options = (struct via_hdmi_options *)via_con->options;
	}

	via_gfx_dpa.clk_adjust = via_hdmi_options->clk_adjust;
	via_gfx_dpa.clk_driving_sel = via_hdmi_options->clk_driving_sel;
	via_gfx_dpa.clk_polarity = via_hdmi_options->clk_polarity;
	via_gfx_dpa.data_driving_sel = via_hdmi_options->data_driving_sel;
	via_gfx_dpa.is_clk_adjust_used = via_hdmi_options->is_clk_adjust_used;
	via_gfx_dpa.is_clk_driving_sel_used =
		via_hdmi_options->is_clk_driving_sel_used;
	via_gfx_dpa.is_clk_polarity_used =
		via_hdmi_options->is_clk_polarity_used;
	via_gfx_dpa.is_data_driving_sel_used =
		via_hdmi_options->is_data_driving_sel_used;
	via_load_hdmi_default_dpa_setting(chip_set, di_port);
	via_chrome9_load_user_gfx_dpa_setting(di_port, &via_gfx_dpa);
}

static void via_chrome9_external_encoder_destroy(
		struct drm_encoder *encoder)
{
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	i2c_unregister_device(client);
}

static void via_chrome9_external_encoder_dpms(struct drm_encoder *encoder,
		int mode)
{
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	struct via_chrome9_crtc *via_chrome9_crtc;
	struct drm_device *dev = encoder->dev;
	int chipset = dev->pci_device;
	int iga_path;
	u32 di_port = card->di_port;
	switch (mode) {
	case DRM_MODE_DPMS_ON:
		if (encoder->crtc == NULL)
			return;
		via_chrome9_crtc = to_via_chrome9_crtc(encoder->crtc);
		iga_path = via_chrome9_crtc->crtc_id;
		via_chrome9_digital_port_pad_on(di_port, iga_path, chipset);
		break;

	case DRM_MODE_DPMS_STANDBY:
	case DRM_MODE_DPMS_SUSPEND:
	case DRM_MODE_DPMS_OFF:
		via_chrome9_digital_port_pad_off(di_port);
		break;
	}
	if (card->ops->dpms)
		card->ops->dpms(card, mode);
}

static bool via_chrome9_external_encoder_mode_fixup(
		struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjust_mode)
{
	struct drm_crtc *crtc = encoder->crtc ;
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	struct via_chrome9_crtc *via_chrome9_crtc = NULL;

	if (crtc) {
		via_chrome9_crtc = to_via_chrome9_crtc(crtc);
		via_chrome9_crtc->crtc_scale_status = 0;
	}

	if (card->ops->mode_fixup)
		return card->ops->mode_fixup(card, mode,
			adjust_mode);
	return true;
}

static void via_chrome9_external_encoder_prepare(
		struct drm_encoder *encoder)
{
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	via_chrome9_external_encoder_dpms(encoder, DRM_MODE_DPMS_OFF);
	if (card->ops->prepare)
		card->ops->prepare(card);
}

static void via_chrome9_external_encoder_commit(
		struct drm_encoder *encoder)
{
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	via_chrome9_external_encoder_dpms(encoder, DRM_MODE_DPMS_ON);
	if (card->ops->commit)
		card->ops->commit(card);
}


static void via_chrome9_external_encoder_mode_set(
		struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode)
{
	struct via_chrome9_external_card *card =
		to_via_chrome9_external_card(encoder);
	KMS_DEBUG("\n");
	via_chrome9_set_digital_port_sync_polarity(card->di_port,
		adjusted_mode);
	/*
	*  mode may be useful here, but cannot confirm
	*/
	if (card->ops->mode_set)
		card->ops->mode_set(card, adjusted_mode);
}

static struct drm_encoder_funcs via_chrome9_external_encoder_funcs = {
	.destroy = via_chrome9_external_encoder_destroy,
};

static struct drm_encoder_helper_funcs
	via_chrome9_external_encoder_helper_funcs = {
	.dpms = via_chrome9_external_encoder_dpms,
	.mode_fixup = via_chrome9_external_encoder_mode_fixup,
	.prepare = via_chrome9_external_encoder_prepare,
	.mode_set = via_chrome9_external_encoder_mode_set,
	.commit = via_chrome9_external_encoder_commit,
};

static enum drm_connector_status via_chrome9_external_card_detect(
		struct drm_connector *connector, bool force)
{
	struct via_chrome9_connector *via_connector;
	struct via_chrome9_external_card *card;
	int ret;
	KMS_DEBUG("\n");
	via_connector = to_via_chrome9_connector(connector);
	card = via_chrome9_connector_get_private(via_connector);

	ret = card->ops->try_detect(card, connector);
	if (ret == connector_status_connected)  {
		KMS_DEBUG("status is connected!\n");
		card->current_con = connector;
		via_connector->detected_encoder = card->encoder;
	}

	return ret;
}

static void via_chrome9_external_card_force(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct via_chrome9_external_card *card =
		via_chrome9_connector_get_private(via_con);

	if (connector->force == DRM_FORCE_ON) {
		card->ops->force(card, connector);
		if (!card->current_con)
			card->current_con = connector;
	}
}

static struct drm_connector_funcs via_chrome9_external_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_external_card_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.force = via_chrome9_external_card_force,
};

static struct drm_connector_helper_funcs
via_chrome9_external_connector_helper_funcs = {
	.best_encoder = via_chrome9_best_single_encoder,
};

int via_chrome9_external_card_register_connector(
		struct via_chrome9_external_card *card,
		int connector_type)
{
	struct via_chrome9_connector *via_con;
	struct drm_connector *connector;
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	struct i2c_adapter *adapter = client->adapter;
	struct via_chrome9_i2c_chan *chan = container_of(adapter,
		struct via_chrome9_i2c_chan, adapter);
	struct via_vga_options *via_vga_options = NULL;
	struct via_lcd_options *via_lcd_options = NULL;
	struct via_dvi_options *via_dvi_options = NULL;
	struct via_hdmi_options *via_hdmi_options = NULL;
	struct via_dvi_connector_private_info *via_dvi_info = NULL;
	int ret;

	via_con = kzalloc(sizeof(struct via_chrome9_connector), GFP_KERNEL);
	if (via_con == NULL) {
		DRM_ERROR("%s kzalloc failed\n", __func__);
		return -ENOMEM;
	}

	mutex_lock(&card->con_mutex);
	list_add_tail(&via_con->list, &card->list_connectors);
	mutex_unlock(&card->con_mutex);
	via_chrome9_connector_set_private(via_con, card);
	via_con->ddc_bus = chan;
	/* most of the case,
	it's right to think that connector and card share the same i2c port,
	but little case, this may be wrong */

	via_con->detect = via_chrome9_external_card_detect;
	via_con->connector_type = VIA_CHROME9_EXTERNAL_CONNECTOR;
	connector = &via_con->base;
	switch (connector_type) {
	case DRM_MODE_CONNECTOR_VGA:
		via_con->base.polled = 0;
		via_vga_options = kzalloc(sizeof(struct via_vga_options),
			GFP_KERNEL);
		if (!via_vga_options) {
			DRM_INFO("allocate the vga options structure error\n");
			return -1;
		}
		connector->interlace_allowed = true;
		connector->doublescan_allowed = false;
		via_con->options = (void *)via_vga_options;
		via_con->set_xorg_options = via_chrome9_set_vga_xorg_options;
		break;

	case DRM_MODE_CONNECTOR_LVDS:
		via_con->base.polled = 0;
		via_lcd_options = kzalloc(sizeof(struct via_lcd_options),
			GFP_KERNEL);
		if (!via_lcd_options) {
			DRM_INFO("allocate the lcd options structure error\n");
			return -1;
		}
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		via_con->options = (void *)via_lcd_options;
		via_con->set_xorg_options = via_chrome9_set_lcd_xorg_options;
		break;

	case DRM_MODE_CONNECTOR_DVII:
	case DRM_MODE_CONNECTOR_DVID:
		via_con->base.polled = DRM_CONNECTOR_POLL_HPD;
		via_dvi_options = kzalloc(sizeof(struct via_dvi_options),
			GFP_KERNEL);
		if (!via_dvi_options) {
			DRM_INFO("allocate the dvi options structure error\n");
			return -1;
		}
		via_dvi_info =
			kzalloc(sizeof(struct via_dvi_connector_private_info),
			GFP_KERNEL);
		if (!via_dvi_info) {
			DRM_INFO("allocate the dvi info structure error\n");
			return -1;
		}
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		via_con->options = (void *)via_dvi_options;
		via_con->connector_info = (void *)via_dvi_info;
		via_con->set_xorg_options = via_chrome9_set_dvi_xorg_options;
		break;

	case DRM_MODE_CONNECTOR_HDMIA:
		via_con->base.polled = DRM_CONNECTOR_POLL_HPD;
		via_hdmi_options = kzalloc(sizeof(struct via_hdmi_options),
			GFP_KERNEL);
		if (!via_hdmi_options) {
			DRM_INFO(
				"allocate the hdmi options structure error\n");
			return -1;
		}
		connector->interlace_allowed = true;
		connector->doublescan_allowed = false;
		via_con->options = (void *)via_hdmi_options;
		via_con->set_xorg_options = via_chrome9_set_hdmi_xorg_options;
		break;

	default:
		break;
	}
	drm_connector_init(card->drm_dev, &via_con->base,
		&via_chrome9_external_connector_funcs, connector_type);
	drm_connector_helper_add(&via_con->base,
		&via_chrome9_external_connector_helper_funcs);
	ret = drm_sysfs_connector_add(&via_con->base);
	if (ret < 0) {
		DRM_ERROR("%s sysfs connecotor add failed\n", __func__);
		goto failed;
	}

	return 0;
failed:
	drm_connector_cleanup(&via_con->base);
	kfree(via_con);
	return ret;


}

void via_chrome9_external_card_unregister_connector(
		struct via_chrome9_connector *connector)
{
	struct via_chrome9_external_card *card =
		via_chrome9_connector_get_private(connector);

	drm_sysfs_connector_remove(&connector->base);
	drm_connector_cleanup(&connector->base);
	connector->ddc_bus = NULL;
	if (connector->edid)
		kfree(connector->edid);

	mutex_lock(&card->con_mutex);
	list_del(&connector->list);
	mutex_unlock(&card->con_mutex);
	kfree(connector);
}

static int via_chrome9_external_card_init_encoder(
		struct via_chrome9_external_card *card)
{
	struct drm_encoder *encoder = card->encoder;
	card->di_port = DISP_DI_DVP1;
	encoder->possible_crtcs = 0x3;
	drm_encoder_init(card->drm_dev, encoder,
		&via_chrome9_external_encoder_funcs, DRM_MODE_ENCODER_NONE);
	drm_encoder_helper_add(encoder,
		&via_chrome9_external_encoder_helper_funcs);

	return 0;
}

static void via_chrome9_external_card_deinit_encoder(
		struct via_chrome9_external_card *card)
{
	struct drm_encoder *encoder = card->encoder;

	drm_encoder_cleanup(encoder);
}

static void via_chrome9_external_card_register(
		struct via_chrome9_external_card *card)
{
	INIT_LIST_HEAD(&card->list_connectors);
	mutex_init(&card->con_mutex);

	mutex_lock(&cards_mutex);
	list_add(&card->list, &card_list);
	mutex_unlock(&cards_mutex);
}

static void via_chrome9_external_card_unregister(
		struct via_chrome9_external_card *card)
{
	mutex_lock(&cards_mutex);
	list_del(&card->list);
	mutex_unlock(&cards_mutex);
}

static void via_chrome9_set_external_card_ops(
		struct via_chrome9_external_card *card)
{
	if (!strcmp(card->name, "vt1625"))
		card->ops = &vt1625_ops;
	else if (!strcmp(card->name, "sil164"))
		#ifdef SUPPORT_SIL164
		card->ops = &sil164_ops;
		#else
		card->ops = NULL;
		#endif
	else if (!strcmp(card->name, "ad9389"))
		#ifdef SUPPORT_AD9389
		card->ops = &ad9389_ops;
		#else
		card->ops = NULL;
		#endif
	else if (!strcmp(card->name, "vt3192"))
		#ifdef SUPPORT_VT3192
		card->ops = &vt3192_ops;
		#else
		card->ops = NULL;
		#endif
	else if (!strcmp(card->name, "vt3345"))
		#ifdef SUPPORT_VT3345
		card->ops = &vt3345_ops;
		#else
		card->ops = NULL;
		#endif
	else if (!strcmp(card->name, "ch7301_ec") ||
		!strcmp(card->name, "ch7301_ea"))
		#ifdef SUPPORT_CH7301
		card->ops = &ch7301_ops;
		#else
		card->ops = NULL;
		#endif
	else
		card->ops = NULL;
}

static int via_chrome9_external_card_i2c_probe(struct i2c_client *client,
		const struct i2c_device_id *id)
{
	int ret;
	struct via_chrome9_external_card *card;
	struct via_chrome9_encoder *encoder;

	DRM_DEBUG("enter %s\n", __func__);

	/* alloc and register external card here */
	encoder = kzalloc(sizeof(struct via_chrome9_encoder), GFP_KERNEL);

	if (encoder == NULL)
		return -ENOMEM;

	card = kzalloc(sizeof(struct via_chrome9_external_card), GFP_KERNEL);

	if (card == NULL) {
		kfree(encoder);
		return -ENOMEM;
	}

	encoder->encoder_private = card;
	card->encoder = &encoder->base;
	i2c_set_clientdata(client, card);
	strcpy(card->name, client->name);
	card->dev = &client->dev;
	card->drm_dev = via_chrome9_drm_dev;
	via_chrome9_set_external_card_ops(card);
	if (card->ops == NULL) {
		DRM_ERROR("%s card no ops\n", __func__);
		ret = -EINVAL;
		goto card_ops_err;
	}

	ret = via_chrome9_external_card_init_encoder(card);
	if (ret < 0) {
		DRM_ERROR("%s external card init encoder failed\n", __func__);
		goto init_enc_err;
	}

	via_chrome9_external_card_register(card);
	ret = card->ops->init(client);
	if (ret < 0) {
		DRM_ERROR("%s external card init err\n", __func__);
		goto card_init_err;
	}

	return 0;

init_enc_err:
	card->ops->deinit(client);
card_init_err:
	via_chrome9_external_card_unregister(card);
card_ops_err:
	kfree(card);
	kfree(encoder);
	return ret;
}

static int via_chrome9_external_card_i2c_remove(struct i2c_client *client)
{
	struct via_chrome9_external_card *card = i2c_get_clientdata(client);
	struct via_chrome9_encoder *encoder =
		to_via_chrome9_encoder(card->encoder);

	via_chrome9_external_card_deinit_encoder(card);
	card->ops->deinit(client);
	via_chrome9_external_card_unregister(card);
	kfree(card);
	kfree(encoder);

	return 0;
}

static struct i2c_driver via_chrome9_external_card_i2c_driver = {
	.driver = {
		.name = "via external encoder",
	},
	.class = I2C_CLASS_DDC,
	.probe = via_chrome9_external_card_i2c_probe,
	.remove = via_chrome9_external_card_i2c_remove,
	.id_table = via_chrome9_external_card_ids,
	.detect = via_chrome9_external_card_i2c_detect,
	.address_list = via_chrome9_external_card_addrs,
};

int via_chrome9_add_external_card_driver(struct drm_device *dev)
{
	via_chrome9_drm_dev = dev;
	return i2c_add_driver(&via_chrome9_external_card_i2c_driver);
}

void via_chrome9_del_external_card_driver(void)
{
	i2c_del_driver(&via_chrome9_external_card_i2c_driver);
}

void via_chrome9_external_card_link_enc_conn(struct drm_device *dev)
{
	struct drm_connector *connector;
	struct via_chrome9_external_card *card;
	struct via_chrome9_connector *via_con;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		via_con = to_via_chrome9_connector(connector);

		if (via_chrome9_is_internal_connector(via_con))
			continue;
		card = via_chrome9_connector_get_private(via_con);
		drm_mode_connector_attach_encoder(connector,
			card->encoder);

	}
}

void via_chrome9_external_card_write(struct i2c_client *client,
		u8 reg, u8 val)
{
	u8 data[2];

	data[0] = reg;
	data[1] = val;

	i2c_master_send(client, data, 2);
}

u8 via_chrome9_external_card_read(struct i2c_client *client, u8 reg)
{
	u8 data = 0;
	int ret;

	ret = i2c_master_send(client, &reg, 1);
	if (ret == 1)
		i2c_master_recv(client, &data, 1);

	return data;
}

void via_chrome9_external_card_write_mask(struct i2c_client *client,
		u8 reg, u8 val, u8 mask)
{
	u8 old, new;
	int change;

	old = via_chrome9_external_card_read(client, reg);

	new = (old & ~mask) | val;
	change = old != new;

	if (change)
		via_chrome9_external_card_write(client, reg, new);
}
