/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
  * Programming info contained in this module for AD9389
  * can be found in
  * http://www.classic.nxp.com/acrobat_download2/literature/9398/39340011.pdf 
  */

#ifndef __VIA_CHROME9_I2C_H_
#define __VIA_CHROME9_I2C_H_

#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>

struct via_chrome9_i2c_chan;
enum i2c_type {
	CHAN_FROM_GPIO = 1,
	CHAN_FROM_SEARIAL
};

struct via_chrome9_i2c_chan {
	char name;
	struct i2c_adapter adapter;
	int chan_id;
	struct drm_device *dev;
	struct i2c_algo_bit_data bit;
	void *iomem;
	void *data;
};

struct via_chrome9_i2c_hw_rec {
	char name;
	enum i2c_type type;
	/*need to be finished*/
	u16 i2c_reg;
	u8 set_scl_mask;
	u8 get_scl_mask;
	u8 set_sda_mask;
	u8 get_sda_mask;
	/* just for gpio */
	u8 scl_ctl_mask;
	/* just for gpio */
	u8 sda_ctl_mask;
};

#define VIA_CHROME9_IIC_HW_REC(n, i2creg, t, \
		setsclshift, setsdashift, getsclshift, getsdashift, \
		sclctlshift, sdactlshift)   \
{	.name = n, .type = t,    \
	.i2c_reg = i2creg, \
	.set_scl_mask = 0x1 << setsclshift, \
	.set_sda_mask = 0x1 << setsdashift,  \
	.get_scl_mask = 0x1 << getsclshift,  \
	.get_sda_mask = 0x1 << getsdashift, \
	.scl_ctl_mask = 0x1 << sclctlshift, \
	.sda_ctl_mask = 0x1 << sdactlshift }

#define VIA_CHROME9_IIC_SEARIAL_HW_REC(name, i2creg, setsclshift, \
		setsdashift, getsclshift, getsdashift, sclctlshift,	\
		sdactlshift) \
	VIA_CHROME9_IIC_HW_REC(name, i2creg, CHAN_FROM_SEARIAL, \
		 setsclshift, setsdashift, getsclshift, getsdashift, \
		 sclctlshift, sdactlshift)


#define VIA_CHROME9_IIC_GPIO_HW_REC(name, i2creg, \
		 setsclshift, setsdashift, getsclshift, getsdashift,\
		 sclctlshift, sdactlshift)   \
	VIA_CHROME9_IIC_HW_REC(name, i2creg, CHAN_FROM_GPIO, \
		 setsclshift, setsdashift, getsclshift, getsdashift, \
		 sclctlshift, sdactlshift)

extern void via_chrome9_i2c_readbytes(struct via_chrome9_i2c_chan *chan,
		u8 slave_addr, char offset,
		u8 *buffer, unsigned int size);
extern void via_chrome9_i2c_writebytes(struct via_chrome9_i2c_chan *chan,
		u8 slav_addr, char offset,
		u8 *data, unsigned int size);

extern struct via_chrome9_i2c_chan *via_chrome9_i2c_create(
		struct drm_device *dev,
		struct via_chrome9_i2c_hw_rec *data,
		const char *name);

extern void via_chrome9_free_i2c_chan(struct via_chrome9_i2c_chan *chan);
extern void via_chrome9_free_all_i2c_chan(void);
extern struct via_chrome9_i2c_chan *via_chrome9_lookup_i2c_chan(char name);
#endif
