/*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Programming info contained in this module for silicon 164
 * can be found in http://www.siliconimage.com/docs/SiI-DS-0021-E-164.pdf.
 */

#include <linux/kernel.h>
#include <linux/module.h>

#include <linux/i2c.h>
#include "via_chrome9_external_card.h"
#include "via_chrome9_sil164.h"

#define sil164_write(c, reg, val)  via_chrome9_external_card_write(c, reg, val)
#define sil164_read(c, reg)	via_chrome9_external_card_read(c, reg)
#define sil164_write_mask(c, reg, val, mask)	\
		via_chrome9_external_card_write_mask(c, reg, val, mask)

static int sil164_init(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
			i2c_get_clientdata(client);

	KMS_DEBUG("enter\n");

	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_DVID);

	return 0;
}

static void sil164_deinit(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
			i2c_get_clientdata(client);
	struct via_chrome9_connector *connector;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");

	list_for_each_safe(p, n, &card->list_connectors) {
		connector = list_entry(p, struct via_chrome9_connector, list);
		via_chrome9_external_card_unregister_connector(connector);
	}
}

static void sil164_dpms(struct via_chrome9_external_card *card, int mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	u8 val = 0;

	KMS_DEBUG("enter\n");

	if (mode == DRM_MODE_DPMS_ON)
		val |= SIL164_8_PD;
	else
		val = 0;

	sil164_write_mask(client, SIL164_REG8, val, SIL164_8_PD);
}

static int via_chrome9_sil164_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	return via_dvi_mode_valid(connector, mode);
}

static int via_chrome9_sil164_get_modes(struct drm_connector *connector)
{
	return via_dvi_get_modes(connector);
}

static void sil164_force(struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{

	struct drm_connector_helper_funcs *connector_func =
			connector->helper_private;

	KMS_DEBUG("enter\n");

	connector_func->get_modes =
		via_chrome9_sil164_get_modes;
	connector_func->mode_valid =
		via_chrome9_sil164_mode_valid;
}

static enum drm_connector_status sil164_try_detect(
		struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_con->options;
	int ret = connector_status_disconnected;
	u8 val;

	KMS_DEBUG("enter\n");

	if (via_dvi_options->no_ddc_value) {
		val = sil164_read(client, SIL164_REG9);

		if (val & SIL164_9_RSEN)
			ret = connector_status_connected;
	} else {

		if (true == via_chrome9_detect_device_by_read_edid(connector,
			DIGITAL_DEVICE))
			ret = connector_status_connected;
	}

	connector_func->get_modes =
		via_chrome9_sil164_get_modes;
	connector_func->mode_valid =
		via_chrome9_sil164_mode_valid;

	return ret;
}

static void sil164_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	u8 val = 0;
	val |= SIL164_8_DSEL | SIL164_8_PD;
	sil164_write_mask(client, SIL164_REG8,
		val, SIL164_8_BSEL | SIL164_8_DSEL);

	via_chrome9_dvi_gfx_dpa_setting(card, mode);
}

struct via_chrome9_external_card_ops sil164_ops = {
	.init = sil164_init,
	.deinit = sil164_deinit,
	.dpms = sil164_dpms,
	.force = sil164_force,
	.try_detect = sil164_try_detect,
	.mode_set = sil164_mode_set,
};

