/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/i2c.h>

#include "via_chrome9_external_card.h"


#define vt3192_write(c, reg, val)  via_chrome9_external_card_write(c, reg, val)
#define vt3192_read(c, reg)	via_chrome9_external_card_read(c, reg)
#define vt3192_write_mask(c, reg, val, mask)	\
	via_chrome9_external_card_write_mask(c, reg, val, mask)


static int vt3192_init(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);

	KMS_DEBUG("enter\n");
	via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_DVID);

	return 0;
}

static void vt3192_deinit(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);
	struct via_chrome9_connector *connector;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");
	list_for_each_safe(p, n, &card->list_connectors) {
		connector = list_entry(p, struct via_chrome9_connector, list);
		via_chrome9_external_card_unregister_connector(connector);
	}
}


static void vt3192_dpms(struct via_chrome9_external_card *card, int mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	KMS_DEBUG("enter\n");

	if (mode == DRM_MODE_DPMS_ON)
		vt3192_write(client, 0x08, 0x3B);
	else
		vt3192_write_mask(client, 0x08, 0x00, 0x01);


}


static int via_chrome9_vt3192_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	return via_dvi_mode_valid(connector, mode);
}


static int via_chrome9_vt3192_get_modes(struct drm_connector *connector)
{
	return via_dvi_get_modes(connector);
}

static void vt3192_force(struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;

	connector_func->get_modes =
		via_chrome9_vt3192_get_modes;
	connector_func->mode_valid =
		via_chrome9_vt3192_mode_valid;

}

static enum drm_connector_status vt3192_try_detect(
		struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_con->options;
	u8 val;
	int ret = connector_status_disconnected;

	KMS_DEBUG("enter\n");
	if (via_dvi_options->no_ddc_value) {
		val = vt3192_read(client, 0x09);
		if (val & 0x04)
			ret = connector_status_connected;

	} else {
		if (true == via_chrome9_detect_device_by_read_edid(connector,
			DIGITAL_DEVICE)) {
			ret = connector_status_connected;
			KMS_DEBUG("vt3192 connector is connected\n");
		}
	}
		connector_func->get_modes =
			via_chrome9_vt3192_get_modes;
		connector_func->mode_valid =
			via_chrome9_vt3192_mode_valid;
	return ret;
}

static void vt3192_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	via_chrome9_dvi_gfx_dpa_setting(card, mode);
}

struct via_chrome9_external_card_ops vt3192_ops = {
	.init = vt3192_init,
	.deinit = vt3192_deinit,
	.dpms = vt3192_dpms,
	.force = vt3192_force,
	.try_detect = vt3192_try_detect,
	.mode_set = vt3192_mode_set,

};
