/*
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * (including the next paragraph) shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL VIA, S3 GRAPHICS, AND/OR
 * ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <linux/kernel.h>
#include <linux/module.h>

#include "via_chrome9_drv.h"
#include "via_chrome9_external_card.h"


#define vt3345_write(c, reg, val)  via_chrome9_external_card_write(c, reg, val)
#define vt3345_read(c, reg)	via_chrome9_external_card_read(c, reg)
#define vt3345_write_mask(c, reg, val, mask)	\
	via_chrome9_external_card_write_mask(c, reg, val, mask)

struct iodata {
	u8 reg;
	u8 mask;
	u8 val;
};

static struct iodata vt3345_init_list[] = {
	{ 0x10, 0xc0, 0x00 },
	{ 0x0b, 0xff, 0x40 },
	{ 0x0c, 0xff, 0x31 },
	{ 0x0d, 0xff, 0x31 },
	{ 0x0e, 0xff, 0x68 },
	{ 0x0f, 0xff, 0x58 },
	{ 0x09, 0xa0, 0xa0 },
	{ 0x10, 0x33, 0x13 },
};
#define NUM_OF_INIT_LIST	ARRAY_SIZE(vt3345_init_list)

static void vt3345_hw_init(struct i2c_client *client)
{
	int i;

	for (i = 0; i < NUM_OF_INIT_LIST; i++) {
		vt3345_write_mask(client, vt3345_init_list[i].reg,
			vt3345_init_list[i].val, vt3345_init_list[i].mask);
	}
}

static void vt3345_set_lcd_info(struct i2c_client *client,
		struct via_lcd_options *opts)
{
	if (opts->dual_channel)
		vt3345_write_mask(client, 0x08, 0xe0, 0xe0);
	else
		vt3345_write_mask(client, 0x08, 0x00, 0xe0);


	if (opts->msb)
		vt3345_write_mask(client, 0x08, 0x10, 0x10);
	else
		vt3345_write_mask(client, 0x08, 0x00, 0x10);

	if (opts->no_dithering)
		vt3345_write_mask(client, 0x0a, 0x00, 0x70);
	else
		vt3345_write_mask(client, 0x0a, 0x50, 0x70);

}

static int vt3345_init(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);
	struct via_lcd_options *lcd_opts;
	struct via_chrome9_connector *connector;
	char name[5];
	int ret;

	KMS_DEBUG("enter\n");
	memset(name, '\0', sizeof(name));

	ret = via_chrome9_external_card_register_connector(card,
		DRM_MODE_CONNECTOR_LVDS);

	if (ret < 0) {
		printk(KERN_ERR "%s connector register err\n", __func__);
		return ret;
	}

	list_for_each_entry(connector, &card->list_connectors, list) {
		if ((connector->base).connector_type_id == 1)
			strcpy(name, "LCD1");
		else if ((connector->base).connector_type_id == 2)
			strcpy(name, "LCD2");
		else if ((connector->base).connector_type_id == 3)
			strcpy(name, "LCD3");

		printk(KERN_INFO "vt3345 lcd command name is %s\n", name);
		break;
	}
	lcd_opts = (struct via_lcd_options *)connector->options;

	ret = via_chrome9_fb_cmd_parse(name, (void *)lcd_opts);

	if (ret < 0) {
		printk(KERN_INFO "%s no correct command line for this connector\n",
			__func__);
		return 0;
	}
	return 0;
}

static void vt3345_deinit(struct i2c_client *client)
{
	struct via_chrome9_external_card *card =
		i2c_get_clientdata(client);
	struct via_chrome9_connector *connector;
	struct list_head *n, *p;

	KMS_DEBUG("enter\n");
	list_for_each_safe(p, n, &card->list_connectors) {
		connector = list_entry(p, struct via_chrome9_connector, list);
		via_chrome9_external_card_unregister_connector(connector);
	}

	kfree(card->private_data);
}

static void vt3345_dpms(struct via_chrome9_external_card *card, int mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);

	KMS_DEBUG("enter\n");

	if (mode == DRM_MODE_DPMS_ON)
		vt3345_write_mask(client, 0x10, 0x20, 0x20);
	else
		vt3345_write_mask(client, 0x10, 0x00, 0x20);
}

static int via_chrome9_vt3345_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	return via_lcd_mode_valid(connector, mode);
}


static int via_chrome9_vt3345_get_modes(struct drm_connector *connector)
{
	return via_lcd_get_modes(connector);
}

static void vt3345_force(struct via_chrome9_external_card *card,
		struct drm_connector *connector)
{
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;

	connector_func->get_modes =
		via_chrome9_vt3345_get_modes;
	connector_func->mode_valid =
		via_chrome9_vt3345_mode_valid;

}

static enum drm_connector_status vt3345_try_detect(
	struct via_chrome9_external_card *card,
	struct drm_connector *connector)
{
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct via_lcd_options *via_lcd_options =
		(struct via_lcd_options *)via_con->options;
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	int ret = connector_status_disconnected;

	KMS_DEBUG("enter\n");

	if (via_lcd_options->panel_index) {
		ret = connector_status_connected;
	} else {
		if (true == via_chrome9_detect_device_by_read_edid(connector,
			DIGITAL_DEVICE)) {
			ret = connector_status_connected;
		}
	}
	connector_func->get_modes =
		via_chrome9_vt3345_get_modes;
	connector_func->mode_valid =
		via_chrome9_vt3345_mode_valid;

	if (connector_status_connected == ret)
		vt3345_set_lcd_info(client, via_lcd_options);

	return ret;
}

static void vt3345_mode_set(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode)
{
	struct i2c_client *client =
		via_chrome9_external_card_to_i2c_client(card);
	vt3345_hw_init(client);
	via_chrome9_lvds_gfx_dpa_setting(card, mode);
}

bool vt3345_mode_fixup(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjust_mode)
{
	return via_chrome9_lvds_mode_fixup(card->encoder, mode, adjust_mode);
}

struct via_chrome9_external_card_ops vt3345_ops = {
	.init = vt3345_init,
	.deinit = vt3345_deinit,
	.dpms = vt3345_dpms,
	.force = vt3345_force,
	.try_detect = vt3345_try_detect,
	.mode_set = vt3345_mode_set,
	.mode_fixup = vt3345_mode_fixup,
};
