/* 
 * File:   ddmpeg.h
 * Author: springvesper@gmail.com
 * Created on January 30, 2010, 4:57 PM
 */

#ifndef _DDMPEG_H
#define	_DDMPEG_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "viampg.h"
#include "vmi_types.h"

typedef  unsigned long InstanceID ;

typedef enum _Decode_Type {
	DECODER_NULL =-1,
	DECODE_MPEG2 = 0,
	DECODE_MPEG4 = 1,
	DECODE_WMV9 = 2,
	DECODE_H264 = 3,
	DECODE_VC1 = 4,
}Decode_Type;

typedef enum _VO_Type {
	VO_X11 = 0x1,
	VO_XV_TEXTURE = 0x2,
	VO_VMI = 0x4,
	VO_ALPHA = 0x8,
	VO_GL = 0x10	/*not implemented */
} VO_Type;

typedef struct _OV_Box32 {  /*this structure is defined to conform to pixman_box32_t*/
	int x1, y1, x2, y2;
}OV_Box32, *OV_Box32Ptr;

typedef enum _OV_Flag{
	DDOVER_HIDE  = 0x00000001,
	DDOVER_SHOW  = 0x00000002,
	DDOVER_KEYDEST = 0x00000004,
	DDOVER_BOB  = 0x002000001,
	DDOVER_ON_TOP = 0x08000000	/*put the overlay video on top of another*/
}OV_Flag, *OV_FlagPtr;

/*define erro mark,when VMI_Init erro,return erro -mark to upper*/
typedef enum _VMIInitErrType{
	ERR_INIT_LOGS = 0x1,
	ERR_INIT_GFX =0x2,
	ERR_INIT_DRM = 0x3,
	ERR_INIT_SURFACE = 0x4,
	ERR_INIT_CMDEXEC = 0x5,
	ERR_INIT_HQV = 0x6,
	ERR_INIT_VO = 0x7,
	ERR_INIT_DECODE = 0x8,
}VMIInitErrType;


typedef struct _ImageParam {
	OV_Box32 rSrc;				/*box in the source surface*/
	OV_Box32 rDest;				/*box in the screen*/
	OV_Flag  flag; 				/* overlay flags */
	unsigned long colorSpaceLowValue;
	unsigned long colorSpaceHighValue;
	unsigned long imgStruct;	 /*frame/field/top/bottom*/
	Bool 	NeedPostProcess; 
	Bool 	saveDataByDstRect;
}ImageParam, *ImageParamPtr;

typedef enum  _Alpha_Type {
	ALPHA_CONSTANT = 0x01,
	ALPHA_STREAM = 0x02,
	ALPHA_DISABLE = 0x03,
	ALPHA_GRAPHIC = 0x04,
	ALPHA_COMBINE = 0x05		
}Alpha_Type;

typedef struct _AlphaParam {
	Alpha_Type type;
	unsigned char constantFactor;
	OV_Box32  rDest;		/*alpha destination on the screen*/
}AlphaParam, *AlphaParamPtr;

typedef enum _SubPicture_Task {
	ENABLE_SUBPICTURE = 0x01,
	DISABLE_SUBPICTURE = 0x2,
	WRITE_RAM_TABLE = 0x4
} SubPicture_Task;

typedef struct _SubPictureParam {
	SubPicture_Task task;
	unsigned long ramTbl[16];
	OV_Box32 rSrc;			/*original osd size*/
	OV_Box32 rDest;			/*osd destination on the main video stream*/
}SubPictureParam, *SubPictureParamPtr;

typedef struct _ColorSpaceMatrix {
	float CSCMatrix[3][4];
}ColorSpaceMatrix, *ColorSpaceMatrixPtr;

typedef struct _ColorSpaceValue{	
	float brightness;            /*range:-1.0~1.0,  normal is:0.0 */
	float contrast;               /*range:0.0~10.0,  normal is:1.0 */
	float saturation; 		 /*range:0.0~10.0,  normal is:1.0 */
	float hue;			/*range:-PI~PI,  normal is:0.0 */
}ColorSpaceValue,*ColorSpaceValuePtr;

typedef enum _Event_Type {
	XRandR_Event = 0x1,
	PM_Event = 0x2
		/*more to come...*/
} Event_Type;

typedef Ddmpeg_Status SurfaceStatus;

typedef enum _Surface_Type {
	Decoder_Buffer = 0x1,
	VideoOut_Buffer = 0x4,
	SubPicture_Buffer = 0x8,
	Alpha_Buffer = 0x10,
	Video_PCIE = 0x20
} Surface_Type;

typedef struct _SurfUnit {
	unsigned long id; /*fourCC or dummy*/
	Surface_Type type;
	unsigned int  width;
	unsigned int height;
	unsigned long index;
	unsigned int  pitch;
	unsigned int size;    /*surface size in bytes*/
	Bool planar; 			/*YCbCr alike format must have this flag set*/
	unsigned long offset; /*offset from the framebuffer start. physical address*/
	unsigned char *vaddr; /*mapped address*/
	SurfaceStatus status; /*surface status*/
	unsigned int BO_name;
} SurfUnit, *SurfUnitPtr;

typedef float Render_Factor;
typedef enum _M_OPT{
	M_NONE = 0x0,
	M_MIX = 0x1,
	M_MAX = 0x2
}M_OPT;

typedef enum _RTOP{
	RT_0 = 0x1,
	RT_90 = 0x2,
	RT_180 = 0x4,
	RT_270 = 0x8,
	RT_360 = 0x10
}RTOP;

typedef struct _RenderOp{
	Render_Factor src_factor;
	Render_Factor des_factor;
	Render_Factor src_alphafactor;
	Render_Factor des_alphafactor;
	Render_Factor constant_factor;
	Render_Factor constant_alphafactor;
	int32_t constant_alpha;
	int32_t constant_rgb;
	RTOP rtop;
	M_OPT min;
}RenderOp,*RenderOpPtr;

/* Vdpau image structure */
typedef struct _viaVdpImageDataRec{
	unsigned long fourcc;
	unsigned long offset ;
	unsigned int bo_name;
	unsigned long w;
	unsigned long h;
	unsigned long pitch;
	/*alpha stream blending paramter maybe could be added here*/
}viaVdpImageDataRec,*viaVdpImageDataPtr;

/*define surface fourCC type  should be open to the sdk */
#define FOURCC(a,b,c,d)						\
	(unsigned int)((((unsigned int)a))+(((unsigned int)b)<<8)	\
			+(((unsigned int)c)<<16)+(((unsigned int)d)<<24))

#define  FOURCC_RGB16		FOURCC('R', 'V', '1', '6')     /*RGB565*/
#define  FOURCC_RGB15       FOURCC('R', 'V', '1', '5')      /*RGB15*/
#define  FOURCC_RGBA		FOURCC('R','G','B','A')          /*RGB with alpha*/
#define  FOURCC_RGB32		FOURCC('R','V','3','2')
#define  FOURCC_RGB24		FOURCC('R','V','2','4')
#define  FOURCC_YUYV		FOURCC('Y', 'U', 'Y', 'V')
#define  FOURCC_YUY2		FOURCC('Y', 'U', 'Y', '2')	/*packed format.*/
#define  FOURCC_YUV422		FOURCC('Y', '4', '2', '2')	/*plannar format.*/
#define  FOURCC_YUV444      FOURCC('Y', '4', '4', '4')
#define  FOURCC_YV12        FOURCC('Y', 'V', '1', '2')	/*YV12*/
#define  FOURCC_NV12		FOURCC('N', 'V', '1', '2')	/* uncompressed overlay surface */
#define  FOURCC_VIA			FOURCC('W', 'L', 'K', 'N')	  /*VIA: 0x4E4B4C57*/
#define  FOURCC_SUBP		FOURCC('S', 'U', 'B', 'P')	/*SUBP*/
#define  FOURCC_ALPHA       FOURCC('A', 'L', 'P', 'H')	/*ALPH*/
#define  FOURCC_UTIL 	   	FOURCC('U', 'T', 'I', 'L')	/*XV, fake surfaces*/
#define  FOURCC_RGB_MASK    FOURCC('R','\0','\0','\0')

#define FOURCC_VDPAU			0x01020304    /* for vdpau specific */
#define FOURCC_VMI_XV			0x04030201
#define FOURCC_RGB555           0x0000000F
#define FOURCC_RGB565           0x00000010
#define FOURCC_XRGB0888         0x00000018
#define FOURCC_ARGB8888         0x00000020
#define RGB_BytePerPixel(format)     (((format + 15) & ~15)/8)


/*initialize the whole vmi graphic systme*/
extern int VMI_Init(void);

/*uninitialize the vmi graphic system*/
extern int VMI_UnInit(void);

/*create a decoder instance*/
extern InstanceID VMI_CreateDecoder(Decode_Type  type);

/*destroy a decoder instance, return NULL to the pointer*/
extern int VMI_DestroyDecoder(InstanceID *decoder);

/*initialize the newly created decoder with default  configuration to make it active*/
extern int VMI_ConnectDecoder(InstanceID decoder);

/*take down the deocder*/
extern int VMI_DisConnectDecoder(InstanceID decoder);

/*create a video output  instance*/
extern InstanceID VMI_CreateVideoOut(VO_Type type,Display *dpy);

/*destroy the video out instance, return NULL to the pointer*/
extern int VMI_DestroyVideoOut(InstanceID *vo);

/*attach one surface to a given instance. return the newly created surface unit.
 *sup must be allocated in the first place
 */
extern int  VMI_AttachSurface(InstanceID inst,
		Surface_Type  type, unsigned long fourcc,  
		unsigned long width, unsigned long height, SurfUnitPtr  attach);

/*detach one surface from a given instance.*/
extern int  VMI_DetachSurface(InstanceID inst,  SurfUnitPtr  detach );

/*render a whole picture stored in the input buffer. The picture information is given in  the picInfo structure */
extern int VMI_Render(InstanceID decoder, PictureInfoPtr picInfo, InputBufPtr  buf);

/*output the decoded image to the display device*/
extern int VMI_PutImage(InstanceID vo,Display *display, Drawable draw,GC gc,
		ImageParamPtr image, SurfUnitPtr sup);
/*Output the image from HQV engine to upper*/
extern int VMI_RenderImage(InstanceID vo,SurfUnitPtr des,SurfUnitPtr src,ImageParamPtr image,RenderOpPtr op);
/*output alpha image*/
extern int VMI_PutAlpha(InstanceID alp, AlphaParamPtr alpha, SurfUnitPtr sup);

/*output subpicture*/
extern int VMI_PutSubPicture(InstanceID vo, SubPictureParamPtr subp, SurfUnitPtr sup);

/*receive CSC Matrix from upper to set colorspace*/
extern int VMI_SetColorSpaceByMatrix(InstanceID vo, ColorSpaceMatrixPtr color);

/*receive CSC value from upper to set colorspace*/
extern int VMI_SetColorSpaceByValue(InstanceID vo,ColorSpaceValuePtr color);

/*colorSpace setting, no need to calculate CSC matrix, runs in different process*/
extern int VMI_SetCSC_Utility(unsigned int engineMask, ColorSpaceValuePtr color);
/*Query decoder capability.*/
/*every 4bit represent a type, they are mpeg2 mpeg4 wmv9 vc1 and h264 decoder class from
* low bit to high bit*/
extern int VMI_QueryDecoderCapability(unsigned int *cap);
/*This interface is used to get videokey in bios.
 *It is called after vmi decoder has been initted.
 *return Success if p2 and checkbios success when vmi_int
 *return -1 if p2 and checkbios failed when vmi_int
 *return 1 if p1*/
 /* videokey = 0x11111, it means valid video key set contains
  * mpeg2 mpeg4 wmv9 h264 and vc1.*/
extern int VMI_GetVideoKey(unsigned int *videokey, unsigned int *svd);



#ifdef	__cplusplus
}
#endif

#endif	/* _DDMPEG_H */

