/*
 *
 *Copyright (c) 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 *
 *This PROPRIETARY SOURCE CODE contains proprietary information of VIA
 *Technologies, Inc. (VIA); Contents of this file shall not be disclosed to any 3rd party, 
 *copied or duplicated in any form, in whole or in part without prior written consent of 
 *VIA.  THIS SOURCE CODE & ANY RELATED DOCUMENTATION 
 *(COLLECTIVELY SOFTWARE) ARE PROVIDED AS IS, WITH ALL FAULTS, & 
 *WITHOUT WARRANTY OF ANY KIND.  VIA DISCLAIMS ALL EXPRESS OR 
 *IMPLIED WARRANTIES.  USER ACKNOWLEDGES THAT PARTIES FOR ANY 
 *DAMAGES ARISING FROM USE OF SOFTWARE OR ITS DERIVATIVE, 
 *INCLUDING, W/O LIMITATION, LOSS OF USE, DATA OR PROFITS. USER
 *WHOLLY ASSUMES ALL RISK ARISING FROM USE & PERFORMANCE OF 
 *SOFTWARE.
 */

/**
 *@file vmf_constants.h
 *@brief define constants of vmfl
 *@author tonybai\@viatech.com.cn
 *@version 1.0.0
 *@date 2007/3/27 
 */

#ifndef __vmf_constants_h__
#define __vmf_constants_h__
/**
 *defined the memory align type
 */ 
#define BUFFER_ALIGN_SIZE_8                              8 /**< memory aligned to 8*/
#define BUFFER_ALIGN_SIZE_16                            16/**<memory aligned to  16*/
#define BUFFER_ALIGN_SIZE_32                            32/**<memory aligned to  32*/  


/**
 *@brief defined for data type of decoder input data
 *@see SVMFDecodeDataIn
 */
#define DECODER_VIDEO_DATA                0x0001      /**< input video stream data*/
#define DECODER_SEQUENCE_HEADER     0x0002      /**< input sequence header*/
#define DECODER_FRAME_RATE                0x0004     /**< input frame rate*/
#define DECODER_FRAME_END                  0x0008     /**< indicate frame end*/
#define DECODER_FRAME_BEGIN               0x0010    /**< indicate frame begin*/
#define DECODER_JPEG_DATA                   0x0020   /**< input jpeg data*/
#define DECODER_AVC1_HEADER               0x0040
#define DECODER_AVC1_DATA                   0x0080
#define DECODE_ONLY_MASK                    0x10000000 /**< input just contain mask*/


//junprince,modified for mpeg2 field out, 2007.12.17
#define VO_FRAME2              0x1 
#define VO_FRAME3              0x2
#define VO_TOP_BOTTOM          0x4
#define VO_BOTTOM_TOP          0x8
#define VO_TOP_BOTTOM_TOP      0x10
#define VO_BOTTOM_TOP_BOTTOM   0x20
#define VO_FRAME                 0x40 

#define VO_TOP_FIELD           0x1
#define VO_BOTTOM_FIELD     0x2
#define VO_BOTH_FIELDS       (VO_TOP_FIELD | VO_BOTTOM_FIELD)

#define  PIC_STRUCTUR_TOP_FIELD              0x1
#define  PIC_STRUCTUR_BOTTOM_FIELD       0x2
#define  PIC_STRUCTUR_FRAME                     0x3

/*
#define VIA_PICT_STRUCT_BOTTOM PIC_STRUCTUR_BOTTOM_FIELD
#define VIA_PICT_STRUCT_TOP PIC_STRUCTUR_TOP_FIELD
#define VIA_PICT_STRUCT_FRAME PIC_STRUCTUR_FRAME */

/**
 *@brief The proerty the decoder can be set and get.
 */ 
typedef enum{
    eSKIP_FRAME=0, /**< tell decoder skip some frames for time is urgent*/
    ePULL_DOWN,      /**< tell decoder to do pull down operation*/
    eFRAME_RATE,    /**< get the frame rate from decoder, this property can not be set*/
    eBIT_RATE,          /**< get the bit rate from decoder, this property can not be set*/
    eSKIP_FRAME_TYPE,/*<set the frame type of skipping*/
    eRESERVED_FRAME_NUM,/*<set number of reserved frame*/
    eFREED_FRAME_NUM,/*<get number of freed frame*/
    /*added by junprince,2008/5/16*/
    eFRAME_TYPE,/*<get frame type: I , P ,B>*/
    eASPECT_RATIO /*only for mpeg2, in sequence header*/
}EVMFDecoderProperty;

/**
 *@brief There are two types of overlay. SW overlay means the uncompressed frame 
 * is not from decoder but from user by flip function, HW overlay means, uncompressed frame 
 * is always below decoder layer. SW overlay doesn't means HW decoder is not used. In some 
 * case, user can get uncompressed frame from HW decoder, then put it to overlay after doing 
 * some operations. 
 */ 
typedef enum{
    eVMF_OVERLAY_INVALID=-1,/**< invalid value*/
    eVMF_OVERLAY_HW=0,/**< use hardware overlay*/
    eVMF_OVERLAY_SW_YUY2,/**< use software overlay and the format is YUY2*/
    eVMF_OVERLAY_SW_YV12,/**< use software overlay and the format is YV12*/
    eVMF_OVERLAY_SW_NV12/**< use software overlay and the format is NV12*/
}EVMFOverlayType;

/**
 * @brief The proerty the overlay object can be set and get.
 */ 
typedef enum{
    eColorKey=0,/**< ColorKey*/
    eColorKeyEnable,/**<enable ColorKey */
    eDEINTERLACE,/**< do deinterlace*/
    eVideoSize,/**< video size*/
    eVideoFormat,/**< video format*/
    eSize,/**< the view size*/
    ePosition,/**< the view postion*/
    eOutputEnable,/**< enable output*/
    eTVOut,/**< TV out*/
    eTVOverlay,/**< TVOverlay*/
    eForceUpdate,/**< force to update, used when there're two overlay, need to change the overlap relationship*/
    eOverlayStay, /**< when this set to 1, the update overlay will not interrupt overlap order of overlays*/
    eRaiseToTop, /**< when call this, the overlay will be raised up to the up most one*/
    eDeinterlaceMode, /** deinterlace mode */
    eVideoPosition,     /** left-top coordinate of the video output rect*/
    eConstantAlphaEnable,/** constant alpha enable for whole overlay, 1:enable,0 disable*/
    eConstantAlphaValue, /** constant alpha value for whole overlay, for 4bit(16alpha mode), the valid alpha value is input&0xf0, for 8bit(256alpha mode), the valid alpha value is input&0xff*/
    eSnapShot,     /*copy the last display frame from video memory*/
    eNotUpdateOverlay, /*make the set property not call UpdateOverlay instantly, the UpdateOverlay will called when set_property(eUpdateOverlay) is called, this can make sure when updateoverlay, all argments are proper*/
    eUpdateOverlay,/*when this property is set, vmfl will call a UpdateOverlay, and then set property will switch to instant UpdateOverlay mode(mean every argument changes will call UpdateOverlay)*/
    eFreeSep,
    eOntop ,    /* this property indicate the video is ontop or not when 2 video is play , add by bruth  */
    eDisplayInfo , 
}EVMFOverlayProperty;
#define DVMFOverlayPropertyNUM 18

#endif
