/*
 *
 *Copyright (c) 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 *
 *This PROPRIETARY SOURCE CODE contains proprietary information of VIA
 *Technologies, Inc. (VIA); Contents of this file shall not be disclosed to any 3rd party, 
 *copied or duplicated in any form, in whole or in part without prior written consent of 
 *VIA.  THIS SOURCE CODE & ANY RELATED DOCUMENTATION 
 *(COLLECTIVELY SOFTWARE) ARE PROVIDED AS IS, WITH ALL FAULTS, & 
 *WITHOUT WARRANTY OF ANY KIND.  VIA DISCLAIMS ALL EXPRESS OR 
 *IMPLIED WARRANTIES.  USER ACKNOWLEDGES THAT PARTIES FOR ANY 
 *DAMAGES ARISING FROM USE OF SOFTWARE OR ITS DERIVATIVE, 
 *INCLUDING, W/O LIMITATION, LOSS OF USE, DATA OR PROFITS. USER
 *WHOLLY ASSUMES ALL RISK ARISING FROM USE & PERFORMANCE OF 
 *SOFTWARE.
 */

/**
 *@file vmf_types.h
 *@brief This is the basic types definition in VMF project.
 *		These basic types can be used in Linux-like/Win platform.
 *@author JulionLiang\@viatech.com.cn
 *@version 1.0.0
 *@date 2006/12/31 
 */
 
#ifndef __vmf_types_h__
#define __vmf_types_h__

#include <stdio.h>

#ifndef _NTYPES_H  /*avoid type define conflict to v264*/
// unsigned
typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;
// Linux/Sparc64 defines uint64_t
#if !(defined (__sparc_v9__) && defined(__linux__))
/* XXX may be done for all 64 bits targets ? */
#if defined (__x86_64__) || defined(__ia64)
typedef unsigned long uint64_t;
#else
typedef unsigned long long uint64_t;
#endif
#endif

// signed
/* if Solaris/__sun__, don't typedef int8_t, as it will be typedef'd
   prior to this and will cause an error in compliation, conflicting
   with /usr/include/sys/int_types.h, line 75 
 */
#ifndef __sun__
typedef signed char int8_t;
#endif
typedef signed short int16_t;
typedef signed int int32_t;
// Linux/Sparc64 defines int64_t
#if !(defined (__sparc_v9__) && defined(__linux__))
#if defined (__x86_64__) || defined(__ia64)
typedef signed long int64_t;
#else
typedef signed long long int64_t;
#endif
#endif
#endif

// file
//typedef struct FILE FILE;
extern int fprintf(FILE *, const char *, ...);
extern int printf(const char *, ...);

// Null
#undef NULL
#define NULL 0

// bool
//typedef uint8_t	BOOL;

/**
 *@brief The basic types defined in project.
 *@internal VMFInt8_t, ...for project using in Linux/Win platform
 */ 

typedef int8_t		VMFInt8_t;
typedef uint8_t		VMFUInt8_t;
typedef VMFInt8_t	       VMFChar_t;
typedef int16_t		VMFInt16_t;
typedef uint16_t 		VMFUInt16_t;
typedef int32_t		VMFInt32_t;
typedef uint32_t		VMFUInt32_t;
typedef int64_t		VMFInt64_t;
typedef uint64_t		VMFUInt64_t;
typedef unsigned long  VMFULong ;
typedef long                VMFLong ; //hyun, for 64bit OS
typedef FILE			VMFFile_t;
//typedef BOOL		VMFBool_t;
typedef uint8_t		VMFBool_t;

/**
 *@brief The basic pointer types defined in project.
 *@internal PVMFInt8_t, ...for project using in Linux/Win platform
 */ 

typedef int8_t*		PVMFInt8_t;
typedef uint8_t*		PVMFUInt8_t;
typedef VMFInt8_t*	PVMFChar_t;
typedef int16_t*		PVMFInt16_t;
typedef uint16_t* 	       PVMFUInt16_t;
typedef int32_t*		PVMFInt32_t;
typedef uint32_t*		PVMFUInt32_t;
typedef int64_t*		PVMFInt64_t;
typedef uint64_t*		PVMFUInt64_t;
typedef FILE*		PVMFFile_t;

/**
 *@brief The basic types bound defined in project.
 *@internal VMFINT8_MIN, ...for project using
 */ 

#define VMFINT8_MIN			(-128)
#define VMFINT16_MIN		(-32767-1)
#define VMFINT32_MIN		(-2147483647-1)
#define VMFINT64_MIN		(-(VMFInt64_t)(9223372036854775807)-1)
#define VMFINT8_MAX		(127)
#define VMFINT16_MAX		(32767)
#define VMFINT32_MAX		(2147483647)
#define VMFINT64_MAX		((VMFInt64_t)(9223372036854775807))
#define VMFUINT8_MAX		(255)
#define VMFUINT16_MAX		(65535)
#define VMFUINT32_MAX		(4294967295U)
#define VMFUINT64_MAX		((VMFUInt64_t)(18446744073709551615))

/**
 *@brief The bool type value defined in project.
 *@internal VMF_FALSE, VMF_TRUE for project using
 */ 

// bool constant
#define VMF_FALSE	0x0
#define VMF_TRUE	(VMFBool_t)(~VMF_FALSE)

/**
 *@brief The NULL value defined in project.
 *@internal VMF_NULL for project using
 */ 

#define VMF_NULL	NULL


typedef struct _SVMFRect
{
    VMFInt32_t xmin;
    VMFInt32_t ymin;
    VMFInt32_t xmax;
    VMFInt32_t ymax;
}SVMFRect;

typedef struct _SVMFRectList
{
    struct _SVMFRectList* next;
    SVMFRect rect;
}SVMFRectList;

typedef struct _SVMFSnapShot
{
    VMFUInt8_t  *Img;              /*used to store the video raw data from surface*/
    VMFUInt32_t  ImgFormat;    /*now VMI only support NV12 format*/
}SVMFSnapShot;

typedef enum
{
    eVMF_OK=0,
    eVMF_ERROR,
    eVMF_FRAME_SKIPED,
    eVMF_NULL_POINTER,
    eVMF_DECODER_LIST_INITED,
    eVMF_HW_NOSUPPORT,
    eVMF_WMC_INIT_FAILED,
    eVMF_RESOURCE_NOFREE,
    eVMF_VMI_CALL_FAILED,
    eVMF_INPUT_ARGV_ERROR,
    eVMF_NO_PROPERTY
} EVMF_Status;

typedef struct _SVMFDisplayInfo
{
    void *pDisplay; /* point to the display device(the return of XOpenDisplay) */
    unsigned long     reserved;
	void * /*Drawable*/ 	draw ; //add by bruth
	void * /*GC*/ 	gc  ;          //add by bruth
} SVMFDisplayInfo, * PSVMFDisplayInfo;

/**@internal this definition of FOURCC  is just for user to check.*/

#ifndef FOURCC_VIA
#define FOURCC_VIA     0x4E4B4C57  /*VIA*/
#endif
#ifndef FOURCC_SUBP 
#define FOURCC_SUBP    0x50425553 
#endif
#ifndef FOURCC_ALPHA
#define FOURCC_ALPHA   0x48504C41 
#endif
#ifndef FOURCC_YUY2
#define FOURCC_YUY2    0x32595559 
#endif
#ifndef FOURCC_YV12
#define FOURCC_YV12    0x32315659 
#endif
#ifndef FOURCC_NV12
#define FOURCC_NV12   0x3231564E
#endif

#endif //__vmf_types_h__

