/*
 *
 *Copyright (c) 1998-2007 VIA Technologies, Inc. All Rights Reserved.
 *
 *This PROPRIETARY SOURCE CODE contains proprietary information of VIA
 *Technologies, Inc. (VIA); Contents of this file shall not be disclosed to any 3rd party, 
 *copied or duplicated in any form, in whole or in part without prior written consent of 
 *VIA.  THIS SOURCE CODE & ANY RELATED DOCUMENTATION 
 *(COLLECTIVELY SOFTWARE) ARE PROVIDED AS IS, WITH ALL FAULTS, & 
 *WITHOUT WARRANTY OF ANY KIND.  VIA DISCLAIMS ALL EXPRESS OR 
 *IMPLIED WARRANTIES.  USER ACKNOWLEDGES THAT PARTIES FOR ANY 
 *DAMAGES ARISING FROM USE OF SOFTWARE OR ITS DERIVATIVE, 
 *INCLUDING, W/O LIMITATION, LOSS OF USE, DATA OR PROFITS. USER
 *WHOLLY ASSUMES ALL RISK ARISING FROM USE & PERFORMANCE OF 
 *SOFTWARE.
 */

 /**
 *@file vmfl.h
 *@brief this is entry header for VMF library
 *@author DaveLiu \@viatech.com.cn
 *@version 1.0.0
 *@date 2006/12/28 
 */
 
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif
#include <vmf_types.h>
#include <vmf_constants.h>

#ifndef __VMFL_H__
#define __VMFL_H__
#ifdef __cplusplus
extern "C"{
#endif

#include "/usr/include/ddmpeg/ddmpeg.h"
#include "/usr/include/ddmpeg/viampg.h"
#include "/usr/include/ddmpeg/vmi_types.h"
//hyun


#define DrOOR


/**
 *@brief There are decoder types. which types supported by customer 
             special vmf library are decided in configure time. 
 */ 
 
 //configure it, expose for ap use
#define    VMF_DEC_MPEG2  0x00000001
#define    VMF_DEC_MPEG4  0x00000002
#define    VMF_DEC_WMV9   0x00000004
#define    VMF_DEC_VC1      0x00000008 
#define    VMF_DEC_H264   0x00000010
#define    VMF_DEC_JPEG   0x00000020
/*
#ifdef     VMF_MPEG2_CONFIGURE 
#define    VMF_DEC_MPEG2  0x00000001
#endif
#ifdef     VMF_MPEG4_CONFIGURE
#define    VMF_DEC_MPEG4  0x00000002
#endif
#ifdef     VMF_WMV9_CONFIGURE
#define    VMF_DEC_WMV9   0x00000004
#endif
#ifdef     VMF_VC1_CONFIGURE
#define    VMF_DEC_VC1    0x00000008 
#endif
#ifdef     VMF_H264_CONFIGURE
#define    VMF_DEC_H264   0x00000010
#endif
#ifdef     VMF_JPEG_CONFIGURE
#define    VMF_DEC_JPEG   0x00000020
#endif
*/
typedef enum{
eSKIPNONE = 0,
eSKIP_P_FRAME,
eSKIP_B_FRAME ,
}ESKIPFrameType;;

/**
 *@brief This Data structure is used by decode functionof decoder.
 */ 

typedef struct _VMFDecodeDataIn{
 VMFUInt32_t uiType;/**< type of data sent into decoder, referencd DECODER_XXX 
                                         in vmf_constants.h*/
 VMFUInt64_t uiPts;   /**< pts of input data*/
 VMFUInt64_t uiDts;   /**< pts of input data*/
 VMFUInt32_t  uiLen,uiDecodedLen; /**< data length*/
  void *Data;           /**< the buffer caontaining the uncompressed data */
}SVMFDecodeDataIn,*PSVMFDecodeDataIn;

/**
 *@brief This structure store the out put data got from decoder
 */ 
typedef struct _VMFDecodeDataOut{
    VMFUInt32_t uiType;/**< define the data type*/
//#ifdef NEWDDMPEG
    //SurfUnitPtr surfptr;
//#endif
    VMFUInt32_t uiSurfaceID;/**< surface ID for hardware flip*/

    VMFChar_t * pcUncompressedData[3];/**< uncompressed frame*/
    VMFUInt64_t pts;/**< real pts gotten fron bit stream*/
    VMFBool_t    bSkiped;/**< whether skip this picture, don't filp*/
    VMFUInt32_t uiWidth;/**< width of this picture*/
    VMFUInt32_t uiHeight;/**< height of this picture*/
    VMFUInt32_t uiPitch;/**< pitch of this picture*/
    VMFUInt32_t uiDuration;/**< display duration of this picture, 
                                            Unit is microseconds*/
    
    /*Follows define how to display picture, reference ISO13818-2*/
    VMFUInt32_t uiPictStructure;/**< structure of this picture:1 = Top field; 
                                                2 = Bottom field; 3 = Frame picture;*/
    VMFUInt32_t uiProgressiveSequence;/**< whether progressive sequence*/
    VMFUInt32_t uiProgressiveFrame;/**< whether progressive frame*/
    VMFUInt32_t uiTopFieldFirst;/**< whether top field first*/
    VMFUInt32_t uiRepeatFirstField;/**< whether repeat first field*/

    /*junprince,2007.12.18*/
    VMFUInt32_t uiFieldOut;/*video out mode, Top-Bottom/Bottom-Top/Frame*/
    VMFUInt32_t uiLeft;/**< left position of video output coordinate accodring to encode coordinate*/
    VMFUInt32_t uiTop;/**< Top position of video output coordinate accodring to encode coordinate*/

}SVMFDecodeDataOut, *PSVMFDecodeDataOut;

/**
 *@brief This structure store the out put data sent to hardware flip
 */ 
typedef struct _VMFFlipData{
    VMFUInt32_t uiSurfaceID;/**< surface ID for hardware flip*/  
    VMFUInt32_t uiPictStructure;/**< structure of this picture:1 = Top field; 
                                                2 = Bottom field; 3 = Frame picture;*/
    VMFBool_t    bSkiped;/**< whether skip this picture, don't filp*/
}SVMFFlipData, *PSVMFFlipData;

/**
 *@brief  This is a callback function for user to get decompressesd frame data,
              which may be a surface id or frame raw data depending on decoder's
              behavior. This function will be called by decoder when a frame be
              decoded successfully.
 *@param[in] UserData can be any data needed and operated by user
 *@param[in] Data        decoded frame data
 *@param[out] no
 *@see PSVMFFlipData
 *@return no
 */ 
typedef void(*PFVMFGetOutCallBack)(void *UserData, PSVMFDecodeDataOut Data);

/**
 *@brief  SVMFDecoder,PSVMFDecoder is in fact the decoder class, which includes
              all functions which can decode a frame or change behavior of decoder. 
 */ 
typedef struct _VMFDecoder{
/**
 *@brief this function implement decoding  function.
 *@param[in] self decoder object
 *@param[in] pDataIn input data
 *@param[out] no
 *@see PSVMFDecodeData
 *@return
        the return value is no more meaning for using, while it will benifit for debuging.
                    VMF_OK       operation  work normal
                    VMF_ERROR there is some error when decoding
 */ 
    EVMF_Status (*Decode) (struct _VMFDecoder *self, PSVMFDecodeDataIn pDataIn);
    
/**
 *@brief  this function will register a callback function for user to get decompressed data
 *@param[in] self this decoder object
 *@param[in] CallBack callback function 
 *@param[in] UserData user's data used by callback function
 *@param[out] no
 *@see PFVMFGetOutCallBack
 *@return no
 */ 
    EVMF_Status (*RegisterGetOutCallBack)(struct _VMFDecoder *self,PFVMFGetOutCallBack  CallBack, void *UserData);

/**
 *@brief  This function will get some properties of decoder, which can change behavior of decoder   
 *@param[in] self this decoder object itself
 *@param[in] Property property type
 *@param[in] value      the value that will be setted to this property. not each property need a value
 *@param[out] no
 *@see EVMFDecoderProperty
 *@return no
 */ 
    EVMF_Status (*Setproperty)(struct _VMFDecoder *self, EVMFDecoderProperty Property, VMFUInt32_t value);
/**
 *@brief  This function will get some properties of decoder, which can change behavior of decoder   
 *@param[in] self this decoder object itself
 *@param[in] Property property type
 *@param[out]  value  the value that has been setted to this property. 
 *@see Setproperty
 *@return no
 */ 
    EVMF_Status (*Getproperty)(struct _VMFDecoder *self, EVMFDecoderProperty Property, VMFUInt32_t *value);  
/**
 *@brief  this function will free  a surfaceId but not by filp action
 *@param[in] 
                    handle created by VMF_HandleCreate
                    uiSurfID the surfaceid want to be freed
 *@param[out]  none    
 @return EVMF_Status
 */
    EVMF_Status (*FreeSurfaceID)(VMFUInt32_t Handle, VMFUInt32_t uiSurfID);
/**
 *@brief this function will reset decoder status 
 *@return EVMF_Status
 */
 	EVMF_Status (*ResetDecoder)(struct _VMFDecoder *self) ;
	int (*CheckFreeDataOfQueue)(VMFUInt32_t handle) ;
}SVMFDecoder,*PSVMFDecoder;


/**
 *@brief This is a OSD(On Screen Display) class. User can create an OSD object 
             by  OSDCreate function of overlay.  OSD provides some basic UI and 
             text functions. In fact subtittle can also be implemented by OSD
 *@see OSDCreate
 */ 

typedef struct _VMFOSD
{

/**
 *@brief  clear this osd object's content
 *@param[in]  self pointer to this osd object
 *@param[out] no
 *@return
                  eVMF_ERROR
                  eVMF_OK
 */ 
 
 EVMF_Status  (* Clear )( struct _VMFOSD *self); 

/**
 * 
 *@brief    :clear osd related vmi sub picture surface.
 *@param[in] PSVMFOSD self:osd object.
 *@param[out] 
 *@return EVMF_Status
 */
 
EVMF_Status  (*ClearSurface)(struct _VMFOSD* self);

/**
 *@brief clear all palettes of this osd object  
 *@param[in] self pointer to this osd object   
 *@param[out] no
 *@see SetPaletteEntry
 *@return 
                eVMF_OK - success
                eVMF_ERROR - failure
 */ 
EVMF_Status  (* ClearPalette )( struct _VMFOSD *self); 

/**
 *@brief  This function writes a color into the palette table.
 *@param[in] self pointer to this osd object 
 *@param[in] index color index to overwrite, it shall be in the range [0..6]
 *@param[in] R Red component in the range [0..255]
 *@param[in] G Green component in the range [0..255]
 *@param[in] B Blue component in the range [0..255]
 *@param[in] alpha Alpha component (0x00 is completely transparent. 0xFF is completely opaque)
 *@param[out] no 
  *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetPaletteEntry )( struct _VMFOSD *self, VMFUInt8_t index, VMFUInt8_t  R, VMFUInt8_t  G, VMFUInt8_t  B, VMFUInt8_t  alpha); 
EVMF_Status  (*SetPaletteEntry_YCbCr)(struct _VMFOSD * self, VMFInt8_t index, VMFUInt8_t Y, VMFUInt8_t Cb, VMFUInt8_t Cr, VMFUInt8_t alpha);

/**
 *@brief  When using function VIAWriteTextOsd, use this function to set the color index for the foreground.
 *@param[in] self pointer to this osd object 
 *@param[in] index foreground color index,  it shall be in the range [0..6]
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetForegroundColor )( struct _VMFOSD *self, VMFUInt8_t  index);

/**
 *@brief move osd object to (x,y).  
 *@param[in] self pointer to this osd object
 *@param[in] (x,y) means top&left point
 *@param[out] no 
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* OsdAreaMove)( struct _VMFOSD *self, VMFUInt32_t  x, VMFUInt32_t y);

/**
 *@brief  setting font used by following write text function
 *@param[in] self pointer to this osd object
 *@param[in] filename font name 
 *@param[in] size size of font
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
/*Font Settings*/
EVMF_Status  (* LoadFont )( struct _VMFOSD *self,  const VMFUInt8_t *filename, VMFUInt32_t size);

/**
 *@brief  Get size of text
 *@param[in] self pointer to this osd object
 *@param[in] text whose size to be retrieved
 *@param[out] width: if NULL,ignore it; otherwise , to store width of text
 *@param[out]   height: if NULL,ignore it; otherwise , to store height of text
 *@return 
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* GetTextSize )( struct _VMFOSD *self, const VMFUInt8_t *text, VMFUInt32_t *width, VMFUInt32_t *height);

/**
 *@brief  set encoding of text 
 *@param[in] self pointer to this osd object
 *@param[in] encoding encoding type, NULL ... no conversion (iso-8859-1), locale encoding,gbk, 
                     gb2312, gb18030, big5...
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (*SetEncoding) ( struct _VMFOSD *self,  VMFUInt8_t *encoding); 

/**
 *@brief  This function write text to the screen
 *@param[in] self pointer to this osd object
 *@param[in] text content 
 *@param[in] (x,y) top&left point of text.
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* WriteText )( struct _VMFOSD *self, VMFUInt8_t *text, VMFUInt32_t x, VMFUInt32_t y); 

/**
 *@brief  This function sets the color index used for the pixel located at the position indicated by x, y.
 *@param[in] self pointer to this osd object 
 *@param[in] x the x pixel coordinate of the text position
 *@param[in] y  the y pixel coordinate of the text position
 *@param[in] indx - color index in OSD palette,  it shall be in the range [0..6]
 *@param[out] no
 *@return
                 eVMF_OK - success
                 eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetPixel )( struct _VMFOSD *self, VMFUInt32_t x, VMFUInt32_t y, VMFUInt8_t indx); 

/**
 *@brief  This function draws a rectangle in OSD.
 *@param[in] self pointer to this osd object 
 *@param[in] x x (leftmost) pixel coordinate of the target rectangle position
 *@param[in] y y (uppermost) pixel coordinate of the target rectangle position
 *@param[in] w width (in pixels) of the target rectangle
 *@param[in] h height (in pixels) of the target rectangle
 *@param[in] indx color index for the target rectangle, it shall be in the range [0..6]
 *@param[out] no
 *@return
                 eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* DrawRect)( struct _VMFOSD * self, VMFInt32_t x, VMFInt32_t y, VMFUInt32_t w, VMFUInt32_t h, VMFUInt8_t index);

/**
 *@brief  This function copy a rectangle context to subpicture.
 *@param[in] self pointer to this osd object 
 *@param[in] x x (leftmost) pixel coordinate of the target rectangle position
 *@param[in] y y (uppermost) pixel coordinate of the target rectangle position
 *@param[in] src_x (leftmost) pixel coordinate of the source rectangle position
 *@param[in] src_y (uppermost) pixel coordinate of the source rectangle position
 *@param[in] w width (in pixels) of the target rectangle
 *@param[in] h height (in pixels) of the target rectangle
 *@param[in] p_src color index buffer for the rect context
 *@param[in] p_srca alpha buffer for the rect context
 *@param[in] inverse_alpha indicate whether we should do the inverse formula alpha=(alpha==0)?0:(0xf0-alpha&0xf0)
 *@param[in] stride stride of both color index buffer and alpha buffer
 *@param[out] no
 *@return
                 eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* CopyRectContext)( struct _VMFOSD * self, VMFInt32_t x, VMFInt32_t y, VMFUInt32_t src_x, VMFUInt32_t src_y,VMFUInt32_t w, VMFUInt32_t h, VMFUInt8_t* p_src,VMFUInt8_t* p_srca,VMFUInt32_t stride,VMFBool_t inverse_alpha);

/**
 *@brief  draw a bmp, which is only contain 8 colors included in your palette.  
 *@param[in] self pointer to this osd object 
 *@param[in] bmp buffer contains bmp raw data. each pixel has a color index
 *@param[in] (w,h) bmp size
 *@param[in] (x,y) bmp position
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* DrawBmp )( struct _VMFOSD *self, VMFUInt8_t *bmp, VMFUInt32_t x, VMFUInt32_t y, VMFUInt32_t w, VMFUInt32_t h);

/**
 *@brief  Get OSD mode-scaled and unscaled
 *@param[in] self pointer to this osd object 
 *@param[out] no
 *@return
                OSD mode, scaled or unscaled   
 */ 
VMFUInt32_t   (*GetShowMode)( struct _VMFOSD *self);

/**
 *@brief  Set OSD mode-scaled or unscaled
 *@param[in] self pointer to this osd object 
 *@param[in] mode scale or unscaled
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status   (* SetShowMode)( struct _VMFOSD *self, VMFUInt32_t mode);


/**
 *@brief  show osd object
 *@param[in] self pointer to this osd object 
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Show )( struct _VMFOSD *self);

/**
 *@brief  hide osdobject
 *@param[in] self pointer to this osd object 
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Hide )(struct _VMFOSD *self);

/**
 *@brief  resize osdobject
 *@param[in] self pointer to this osd object 
 *@param[in] new width 
 *@param[in] new height 
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Resize )(struct _VMFOSD *self,VMFUInt32_t width, VMFUInt32_t height);

}SVMFOSD,*PSVMFOSD;

/**
 *@brief Alpha class containes all operations about alpha. alpha object is created by  AlphaObjectCreate
 * of ovaerlay object.
 */ 
typedef struct _VMFAlpha
{
/**
 *@brief  Set alpha surface.
 *@param[in] self alpha object itself
 *@param[in] palpha: pointer to the user created alpha surface(i.e.,alpha value array).
 *@param[in] width: the surface width
 *@param[in] height: the surface height
 *@param[out] no 
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Set)( struct _VMFAlpha *self, VMFUInt8_t *palpha,VMFUInt32_t width, VMFUInt32_t height); 

/**
 *@brief  Set the alpha value
 *@param[in] self alpha object itself
 *@param[in] alpha: the constant value to be set.
 *@param[in] width: the surface width
 *@param[in] height: the surface height
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetConstant)( struct _VMFAlpha * self, VMFUInt8_t alpha,VMFUInt32_t width, VMFUInt32_t height); 

/**
 *@brief  Generate alpha picture for hollow picture or hollow text.
   Input picture's format is described by format, which is the FourCC code.
   Input picture buffer is described by starting address, picture width and height.
   One alpha value is generated for each pixel in the sub rectangle starting at (draw_left, draw_top)
   with width and height (draw_width, draw_height). Their validity will be checked.
   
   There are two alpha value to choose, so the generated alpha picture is called
   "binary alpha". Pixel with color value that falls in hollow_high_color and
   hollow_low_color will be given the hollow_alpha alpha value, others will
   be given normal_alpha. Color values are compared by components. When
   hollow_high_color == hollow_low_color, color values are compared by
   simply testing if the color value equals to hollow_high_color or hollow_low_color,
   which can be faster, and is recommended to use on text picture, which is drawn by
   font processing library.
 *@param[in] self alpha object itself
 *@param[in] format: FourCC code, can be RGB555, RGB565, RGB555X, RGB565X, RGB24, BGR24,
                                RGB32 and BGR32, which are all defined by V4L2 video image formats spec.
                                but formats with alpha are also supported, including BGRA32 and RGBA32,
                                whose unused byte of each pixel in the V4L2 definition are used for alpha,
                                and for RGBT555 and RGBT555X, whose unused bit in V4L2 definition are used
                                for transparency bit.
  *@param[in] ppicture: Pointer to the RGB picture buffer with picture_width wide and picture_height high. All RGB formats
                                 here are pixel wise packed, so only one pointer is enough.Noting that, this doesn't include bmp header.
  *@param[in] picture_width: width of the RGB picture
  *@param[in] picture_height:height of the RGB picture
  *@param[in] draw_left,  draw_top, draw_width,  draw_height: 
	              They define a sub rectangle  in the picture where each pixel will generate a alpha value.
  *@param[in] normal_alpha, hollow_alpha: The two alphas can be set by function caller
                    so that the alpha value are not necessarily 0 or 255 but can be any desired
                    value between 0 and 255. Alpha value meaning is:
                    displayed_color_component = (alpha/255)*video + ((255-alpha)/255)*graphics;
                    And here hollow means that THE RGB PICTURE WILL BE DISPLAYED HOLLOW,
                    so normally hollow_alpha will be greater than normal_alpha.
                    normal_alpha = 0 and hollow_alpha = 255 is the real hollow and real normal.
  *@param[in] hollow_low_color,hollow_high_color:
                    Color are described with a 4 byte unsigned int value:
                    {
                         unsigned char A, R, G, B;
                        color = (A << 24) | (B << 16) | (G << 8) | R;
                     }
                   And color are compared component by component.
                   Every component of hollow_high_color must be larger than corresponding
                   component of hollow_low_color. The two arguments allow some tolerance
                   of the color which is used as color key for hollow alpha value.
 *@param[in] amode: This is an enum argument.
                   This argument is specially provided for input formats with alpha or
                   transprency component. For plain RGB format this argument has no
                   effect.
                  Here is the meaning of different modes:
                  IGNORE_ALPHA:
                     Treate the input image as if it has no alpha or transparency component.
                  COUNT_IN_ALPHA:
                   In this mode, the A component of hollow_high_color and hollow_low_color
                   takes effect. The A component is also compared to generate binary alpha
                   value. This mode makes the compare complexed and I don't think this mode
                   is very useful.
                 RETRIEVE_ALPHA:
                   In this mode, the alpha value is retrieved to compose the alpha image, and
                   all RGB components are ignored. According to FourCC definition, alpha
                   components from the input image means, when the image is displayed on
                   an other image, the blended result is:
                   output = (alpha/255)*image + ((255-alpha)/255)*background;
               For RGBA32 and BGRA32 input image, the normal_alpha and the hollow_alpha
                  argument are ignored. And the output alpha image is generated with this
                  equation:
                  output = 255 - src_alpha;
               For RGBT555 and RGBT555X, the argument normal_alpha and hollow_alpha are used.
                   A pixel with transparent component == 1 will generate a pixel with hollow_alpha
                   in the alpha image, and transparent == 0 will result in a pixel with normal_alpha.
               GEN_FROM_ALPHA:
                   In this mode, the A component of RGBA32 or BGRA32 input image will be compared against
                   hollow_high_color.A and hollow_low_color.A to generate alpha image,but R, G and B
                   component of the input image are ignored.
                   For RGBT555 and RGBT555X input image, hollow_low_color and hollow_height_color are also
                   used. To the input image, transparent == 1 is equivalent to its alpha == 0
                   and transparent == 0 is equivalent to its alpha == 255. The two values are used
                   to compare with hollow_high_color.A and hollow_low_color.A.
                   Two special and useful case are:
                   hollow_high_color.A == hollow_low_color.A = 0;
                  Then the generated alpha image is the same as mode "RETRIEVE_ALPHA".
                  hollow_heigh_color.A == hollow_low_color.A = 255;
                  The generated alpha image is the reverse of the former case.
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetHollowPicture)(struct _VMFAlpha *self, 
VMFUInt32_t format,VMFUInt8_t *ppicture, VMFUInt32_t picture_width, VMFUInt32_t picture_height,
VMFUInt32_t draw_left, VMFUInt32_t draw_top,VMFUInt32_t draw_width, VMFUInt32_t draw_height,
VMFUInt8_t normal_alpha,VMFUInt8_t hollow_alpha,VMFUInt32_t hollow_low_color, VMFUInt32_t hollow_high_color,VMFUInt32_t amode);

/**
 *@brief  Generate alpha picture for hollow picture directly from a BMP file url. 
    This is a convenient variant of Set_Hollow_Picture.However, we now only support
    VIA_PIX_FMT_BGR24 and  VIA_PIX_FMT_BGR555 format bmp files in this API.
 *@param[in] self alpha object itself
 *@param[in] url Pointer to the RGB picture file
 *@param[in] The rest arguments can be refered to those of Set_Hollow_Picture:
                     draw_left,  draw_top, draw_width,  draw_height, normal_alpha, hollow_alpha, 
                     hollow_low_color,hollow_high_color, amode 
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (*SetHollowPictureFromUrl)(struct _VMFAlpha *self, VMFChar_t *url, 
VMFUInt32_t draw_left, VMFUInt32_t draw_top,VMFUInt32_t draw_width, VMFUInt32_t draw_height,
VMFUInt8_t normal_alpha,VMFUInt32_t hollow_alpha,VMFUInt32_t hollow_low_color, VMFUInt32_t hollow_high_color,VMFUInt32_t amode);

/**
 *@brief Generate alpha picture for hollow picture directly from a rectangle on screen, 
     and use this alpha picture to blend this rectangle with video.
 *@param[in] self alpha object itself
 *@param[in] draw_left, draw_top, draw_width, draw_height define position and  size of 
                     the rectangle on screen
 *@param[in] normal_alpha, hollow_alpha, hollow_low_color, hollow_high_color, amode can
                     be found in note of Set_Hollow_Picture

 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* SetHollowPictureFromOnScreen)(struct _VMFAlpha *self, 
VMFUInt32_t draw_left, VMFUInt32_t draw_top,VMFUInt32_t draw_width, VMFUInt32_t draw_height,
VMFUInt8_t normal_alpha,VMFUInt8_t hollow_alpha,VMFUInt32_t hollow_low_color, VMFUInt32_t hollow_high_color,VMFUInt32_t amode);

/**
 *@brief  Clear the set alpha surface.
 *@param[in] self alpha object itself
 *@param[out] 
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Clear)( struct _VMFAlpha *self); 

/**
 *@brief  Move the set surface to the screen coordinate(x,y).
 *@param[in] self alpha object itself
 *@param[in] x:the leftmost coordinate
 *@param[in] y:the upper coordinate
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Move)( struct _VMFAlpha *self, VMFUInt32_t x, VMFUInt32_t y);

/**
 *@brief  show alpha object
 *@param[in] self alpha object itself
 *@param[out] no
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Show )(struct _VMFAlpha *self);

/**
 *@brief  Hide the shown  alpha surface.
 *@param[in] self alpha object itself
 *@param[out] no
 *@return
                 eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Hide )(struct _VMFAlpha *self);

/**
 *@brief Disable constant and surface alpha. 
 *@param[in] self alpha object itself
 *@param[out] no
 *@return
                 eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status  (* Disable )(struct _VMFAlpha *self);

}SVMFAlpha,*PSVMFAlpha;

/**
 *@brief This structure is class for overlay. overlay object is created by VMF_OverlayCreate
 */ 

typedef struct _VMFOverlay
{
/**
 *@brief This function will flip uncompressed data to HW then show it on screen. 
     OSD update only after this function called
 *@param[in] self pointer to this object
 *@param[in] FlipDataID the data for uncompress frame
 *@param[out] no
 *@see SVMFFlipData
 *@return
                   eVMF_OK - success
                   eVMF_ERROR - failure
 */ 
EVMF_Status (*Flip)(struct _VMFOverlay *self, PSVMFFlipData FlipDataID);

EVMF_Status (*FlipNoFree)(struct _VMFOverlay *self, PSVMFFlipData FlipDataID);
/**
 *@brief   This function will flip uncompressed data to HW then show it on screen
 *@param[in] self pointer to this object
 *@param[in] Property property type
 *@param[in] Value the value to this property
 *@param[out] no
 *@see EVMFOverlayProperty 
 *@return
                    eVMF_OK - success
                    eVMF_ERROR - failure
 */ 
EVMF_Status (*SetProperty)(struct _VMFOverlay *self,EVMFOverlayProperty Property,VMFInt32_t Value);

/**
 *@brief  This function will flip uncompressed data to HW then show it on screen
 *@param[in] self pointer to this object
 *@param[in] Property property type
 *@param[out] Value the value to this property
 *@see EVMFOverlayProperty
 *@return
                    eVMF_OK - success
                    eVMF_ERROR - failure
 */ 
EVMF_Status (*GetProperty)(struct _VMFOverlay *self ,EVMFOverlayProperty Property,VMFInt32_t *Value);

/**
 *@brief   This function will create sw decoder flip data queue,which will be hold by vmflibrary. 
     this function is called only when decoder is 3rd SW decoder
 *@param[in] self pointer to this overlay object
 *@param[in] width,height the size of frame 
 *@param[in] format FOURCC code
 *@param[out] no
 *@see SWDecodeDataQueueDestroy
 *@return
                    eVMF_OK - success
                    eVMF_ERROR - failure
 */ 
EVMF_Status(*SWDecodeDataQueueCreate)(struct _VMFOverlay *self,VMFUInt32_t width, VMFUInt32_t height, VMFUInt32_t format);

/**
 *@brief   this function will destroy sw decoder flip data queue, this is called only when decoder is 3rd SW decoder
 *@param[in] self pointer to this overlay object
 *@param[out] no
 *@see SWDecodeDataQueueCreate
 *@return
                    eVMF_OK - success
                    eVMF_ERROR - failure
 */ 
EVMF_Status(*SWDecodeDataQueueDestroy)(struct _VMFOverlay *self);

/**
 *@brief   this function will give out address of data pointer in sw decoder flip data queue. user need fill 
     pcUncompressedData[3] in this structure by decompressed frame data
 *@param[in] 
                    self pointer to this overlay object
                    Data address of pointer that point the data buffer in queue 
 *@param[out] 
                   
 *@see PSVMFDecodeDataOut
 *@return
                    eVMF_OK - success
                    eVMF_ERROR - failure
 */ 
EVMF_Status(*GetSWDecodeDataBuf)(struct _VMFOverlay *self, PSVMFDecodeDataOut Data);


/**
 *@brief   this function will enable OSD function, all osd operations are unavaible before this function is called
 *@param[in] self pointer to this Overlay object
 *@param[in] width, height the size of real OSD surface, which will be sent to HW finally.
                    Note, they are different from width and height in OSDObjectCreate, which are buffers 
                    before user and real OSD surface.    
 *@param[out] no
 *@see OSDDeInit
 *@return
                eVMF_OK - success
                eVMF_ERROR - fail
 */ 
 EVMF_Status (*OSDInit)(struct _VMFOverlay *self,VMFUInt32_t width, VMFUInt32_t height);

/**
 *@brief   this function will create an OSD object
 *@param[in] self pointer to this Overlay object
 *@param[in] width, height the size for sub-surface of this OSD object
 *@param[out] no
 *@see OSDObjectDestroy
 *@return
                NULL                                   failed
                OSD object adress created   sucedded
 */ 
PSVMFOSD (*OSDObjectCreate)(struct _VMFOverlay *self, VMFUInt32_t width, VMFUInt32_t height);

/**
 *@brief   this function will destroy an OSD object
 *@param[in] self pointer to this Overlay object
 *@param[out] no
 *@see OSDObjectCreate
 *@return
                    eVMF_OK - success
                eVMF_ERROR - failure
 */ 
EVMF_Status (*OSDObjectDestroy)(PSVMFOSD OsdObject);

/**
 *@brief   this function will disable OSD function, all osd operations are unavaible after this function is called
 *@param[in] self pointer to this Overlay object
 *@param[out] no
 *@see OSDDeInit
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure 
 */ 
EVMF_Status (*OSDDeInit)(struct _VMFOverlay * self);

/**
 *@brief   this function will set the Palette
 *@param[in] self pointer to this Overlay object
 *@param[in] pointer to the palette array 
 *@param[out] no
 *@see OSDDeInit
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure 
 */ 
EVMF_Status (*OSDSetPaletteYUV)(struct _VMFOverlay * self,VMFUInt32_t* p_data);

/**
 *@brief   this function will enable Alpha function, all alpha operations are unavaible before this function is called
 *@param[in] self pointer to this Overlay object
 *@param[in] width, height the size of real Alpha surface, which will be sent to HW finally.
 *@param[out] no
 *@see AlphaDeInit
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure  
 */ 
EVMF_Status (*AlphaInit)(struct _VMFOverlay *self,VMFUInt32_t width, VMFUInt32_t height);

/**
 *@brief   this function will create an alpha object
 *@param[in] self pointer to this Overlay object
 *@param[in] width, height the size for sub-surface of this alpha object. Sub surface is a part of Alpha surface
 *@param[out] no
 *@see AlphaObjectDestroy
 *@return
                NULL                                   failed
                alpha object adress created   sucedded
 */ 
PSVMFAlpha (*AlphaObjectCreate)(struct _VMFOverlay *self, VMFUInt32_t width, VMFUInt32_t height);

/**
 *@brief   this function will destroy an alpha object
 *@param[in] self pointer to this alpha object
 *@param[out] no
 *@see AlphaObjectCreate
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure
 */ 
EVMF_Status (*AlphaObjectDestroy)(PSVMFAlpha AlphaObject);

/**
 *@brief   this function will disable Alpha function, all alpha operations will be unavaible after this function is called
 *@param[in] self pointer to this Overlay object
 *@param[out] no
 *@see AlphaInit
 *@return
                eVMF_OK - success
                eVMF_ERROR - failure
 */ 
EVMF_Status (*AlphaDeInit)(struct _VMFOverlay *self);

}SVMFOverlay,*PSVMFOverlay;

VMFUInt32_t VMF_HandleCreate(PSVMFDisplayInfo psVMFDisplayInfo);/*each handle can only do one time operation of following*/ 
PSVMFDecoder VMF_DecoderCreate(VMFUInt32_t Handle,VMFUInt32_t type);
EVMF_Status VMF_DecoderDestroy(PSVMFDecoder self);
PSVMFOverlay VMF_OverlayCreate(VMFUInt32_t Handle,VMFUInt32_t Fourcc);
EVMF_Status VMF_OverlayDestroy(PSVMFOverlay self);
EVMF_Status VMF_HandleDestroy(VMFUInt32_t Handle);
VMFUInt32_t VMF_GetDecodeCaps(VMFUInt32_t Handle);

/*added by junprince,2008/5/19*/
VMFUInt32_t VMF_GetVideoCaps(VMFUInt32_t Handle);

EVMF_Status VMF_FreeSurfaceID(VMFUInt32_t Handle, VMFUInt32_t uiSurfID);
EVMF_Status VMF_ResetDataOfQueue(VMFUInt32_t Handle);

#ifdef FRAMEDECTIME
void vmf_start_timer();
void vmf_end_timer();
void vmf_frame_end();
void vmf_write_log(char * str_content);
void calculate_frame_duration(VMFUInt64_t frame_pts);
#endif

#ifdef __cplusplus
}
#endif
#endif
